@@ -12,8 +12,9 @@ from ckan.lib.search import (query_for, QueryOptions, SearchIndexError, SearchEr
                              convert_legacy_parameters_to_solr)
 from ckan.plugins import PluginImplementations, IGroupController
 from ckan.lib.navl.dictization_functions import DataError
+from ckan.lib.munge import munge_name, munge_title_to_name, munge_tag
 from ckan.logic import get_action, check_access
-from ckan.logic import NotFound, NotAuthorized, ValidationError
+from ckan.logic import NotFound, NotAuthorized, ValidationError, ParameterError
 from ckan.lib.jsonp import jsonpify
 from ckan.forms.common import package_exists, group_exists
 
@@ -172,6 +173,8 @@ class ApiController(BaseController):
         except NotFound:
             return_dict['error'] = {'__type': 'Not Found Error',
                                     'message': _('Not found')}
+            if e.extra_msg:
+                return_dict['error']['message'] += ': %s' % e.extra_msg
             return_dict['success'] = False
             return self._finish(404, return_dict, content_type='json')
         except ValidationError, e:
@@ -181,6 +184,13 @@ class ApiController(BaseController):
             return_dict['success'] = False
             log.error('Validation error: %r' % str(e.error_dict))
             return self._finish(409, return_dict, content_type='json')
+        except ParameterError, e:
+            return_dict['error'] = {'__type': 'Parameter Error',
+                                    'message': '%s: %s' % \
+                                    (_('Parameter Error'), e.extra_msg)}
+            return_dict['success'] = False
+            log.error('Parameter error: %r' % e.extra_msg)
+            return self._finish(409, return_dict, content_type='json')            
         except SearchQueryError, e:
             return_dict['error'] = {'__type': 'Search Query Error',
                                     'message': 'Search Query is invalid: %r' % e.args }
@@ -208,6 +218,11 @@ class ApiController(BaseController):
             ('package', 'relationships'): get_action('package_relationships_list'),
             ('dataset', 'revisions'): get_action('package_revision_list'),
             ('package', 'revisions'): get_action('package_revision_list'),
+            ('package', 'activity'): get_action('package_activity_list'),
+            ('dataset', 'activity'): get_action('package_activity_list'),
+            ('group', 'activity'): get_action('group_activity_list'),
+            ('user', 'activity'): get_action('user_activity_list'),
+            ('activity', 'details'): get_action('activity_detail_list')
         }
 
         action = action_map.get((register, subregister)) 
@@ -265,8 +280,8 @@ class ApiController(BaseController):
     def create(self, ver=None, register=None, subregister=None, id=None, id2=None):
 
         action_map = {
-            ('dataset', 'relationships'): get_action('package_relationship_create'),
-            ('package', 'relationships'): get_action('package_relationship_create'),
+            ('dataset', 'relationships'): get_action('package_relationship_create_rest'),
+            ('package', 'relationships'): get_action('package_relationship_create_rest'),
              'group': get_action('group_create_rest'),
              'dataset': get_action('package_create_rest'),
              'package': get_action('package_create_rest'),
@@ -274,8 +289,8 @@ class ApiController(BaseController):
         }
 
         for type in model.PackageRelationship.get_all_types():
-            action_map[('dataset', type)] = get_action('package_relationship_create')
-            action_map[('package', type)] = get_action('package_relationship_create')
+            action_map[('dataset', type)] = get_action('package_relationship_create_rest')
+            action_map[('package', type)] = get_action('package_relationship_create_rest')
 
         context = {'model': model, 'session': model.Session, 'user': c.user,
                    'api_version': ver}
@@ -295,6 +310,7 @@ class ApiController(BaseController):
             return self._finish_bad_request(
                 gettext('Cannot create new entity of this type: %s %s') % \
                 (register, subregister))
+
         try:
             response_data = action(context, data_dict)
             location = None
@@ -324,17 +340,16 @@ class ApiController(BaseController):
             raise
             
     def update(self, ver=None, register=None, subregister=None, id=None, id2=None):
-
         action_map = {
-            ('dataset', 'relationships'): get_action('package_relationship_update'),
-            ('package', 'relationships'): get_action('package_relationship_update'),
+            ('dataset', 'relationships'): get_action('package_relationship_update_rest'),
+            ('package', 'relationships'): get_action('package_relationship_update_rest'),
              'dataset': get_action('package_update_rest'),
              'package': get_action('package_update_rest'),
              'group': get_action('group_update_rest'),
         }
         for type in model.PackageRelationship.get_all_types():
-            action_map[('dataset', type)] = get_action('package_relationship_update')
-            action_map[('package', type)] = get_action('package_relationship_update')
+            action_map[('dataset', type)] = get_action('package_relationship_update_rest')
+            action_map[('package', type)] = get_action('package_relationship_update_rest')
 
         context = {'model': model, 'session': model.Session, 'user': c.user,
                    'api_version': ver, 'id': id}
@@ -375,15 +390,15 @@ class ApiController(BaseController):
 
     def delete(self, ver=None, register=None, subregister=None, id=None, id2=None):
         action_map = {
-            ('dataset', 'relationships'): get_action('package_relationship_delete'),
-            ('package', 'relationships'): get_action('package_relationship_delete'),
+            ('dataset', 'relationships'): get_action('package_relationship_delete_rest'),
+            ('package', 'relationships'): get_action('package_relationship_delete_rest'),
              'group': get_action('group_delete'),
              'dataset': get_action('package_delete'),
              'package': get_action('package_delete'),
         }
         for type in model.PackageRelationship.get_all_types():
-            action_map[('dataset', type)] = get_action('package_relationship_delete')
-            action_map[('package', type)] = get_action('package_relationship_delete')
+            action_map[('dataset', type)] = get_action('package_relationship_delete_rest')
+            action_map[('package', type)] = get_action('package_relationship_delete_rest')
 
         context = {'model': model, 'session': model.Session, 'user': c.user,
                    'api_version': ver}
@@ -597,9 +612,25 @@ class ApiController(BaseController):
         if slugtype==u'group':
             response_data = dict(valid=not bool(group_exists(slug)))
             return self._finish_ok(response_data)
-        return self._finish_bad_request(gettext('Bad slug type: %s') % slugtype)
+        return self._finish_bad_request('Bad slug type: %s' % slugtype)
             
 
+    def dataset_autocomplete(self):
+        q = request.params.get('incomplete', '')
+        q_lower = q.lower()
+        limit = request.params.get('limit', 10)
+        tag_names = []
+        if q:
+            context = {'model': model, 'session': model.Session,
+                       'user': c.user or c.author}
+
+            data_dict = {'q': q, 'limit': limit}
+
+            package_dicts = get_action('package_autocomplete')(context, data_dict)
+
+        resultSet = {'ResultSet': {'Result': package_dicts}}
+        return self._finish_ok(resultSet)
+
     def tag_autocomplete(self):
         q = request.params.get('incomplete', '')
         limit = request.params.get('limit', 10)
@@ -608,9 +639,9 @@ class ApiController(BaseController):
             context = {'model': model, 'session': model.Session,
                        'user': c.user or c.author}
 
-            data_dict = {'q':q,'limit':limit}
+            data_dict = {'q': q, 'limit': limit}
 
-            tag_names = get_action('tag_autocomplete')(context,data_dict)
+            tag_names = get_action('tag_autocomplete')(context, data_dict)
 
         resultSet = {
             'ResultSet': {
@@ -635,3 +666,24 @@ class ApiController(BaseController):
             }
         }
         return self._finish_ok(resultSet)
+
+    def munge_package_name(self):
+        name = request.params.get('name')
+        munged_name = munge_name(name)
+        return self._finish_ok(munged_name)
+
+    def munge_title_to_package_name(self):
+        name = request.params.get('title') or request.params.get('name')
+        munged_name = munge_title_to_name(name)
+        return self._finish_ok(munged_name)        
+        
+    def munge_tag(self):
+        tag = request.params.get('tag') or request.params.get('name')
+        munged_tag = munge_tag(tag)
+        return self._finish_ok(munged_tag)
+
+    def status(self):
+        context = {'model': model, 'session': model.Session}
+        data_dict = {}
+        status = get_action('status_show')(context, data_dict)
+        return self._finish_ok(status)
