@@ -143,8 +143,6 @@ class AuthorizationGroupController(BaseController):
                     if usr and usr not in authorization_group.users:
                         model.add_user_to_authorization_group(usr, authorization_group, model.Role.READER)
             model.repo.commit_and_remove()
-            from nose.tools import set_trace; set_trace()
-            
             h.redirect_to(controller='authorization_group', action='read', id=c.authorization_group_name)
 
     def authz(self, id):
@@ -159,23 +157,11 @@ class AuthorizationGroupController(BaseController):
         if not c.authz_editable:
             abort(401, gettext('User %r not authorized to edit %s authorizations') % (c.user, id))
 
-        current_uors = self._get_userobjectroles(id)
-        self._handle_update_of_authz(current_uors, authorization_group)
-
-        # get the roles again as may have changed
-        user_object_roles = self._get_userobjectroles(id)
-        self._prepare_authz_info_for_render(user_object_roles)
+        roles = self._handle_update_of_authz(authorization_group)
+        self._prepare_authz_info_for_render(roles)
         return render('authorization_group/authz.html')
 
 
-    def _get_userobjectroles(self, authzgroup_name_or_id):
-        authorization_group = model.AuthorizationGroup.by_name(authzgroup_name_or_id)  or\
-               model.Session.query(model.AuthorizationGroup).get(authzgroup_name_or_id)
-        uors = model.Session.query(model.AuthorizationGroupRole).join('authorization_group').\
-               filter_by(id=authorization_group.id).all()
-        return uors
-
-
     def _render_edit_form(self, fs):
         # errors arrive in c.error and fs.errors
         c.fieldset = fs
