@@ -12,8 +12,10 @@ from ckan.logic import check_access, get_action
 from ckan.lib.i18n import set_session_locale, get_lang
 from ckan.lib.search import query_for, QueryOptions, SearchError
 from ckan.lib.base import *
-import ckan.lib.stats
 from ckan.lib.hash import get_redirect
+from ckan.lib.helpers import url_for
+
+CACHE_PARAMETER = '__cache'
 
 class HomeController(BaseController):
     repo = model.repo
@@ -60,6 +62,29 @@ class HomeController(BaseController):
             c.package_count = 0
             c.groups = []
 
+        if c.userobj is not None:
+            msg = None
+            url = url_for(controller='user', action='edit')
+            is_google_id = \
+                c.userobj.name.startswith('https://www.google.com/accounts/o8/id')
+            if not c.userobj.email and (is_google_id and not c.userobj.fullname):
+                msg = _('Please <a href="%s">update your profile</a>'
+                    ' and add your email address and your full name. ') % url + \
+                    _('%s uses your email address'
+                      ' if you need to reset your password.') \
+                      % g.site_title
+            elif not c.userobj.email:
+                msg = _('Please <a href="%s">update your profile</a>'
+                        ' and add your email address. ') % url + \
+                        _('%s uses your email address'
+                          ' if you need to reset your password.') \
+                          % g.site_title
+            elif is_google_id and not c.userobj.fullname:
+                msg = _('Please <a href="%s">update your profile</a>'
+                    ' and add your full name.') % (url)
+            if msg:
+                h.flash_notice(msg, allow_html=True)
+
         return render('home/index.html')
 
     def license(self):
@@ -75,16 +100,18 @@ class HomeController(BaseController):
                 set_session_locale(locale)
             except ValueError:
                 abort(400, _('Invalid language specified'))
-            try:
+            if locale == 'en':
+                # set_lang('en') causes an exception. Strings are in English
+                # anyway, so no need to do it.
+                h.flash_notice('Language has been set to: English')
+            else:
                 set_lang(locale)
                 # NOTE: When translating this string, substitute the word
                 # 'English' for the language being translated into.
                 # We do it this way because some Babel locales don't contain
                 # a display_name!
                 # e.g. babel.Locale.parse('no').get_display_name() returns None
-                h.flash_notice(_("Language has been set to: English"))
-            except:
-                h.flash_notice("Language has been set to: English")
+                h.flash_notice(_('Language has been set to: English'))
         else:
             abort(400, _("No language given!"))
         return_to = get_redirect()
@@ -93,7 +120,7 @@ class HomeController(BaseController):
             return 
         return_to += '&' if '?' in return_to else '?'
         # hack to prevent next page being cached
-        return_to += '__cache=%s' %  int(random.random()*100000000)
+        return_to += '%s=%s' % (CACHE_PARAMETER, int(random.random()*100000000))
         redirect_to(return_to)
 
     def cache(self, id):
