@@ -17,7 +17,6 @@ from ckan.lib.helpers import date_str_to_datetime
 from ckan.lib.base import request, c, BaseController, model, abort, h, g, render
 from ckan.lib.base import response, redirect, gettext
 from ckan.authz import Authorizer
-from ckan.lib.search import SearchIndexError, SearchError
 from ckan.lib.package_saver import PackageSaver, ValidationException
 from ckan.lib.navl.dictization_functions import DataError, unflatten, validate
 from ckan.lib.helpers import json
@@ -25,12 +24,15 @@ from ckan.logic import NotFound, NotAuthorized, ValidationError
 from ckan.logic import tuplize_dict, clean_dict, parse_params, flatten_to_string_key
 from ckan.lib.dictization import table_dictize
 from ckan.lib.i18n import get_lang
+from ckan.plugins import PluginImplementations, IDatasetForm
 import ckan.forms
 import ckan.authz
 import ckan.rating
 import ckan.misc
+import ckan.logic.action.get
+from home import CACHE_PARAMETER
 
-log = logging.getLogger('ckan.controllers')
+log = logging.getLogger(__name__)
 
 def search_url(params):
     url = h.url_for(controller='package', action='search')
@@ -47,26 +49,106 @@ autoneg_cfg = [
     ("text", "x-graphviz", ["dot"]),
     ]
 
-class PackageController(BaseController):
+##############  Methods and variables related to the pluggable  ##############
+##############       behaviour of the package controller        ############## 
 
-    ## hooks for subclasses 
-    package_form = 'package/new_package_form.html'
+# Mapping from package-type strings to IDatasetForm instances
+_controller_behaviour_for = dict()
 
-    def _form_to_db_schema(self):
+# The fallback behaviour
+_default_controller_behaviour = None
+
+def register_pluggable_behaviour(map):
+    """
+    Register the various IDatasetForm instances.
+
+    This method will setup the mappings between package types and the registered
+    IDatasetForm instances.  If it's called more than once an
+    exception will be raised.
+    """
+    global _default_controller_behaviour
+    
+    # Check this method hasn't been invoked already.
+    # TODO: This method seems to be being invoked more than once during running of
+    #       the tests.  So I've disbabled this check until I figure out why.
+    #if _default_controller_behaviour is not None:
+        #raise ValueError, "Pluggable package controller behaviour is already defined "\
+                          #"'%s'" % _default_controller_behaviour
+
+    # Create the mappings and register the fallback behaviour if one is found.
+    for plugin in PluginImplementations(IDatasetForm):
+        if plugin.is_fallback():
+            if _default_controller_behaviour is not None:
+                raise ValueError, "More than one fallback "\
+                                  "IDatasetForm has been registered"
+            _default_controller_behaviour = plugin
+
+        for package_type in plugin.package_types():
+            # Create a connection between the newly named type and the package controller
+            # but first we need to make sure we are not clobbering an existing domain
+            map.connect('/%s/new' % (package_type,), controller='package', action='new')    
+            map.connect('%s_read' % (package_type,), '/%s/{id}' %  (package_type,), controller='package', action='read')                        
+            map.connect('%s_action' % (package_type,),
+                        '/%s/{action}/{id}' % (package_type,), controller='package',
+                requirements=dict(action='|'.join(['edit', 'authz', 'history' ]))
+            )            
+                    
+            if package_type in _controller_behaviour_for:
+                raise ValueError, "An existing IDatasetForm is "\
+                                  "already associated with the package type "\
+                                  "'%s'" % package_type
+            _controller_behaviour_for[package_type] = plugin
+
+    # Setup the fallback behaviour if one hasn't been defined.
+    if _default_controller_behaviour is None:
+        _default_controller_behaviour = DefaultDatasetForm()
+
+def _lookup_plugin(package_type):
+    """
+    Returns the plugin controller associoated with the given package type.
+
+    If the package type is None or cannot be found in the mapping, then the
+    fallback behaviour is used.
+    """
+    if package_type is None:
+        return _default_controller_behaviour
+    return _controller_behaviour_for.get(package_type,
+                                         _default_controller_behaviour)
+
+class DefaultDatasetForm(object):
+    """
+    Provides a default implementation of the pluggable package controller behaviour.
+
+    This class has 2 purposes:
+
+     - it provides a base class for IDatasetForm implementations
+       to use if only a subset of the 5 method hooks need to be customised.
+
+     - it provides the fallback behaviour if no plugin is setup to provide
+       the fallback behaviour.
+
+    Note - this isn't a plugin implementation.  This is deliberate, as
+           we don't want this being registered.
+    """
+
+    def package_form(self):
+        return 'package/new_package_form.html'
+
+    def form_to_db_schema(self):
         return package_form_schema()
 
-    def _db_to_form_schema(self):
+    def db_to_form_schema(self):
         '''This is an interface to manipulate data from the database
         into a format suitable for the form (optional)'''
 
-    def _check_data_dict(self, data_dict):
+    def check_data_dict(self, data_dict):
         '''Check if the return data is correct, mostly for checking out if
         spammers are submitting only part of the form'''
 
         # Resources might not exist yet (eg. Add Dataset)
         surplus_keys_schema = ['__extras', '__junk', 'state', 'groups',
                                'extras_validation', 'save', 'return_to',
-                               'resources']
+                               'resources', 'type']
 
         schema_keys = package_form_schema().keys()
         keys_in_schema = set(schema_keys) - set(surplus_keys_schema)
@@ -76,10 +158,10 @@ class PackageController(BaseController):
         if missing_keys:
             #print data_dict
             #print missing_keys
-            log.info('incorrect form fields posted')
+            log.info('incorrect form fields posted, missing %s' % missing_keys )
             raise DataError(data_dict)
 
-    def _setup_template_variables(self, context, data_dict):
+    def setup_template_variables(self, context, data_dict):
         c.groups_authz = get_action('group_list_authz')(context, data_dict)
         data_dict.update({'available_only':True})
         c.groups_available = get_action('group_list_authz')(context, data_dict)
@@ -98,11 +180,33 @@ class PackageController(BaseController):
             except NotAuthorized:
                 c.auth_for_change_state = False
 
-    ## end hooks
+##############      End of pluggable package behaviour stuff    ############## 
+
+class PackageController(BaseController):
+
+    def _package_form(self, package_type=None):    
+        return _lookup_plugin(package_type).package_form()
+
+    def _form_to_db_schema(self, package_type=None):
+        return _lookup_plugin(package_type).form_to_db_schema()
+
+    def _db_to_form_schema(self, package_type=None):
+        '''This is an interface to manipulate data from the database
+        into a format suitable for the form (optional)'''
+        return _lookup_plugin(package_type).db_to_form_schema()
+
+    def _check_data_dict(self, data_dict, package_type=None):
+        '''Check if the return data is correct, mostly for checking out if
+        spammers are submitting only part of the form'''
+        return _lookup_plugin(package_type).check_data_dict(data_dict)
+
+    def _setup_template_variables(self, context, data_dict, package_type=None):
+        return _lookup_plugin(package_type).setup_template_variables(context, data_dict)
 
     authorizer = ckan.authz.Authorizer()
 
     def search(self):
+        from ckan.lib.search import SearchError
         try:
             context = {'model':model,'user': c.user or c.author}
             check_access('site_read',context)
@@ -141,11 +245,15 @@ class PackageController(BaseController):
 
         try:
             c.fields = []
+            search_extras = {}
             for (param, value) in request.params.items():
                 if not param in ['q', 'page'] \
                         and len(value) and not param.startswith('_'):
-                    c.fields.append((param, value))
-                    q += ' %s: "%s"' % (param, value)
+                    if not param.startswith('ext_'):
+                        c.fields.append((param, value))
+                        q += ' %s: "%s"' % (param, value)
+                    else:
+                        search_extras[param] = value
 
             context = {'model': model, 'session': model.Session,
                        'user': c.user or c.author}
@@ -155,6 +263,7 @@ class PackageController(BaseController):
                 'facet.field':g.facets,
                 'rows':limit,
                 'start':(page-1)*limit,
+                'extras':search_extras
             }
 
             query = get_action('package_search')(context,data_dict)
@@ -169,6 +278,7 @@ class PackageController(BaseController):
             c.facets = query['facets']
             c.page.items = query['results']
         except SearchError, se:
+            log.error('Dataset search error: %r', se.args)
             c.query_error = True
             c.facets = {}
             c.page = h.Page(collection=[])
@@ -177,9 +287,10 @@ class PackageController(BaseController):
 
 
     def read(self, id):
+        package_type = self._get_package_type(id.split('@')[0])
         context = {'model': model, 'session': model.Session,
                    'user': c.user or c.author, 'extras_as_string': True,
-                   'schema': self._form_to_db_schema()}
+                   'schema': self._form_to_db_schema(package_type=package_type)}
         data_dict = {'id': id}
 
         # interpret @<revision_id> or @<date> suffix
@@ -205,7 +316,7 @@ class PackageController(BaseController):
             c.pkg = context['package']
             c.pkg_json = json.dumps(c.pkg_dict)
         except NotFound:
-            abort(404, _('Package not found'))
+            abort(404, _('Dataset not found'))
         except NotAuthorized:
             abort(401, _('Unauthorized to read package %s') % id)
         
@@ -215,7 +326,14 @@ class PackageController(BaseController):
 
         # used by disqus plugin
         c.current_package_id = c.pkg.id
-        
+
+        # Add the package's activity stream (already rendered to HTML) to the
+        # template context for the package/read.html template to retrieve
+        # later.
+        c.package_activity_stream = \
+                ckan.logic.action.get.package_activity_list_html(context,
+                    {'id': c.current_package_id})
+
         if config.get('rdf_packages'):
             accept_header = request.headers.get('Accept', '*/*')
             for content_type, exts in negotiate(autoneg_cfg, accept_header):
@@ -228,16 +346,17 @@ class PackageController(BaseController):
         return render('package/read.html')
 
     def comments(self, id):
+        package_type = self._get_package_type(id)
         context = {'model': model, 'session': model.Session,
                    'user': c.user or c.author, 'extras_as_string': True,
-                   'schema': self._form_to_db_schema()}
+                   'schema': self._form_to_db_schema(package_type=package_type)}
 
         #check if package exists
         try:
             c.pkg_dict = get_action('package_show')(context, {'id':id})
             c.pkg = context['package']
         except NotFound:
-            abort(404, _('Package not found'))
+            abort(404, _('Dataset not found'))
         except NotAuthorized:
             abort(401, _('Unauthorized to read package %s') % id)
 
@@ -278,16 +397,16 @@ class PackageController(BaseController):
         except NotAuthorized:
             abort(401, _('Unauthorized to read package %s') % '')
         except NotFound:
-            abort(404, _('Package not found'))
+            abort(404, _('Dataset not found'))
 
         format = request.params.get('format', '')
         if format == 'atom':
             # Generate and return Atom 1.0 document.
             from webhelpers.feedgenerator import Atom1Feed
             feed = Atom1Feed(
-                title=_(u'CKAN Package Revision History'),
+                title=_(u'CKAN Dataset Revision History'),
                 link=h.url_for(controller='revision', action='read', id=c.pkg_dict['name']),
-                description=_(u'Recent changes to CKAN Package: ') + (c.pkg_dict['title'] or ''),
+                description=_(u'Recent changes to CKAN Dataset: ') + (c.pkg_dict['title'] or ''),
                 language=unicode(get_lang()),
             )
             for revision_dict in c.pkg_revisions:
@@ -320,10 +439,15 @@ class PackageController(BaseController):
         return render('package/history.html')
 
     def new(self, data=None, errors=None, error_summary=None):
+        
+        package_type = request.path.strip('/').split('/')[0]
+        if package_type == 'group':
+            package_type = None
+        
         context = {'model': model, 'session': model.Session,
                    'user': c.user or c.author, 'extras_as_string': True,
                    'save': 'save' in request.params,
-                   'schema': self._form_to_db_schema()}
+                   'schema': self._form_to_db_schema(package_type=package_type)}
 
         try:
             check_access('package_create',context)
@@ -333,37 +457,47 @@ class PackageController(BaseController):
         if context['save'] and not data:
             return self._save_new(context)
 
-        data = data or dict(request.params) 
+        data = data or clean_dict(unflatten(tuplize_dict(parse_params(
+            request.params, ignore_keys=[CACHE_PARAMETER]))))
+
         errors = errors or {}
         error_summary = error_summary or {}
         vars = {'data': data, 'errors': errors, 'error_summary': error_summary}
 
         self._setup_template_variables(context, {'id': id})
-        c.form = render(self.package_form, extra_vars=vars)
 
+        # TODO: This check is to maintain backwards compatibility with the old way of creating
+        # custom forms. This behaviour is now deprecated.
+        if hasattr(self, 'package_form'):
+            c.form = render(self.package_form, extra_vars=vars)
+        else:
+            c.form = render(self._package_form(package_type=package_type), extra_vars=vars)
         return render('package/new.html')
 
 
     def edit(self, id, data=None, errors=None, error_summary=None):
+        package_type = self._get_package_type(id)
         context = {'model': model, 'session': model.Session,
                    'user': c.user or c.author, 'extras_as_string': True,
                    'save': 'save' in request.params,
                    'moderated': config.get('moderated'),
                    'pending': True,
-                   'schema': self._form_to_db_schema()}
+                   'schema': self._form_to_db_schema(package_type=package_type)}
 
         if context['save'] and not data:
             return self._save_edit(id, context)
         try:
             old_data = get_action('package_show')(context, {'id':id})
-            schema = self._db_to_form_schema()
+            schema = self._db_to_form_schema(package_type=package_type)
             if schema and not data:
                 old_data, errors = validate(old_data, schema, context=context)
             data = data or old_data
+            # Merge all elements for the complete package dictionary
+            c.pkg_dict = dict(old_data.items() + data.items())
         except NotAuthorized:
             abort(401, _('Unauthorized to read package %s') % '')
         except NotFound:
-            abort(404, _('Package not found'))
+            abort(404, _('Dataset not found'))
 
         c.pkg = context.get("package")
         c.pkg_json = json.dumps(data)
@@ -376,30 +510,36 @@ class PackageController(BaseController):
         errors = errors or {}
         vars = {'data': data, 'errors': errors, 'error_summary': error_summary}
 
-        self._setup_template_variables(context, {'id': id})
+        self._setup_template_variables(context, {'id': id}, package_type=package_type)
 
-        c.form = render(self.package_form, extra_vars=vars)
+        # TODO: This check is to maintain backwards compatibility with the old way of creating
+        # custom forms. This behaviour is now deprecated.
+        if hasattr(self, 'package_form'):
+            c.form = render(self.package_form, extra_vars=vars)
+        else:
+            c.form = render(self._package_form(package_type=package_type), extra_vars=vars)
         return render('package/edit.html')
 
     def read_ajax(self, id, revision=None):
+        package_type=self._get_package_type(id)
         context = {'model': model, 'session': model.Session,
                    'user': c.user or c.author,
                    'extras_as_string': True,
-                   'schema': self._form_to_db_schema(),
+                   'schema': self._form_to_db_schema(package_type=package_type),
                    'revision_id': revision}
 
         try:
             data = get_action('package_show')(context, {'id': id})
-            schema = self._db_to_form_schema()
+            schema = self._db_to_form_schema(package_type=package_type)
             if schema:
                 data, errors = validate(data, schema)
         except NotAuthorized:
             abort(401, _('Unauthorized to read package %s') % '')
         except NotFound:
-            abort(404, _('Package not found'))
+            abort(404, _('Dataset not found'))
 
         ## hack as db_to_form schema should have this
-        data['tag_string'] = ' '.join([tag['name'] for tag in data.get('tags', [])])
+        data['tag_string'] = ', '.join([tag['name'] for tag in data.get('tags', [])])
         data.pop('tags')
         data = flatten_to_string_key(data)
         response.headers['Content-Type'] = 'application/json;charset=utf-8'
@@ -416,7 +556,7 @@ class PackageController(BaseController):
         except NotAuthorized:
             abort(401, _('Unauthorized to read package %s') % '')
         except NotFound:
-            abort(404, _('Package not found'))
+            abort(404, _('Dataset not found'))
 
 
         data = []
@@ -437,10 +577,37 @@ class PackageController(BaseController):
         response.headers['Content-Type'] = 'application/json;charset=utf-8'
         return json.dumps(data)
 
-    def _save_new(self, context):
+    def _get_package_type(self, id):
+        """
+        Given the id of a package it determines the plugin to load 
+        based on the package's type name (type). The plugin found
+        will be returned, or None if there is no plugin associated with 
+        the type.
+
+        Uses a minimal context to do so.  The main use of this method
+        is for figuring out which plugin to delegate to.
+
+        aborts if an exception is raised.
+        """
+        global _controller_behaviour_for
+
+        context = {'model': model, 'session': model.Session,
+                   'user': c.user or c.author}
+        try:
+            data = get_action('package_show')(context, {'id': id})
+        except NotFound:
+            abort(404, _('Dataset not found'))
+        except NotAuthorized:
+            abort(401, _('Unauthorized to read package %s') % id)
+
+        return data['type']
+
+    def _save_new(self, context, package_type=None):
+        from ckan.lib.search import SearchIndexError
         try:
             data_dict = clean_dict(unflatten(
                 tuplize_dict(parse_params(request.POST))))
+            data_dict['type'] = package_type
             self._check_data_dict(data_dict)
             context['message'] = data_dict.get('log_message', '')
             pkg = get_action('package_create')(context, data_dict)
@@ -449,7 +616,7 @@ class PackageController(BaseController):
         except NotAuthorized:
             abort(401, _('Unauthorized to read package %s') % '')
         except NotFound, e:
-            abort(404, _('Package not found'))
+            abort(404, _('Dataset not found'))
         except DataError:
             abort(400, _(u'Integrity Error'))
         except SearchIndexError, e:
@@ -460,10 +627,12 @@ class PackageController(BaseController):
             return self.new(data_dict, errors, error_summary)
 
     def _save_edit(self, id, context):
+        from ckan.lib.search import SearchIndexError
         try:
+            package_type = self._get_package_type(id)
             data_dict = clean_dict(unflatten(
                 tuplize_dict(parse_params(request.POST))))
-            self._check_data_dict(data_dict)
+            self._check_data_dict(data_dict, package_type=package_type)
             context['message'] = data_dict.get('log_message', '')
             if not context['moderated']:
                 context['pending'] = False
@@ -478,7 +647,7 @@ class PackageController(BaseController):
         except NotAuthorized:
             abort(401, _('Unauthorized to read package %s') % id)
         except NotFound, e:
-            abort(404, _('Package not found'))
+            abort(404, _('Dataset not found'))
         except DataError:
             abort(400, _(u'Integrity Error'))
         except SearchIndexError, e:
@@ -524,7 +693,7 @@ class PackageController(BaseController):
     def authz(self, id):
         pkg = model.Package.get(id)
         if pkg is None:
-            abort(404, gettext('Package not found'))
+            abort(404, gettext('Dataset not found'))
         c.pkg = pkg # needed to add in the tab bar to the top of the auth page
         c.pkgname = pkg.name
         c.pkgtitle = pkg.title
@@ -532,27 +701,18 @@ class PackageController(BaseController):
             context = {'model':model,'user':c.user or c.author, 'package':pkg}
             check_access('package_edit_permissions',context)
             c.authz_editable = True
+            c.pkg_dict = get_action('package_show')(context, {'id': id})
         except NotAuthorized:
             c.authz_editable = False
         if not c.authz_editable:
             abort(401, gettext('User %r not authorized to edit %s authorizations') % (c.user, id))
 
-        current_uors = self._get_userobjectroles(id)
-        self._handle_update_of_authz(current_uors, pkg)
-
-        # get the roles again as may have changed
-        user_object_roles = self._get_userobjectroles(id)
-        self._prepare_authz_info_for_render(user_object_roles)
+        roles = self._handle_update_of_authz(pkg)
+        self._prepare_authz_info_for_render(roles)
         return render('package/authz.html')
 
-
-    def _get_userobjectroles(self, pkg_id):
-        pkg = model.Package.get(pkg_id)
-        uors = model.Session.query(model.PackageRole).join('package').filter_by(name=pkg.name).all()
-        return uors
-
-
     def autocomplete(self):
+        # DEPRECATED in favour of /api/2/util/dataset/autocomplete
         q = unicode(request.params.get('q', ''))
         if not len(q): 
             return ''
@@ -566,10 +726,7 @@ class PackageController(BaseController):
 
         pkg_list = []
         for pkg in packages:
-            if pkg['name'].lower().startswith(q.lower()):
-                pkg_list.append('%s|%s' % (pkg['name'], pkg['name']))
-            else:
-                pkg_list.append('%s (%s)|%s' % (pkg['title'].replace('|', ' '), pkg['name'], pkg['name']))
+            pkg_list.append('%s|%s' % (pkg['match_displayed'].replace('|', ' '), pkg['name']))
         return '\n'.join(pkg_list)
 
     def _render_edit_form(self, fs, params={}, clear_session=False):
@@ -623,3 +780,28 @@ class PackageController(BaseController):
                 return name
             else:
                 return reference + " unknown"
+
+    def resource_read(self, id, resource_id):
+        context = {'model': model, 'session': model.Session,
+                   'user': c.user or c.author}
+
+        try:
+            c.resource = get_action('resource_show')(context, {'id': resource_id})
+            c.package = get_action('package_show')(context, {'id': id})
+            # required for nav menu
+            c.pkg = context['package']
+            c.resource_json = json.dumps(c.resource)
+            c.pkg_dict = c.package
+        except NotFound:
+            abort(404, _('Resource not found'))
+        except NotAuthorized:
+            abort(401, _('Unauthorized to read resource %s') % id)
+        # get package license info
+        license_id = c.package.get('license_id')
+        try:
+            c.package['isopen'] = model.Package.get_license_register()[license_id].isopen()
+        except KeyError:
+            c.package['isopen'] = False
+
+        return render('package/resource_read.html')
+
