@@ -12,13 +12,11 @@ from ckan.logic import NotFound, NotAuthorized, ValidationError
 from ckan.logic import check_access, get_action
 from ckan.logic import tuplize_dict, clean_dict, parse_params
 from ckan.logic.schema import user_new_form_schema, user_edit_form_schema 
+from ckan.logic.action.get import user_activity_list_html
 from ckan.lib.captcha import check_recaptcha, CaptchaError
 
 log = logging.getLogger(__name__)
 
-def login_form():
-    return render('user/login_form.html').replace('FORM_ACTION', '%s')
-
 class UserController(BaseController):
 
     def __before__(self, action, **env):
@@ -83,13 +81,10 @@ class UserController(BaseController):
         return render('user/list.html')
 
     def read(self, id=None):
-
         context = {'model': model,
                    'user': c.user or c.author}
-
         data_dict = {'id':id,
                      'user_obj':c.userobj}
-
         try:
             check_access('user_show',context, data_dict)
         except NotAuthorized:
@@ -103,9 +98,10 @@ class UserController(BaseController):
         c.user_dict = user_dict
         c.is_myself = user_dict['name'] == c.user
         c.about_formatted = self._format_about(user_dict['about'])
-
+        c.user_activity_stream = user_activity_list_html(context,
+            {'id':c.user_dict['id']})
         return render('user/read.html')
-    
+
     def me(self):
         if not c.user:
             h.redirect_to(controller='user', action='login', id=None)
@@ -131,6 +127,10 @@ class UserController(BaseController):
 
         if context['save'] and not data:
             return self._save_new(context)
+
+        if c.user and not data:
+            # #1799 Don't offer the registration form if already logged in
+            return render('user/logout_first.html')            
         
         data = data or {}
         errors = errors or {}
@@ -162,11 +162,16 @@ class UserController(BaseController):
             errors = e.error_dict
             error_summary = e.error_summary
             return self.new(data_dict, errors, error_summary)
-        # Redirect to a URL picked up by repoze.who which performs the login
-        h.redirect_to('/login_generic?login=%s&password=%s' % (
-            str(data_dict['name']),
-            quote(data_dict['password1'].encode('utf-8'))))
-
+        if not c.user:
+            # Redirect to a URL picked up by repoze.who which performs the login
+            h.redirect_to('/login_generic?login=%s&password=%s' % (
+                str(data_dict['name']),
+                quote(data_dict['password1'].encode('utf-8'))))
+        else:
+            # #1799 User has managed to register whilst logged in - warn user
+            # they are not re-logged in as new user.
+            h.flash_success(_('User "%s" is now registered but you are still logged in as "%s" from before') % (data_dict['name'], c.user))
+            return render('user/logout_first.html')
 
     def edit(self, id=None, data=None, errors=None, error_summary=None):
         context = {'model': model, 'session': model.Session,
@@ -240,7 +245,16 @@ class UserController(BaseController):
     def login(self):
         if 'error' in request.params:
             h.flash_error(request.params['error'])
-        return render('user/login.html')
+
+        if request.environ['SCRIPT_NAME'] and g.openid_enabled:
+            # #1662 restriction
+            log.warn('Cannot mount CKAN at a URL and login with OpenID.')
+            g.openid_enabled = False
+
+        if not c.user:
+            return render('user/login.html')
+        else:
+            return render('user/logout_first.html')
     
     def logged_in(self):
         if c.user:
@@ -308,7 +322,7 @@ class UserController(BaseController):
                 try:
                     mailer.send_reset_link(user_obj)
                     h.flash_success(_('Please check your inbox for a reset code.'))
-                    redirect('/')
+                    h.redirect_to('/')
                 except mailer.MailerException, e:
                     h.flash_error(_('Could not send reset link: %s') % unicode(e))
         return render('user/request_reset.html')
@@ -339,7 +353,7 @@ class UserController(BaseController):
                 user = get_action('user_update')(context, user_dict)
 
                 h.flash_success(_("Your password has been reset."))
-                redirect('/')
+                h.redirect_to('/')
             except NotAuthorized:
                 h.flash_error(_('Unauthorized to edit user %s') % id)
             except NotFound, e:
@@ -358,7 +372,7 @@ class UserController(BaseController):
             html = genshi.HTML(about_formatted)
         except genshi.ParseError, e:
             log.error('Could not print "about" field Field: %r Error: %r', about, e)
-            html = 'Error: Could not parse About text'
+            html = _('Error: Could not parse About text')
         return html
     
     def _get_form_password(self):
