@@ -14,6 +14,7 @@ import ckan.model as model
 import ckan.lib.helpers as h
 import ckan.lib.field_types as field_types
 import ckan.misc
+import ckan.lib.dictization.model_save as model_save
 
 log = logging.getLogger(__name__)
 
@@ -481,7 +482,7 @@ class TagField(ConfiguredField):
 
     class TagEditRenderer(formalchemy.fields.FieldRenderer):
         def render(self, **kwargs):
-            kwargs['value'] = ' '.join(self.value)
+            kwargs['value'] = ', '.join(self.value)
             kwargs['size'] = 60
             api_url = config.get('ckan.api_url', '/').rstrip('/')
             tagcomplete_url = api_url+h.url_for(controller='api',
@@ -495,7 +496,7 @@ class TagField(ConfiguredField):
         def _tag_links(self):
             tags = self.value
             tag_links = [h.link_to(tagname, h.url_for(controller='tag', action='read', id=tagname)) for tagname in tags]
-            return literal(' '.join(tag_links))
+            return literal(', '.join(tag_links))
 
         def render_readonly(self, **kwargs):
             tag_links = self._tag_links()
@@ -504,21 +505,29 @@ class TagField(ConfiguredField):
         def _serialized_value(self):
             # despite being a collection, there is only one field to get
             # the values from
-            tags_as_string = self.params.getone(self.name)
-            tags = tags_as_string.replace(',', ' ').lower().split()
+            tags_as_string = self.params.getone(self.name).strip()
+            if tags_as_string == "":
+                return []
+            tags = map(lambda s: s.strip(), tags_as_string.split(','))
             return tags
             
-    tagname_match = re.compile('[\w\-_.]*$', re.UNICODE)
-    tagname_uppercase = re.compile('[A-Z]')
+    tagname_match = re.compile('[^"]*$') # already split on commas
     def tag_name_validator(self, val, field):
         for tag in val:
+        
+            # formalchemy deserializes an empty string into None.
+            # This happens if the tagstring gets split on commas, and
+            # there's an empty string in the resulting list.
+            # e.g. "tag1,,tag2" ; "  ,tag1" and "tag1," will all result
+            # in an empty tag name.
+            if tag is None:
+                tag = u'' # let the minimum length validator take care of it.
+
             min_length = 2
             if len(tag) < min_length:
                 raise formalchemy.ValidationError(_('Tag "%s" length is less than minimum %s') % (tag, min_length))
             if not self.tagname_match.match(tag):
-                raise formalchemy.ValidationError(_('Tag "%s" must be alphanumeric characters or symbols: -_.') % (tag))
-            if self.tagname_uppercase.search(tag):
-                raise formalchemy.ValidationError(_('Tag "%s" must not be uppercase' % (tag)))
+                raise formalchemy.ValidationError(_('Tag "%s" must not contain any quotation marks: "') % (tag))
 
 class ExtrasField(ConfiguredField):
     '''A form field for arbitrary "extras" dataset data.'''
@@ -693,28 +702,13 @@ class GroupSelectField(ConfiguredField):
 
         def _update_groups(self):
             new_group_ids = self._deserialize() or []
-            
-            # Get groups which have alread been associated.
-            old_groups = self.parent.model.groups
 
-            # Calculate which to append and which to remove.
-            editable_set = set([g.id for g in self.user_editable_groups])
-            old_group_ids = [g.id for g in old_groups]
-            new_set = set(new_group_ids)
-            old_set = set(old_group_ids)
-            append_set = (new_set - old_set).intersection(editable_set)
-            remove_set = (old_set - new_set).intersection(editable_set)
-            
-            # Create dataset group associations.
-            for id in append_set:
-                group = model.Session.query(model.Group).autoflush(False).get(id)
-                if group:
-                    self.parent.model.groups.append(group)
+            group_dicts = [dict(id = group_id) for 
+                           group_id in new_group_ids]
 
-            # Delete dataset group associations.
-            for group in self.parent.model.groups:
-                if group.id in remove_set:
-                    self.parent.model.groups.remove(group)
+            context = {'model': model, 'session': model.Session}
+            model_save.package_membership_list_save(
+                group_dicts, self.parent.model, context)
             
         def requires_label(self):
             return False
@@ -722,7 +716,7 @@ class GroupSelectField(ConfiguredField):
 
     class GroupSelectEditRenderer(formalchemy.fields.FieldRenderer):
         def _get_value(self, **kwargs):
-            return self.field.parent.model.groups
+            return self.field.parent.model.get_groups()
 
         def _get_user_editable_groups(self):
             return self.field.user_editable_groups
