@@ -18,11 +18,12 @@ class Globals(object):
         """
         self.site_title = config.get('ckan.site_title', '')
         self.favicon = config.get('ckan.favicon',
-                                  'http://assets.okfn.org/p/ckan/img/ckan.ico')
+                                  '/images/icons/ckan.ico')
         self.site_logo = config.get('ckan.site_logo', '')
         self.site_url = config.get('ckan.site_url', '')
         self.site_url_nice = self.site_url.replace('http://','').replace('www.','')
         self.site_description = config.get('ckan.site_description', '')
+        self.site_about = config.get('ckan.site_about', '')
         
         self.facets = config.get('search.facets', 'groups tags res_format license').split()
         
