@@ -87,6 +87,18 @@ class BaseController(WSGIController):
         i18n.handle_request(request, c)
 
     def _identify_user(self):
+        '''
+        Identifies the user using two methods:
+        a) If he has logged into the web interface then repoze.who will
+           set REMOTE_USER.
+        b) For API calls he may set a header with his API key.
+        If the user is identified then:
+          c.user = user name (unicode)
+          c.author = user name
+        otherwise:
+          c.user = None
+          c.author = user\'s IP address (unicode)
+        '''
         # see if it was proxied first
         c.remote_addr = request.environ.get('HTTP_X_FORWARDED_FOR', '')
         if not c.remote_addr:
@@ -98,11 +110,11 @@ class BaseController(WSGIController):
             c.user = c.user.decode('utf8')
             c.userobj = model.User.by_name(c.user)
             if c.userobj is None:
-                # This occurs when you are logged in with openid, clean db
+                # This occurs when you are logged in, clean db
                 # and then restart i.e. only really for testers. There is no
                 # user object, so even though repoze thinks you are logged in
                 # and your cookie has ckan_display_name, we need to force user
-                # to login again to get the User object.
+                # to logout and login again to get the User object.
                 c.user = None
         else:
             c.userobj = self._get_user_for_apikey()
@@ -279,20 +291,93 @@ class BaseController(WSGIController):
         )
         return fieldset
 
-    def _handle_update_of_authz(self, current_uors, domain_object):
-        # In the event of a post request, work out which of the four possible actions
-        # is to be done, and do it before displaying the page
-        if 'add' in request.POST:
-            self._add_user_object_role('users', current_uors, domain_object)
+    def _handle_update_of_authz(self, domain_object):
+        '''In the event of a post request to a domain object\'s authz form,
+        work out which of the four possible actions is to be done,
+        and do it before displaying the page.
 
-        if 'authz_add' in request.POST:
-            self._add_user_object_role('authz_groups', current_uors, domain_object)
+        Returns the updated roles for the domain_object.
+        '''
+        from ckan.logic import NotFound, get_action
 
+        context = {'model': model, 'session': model.Session,
+                   'user': c.user or c.author}
+        data_dict = {'domain_object': domain_object.id}
+
+        # Work out actions needed, depending on which button was pressed
         if 'save' in request.POST:
-            self._update_user_object_roles('users', current_uors, domain_object)
+            user_or_authgroup = 'user'
+            update_or_add = 'update'
+        elif 'add' in request.POST:
+            user_or_authgroup = 'user'
+            update_or_add = 'add'
+        elif 'authz_save' in request.POST:
+            user_or_authgroup = 'authorization_group'
+            update_or_add = 'update'
+        elif 'authz_add' in request.POST:
+            user_or_authgroup = 'authorization_group'
+            update_or_add = 'add'
+        else:
+            user_or_authgroup = None
+            update_or_add = None            
 
-        if 'authz_save' in request.POST:
-            self._update_user_object_roles('authz_groups', current_uors, domain_object)
+        # Work out what role checkboxes are checked or unchecked
+        checked_roles = [ box_id for (box_id, value) in request.params.items() \
+                          if (value == u'on')]
+        unchecked_roles = [ box_id for (box_id, value) in request.params.items() \
+                          if (value == u'submitted')]
+
+        action = None
+        if update_or_add is 'update':
+            # Get user_roles by decoding the checkbox grid - user$role strings
+            user_roles = {}
+            for checked_role in checked_roles:
+                user_or_authgroup_id, role = checked_role.split('$')
+                if user_or_authgroup_id not in user_roles:
+                    user_roles[user_or_authgroup_id] = []
+                user_roles[user_or_authgroup_id].append(role)
+            # Users without roles need adding to the user_roles too to make
+            # their roles be deleted
+            for unchecked_role in unchecked_roles:
+                user_or_authgroup_id, role = unchecked_role.split('$')
+                if user_or_authgroup_id not in user_roles:
+                    user_roles[user_or_authgroup_id] = []
+            # Convert user_roles to role dictionaries
+            role_dicts = []
+            for user, roles in user_roles.items():
+                role_dicts.append({user_or_authgroup: user, 'roles': roles})
+            data_dict['user_roles'] = role_dicts
+
+            action = 'user_role_bulk_update'
+            success_message = _('Updated')
+        elif update_or_add is 'add':
+            # Roles for this new user is a simple list from the checkbox row
+            data_dict['roles'] = checked_roles
+
+            # User (or "user group" aka AuthorizationGroup) comes from
+            # the input box.
+            new_user = request.params.get('new_user_name')
+            if new_user:
+                data_dict[user_or_authgroup] = new_user
+
+                action = 'user_role_update'
+                success_message = _('User role(s) added')
+            else:
+                h.flash_error(_('Please supply a user name'))
+
+        if action:
+            try:
+                roles = get_action(action)(context, data_dict)
+            except NotFound, e:
+                h.flash_error(_('Not found') + (': %s' % e if str(e) else ''))
+            else:
+                h.flash_success(success_message)
+
+        # Return roles for all users on this domain object
+        if update_or_add is 'add':
+            if user_or_authgroup in data_dict:
+                del data_dict[user_or_authgroup]
+        return get_action('roles_show')(context, data_dict)
 
     def _prepare_authz_info_for_render(self, user_object_roles):
         # =================
@@ -302,13 +387,16 @@ class BaseController(WSGIController):
         # associated with any object, so that's easy:
         possible_roles = model.Role.get_all()
 
-
         # uniquify and sort
-        users = sorted(list(set([uor.user.name for uor in user_object_roles if uor.user])))
-        authz_groups = sorted(list(set([uor.authorized_group.name for uor in user_object_roles if uor.authorized_group])))
+        users = sorted(list(set([uor['user_id'] for uor in user_object_roles['roles'] if uor['user_id']])))
+        authz_groups = sorted(list(set([uor['authorized_group_id'] \
+                                        for uor in user_object_roles['roles'] \
+                                        if uor['authorized_group_id']])))
 
         # make a dictionary from (user, role) to True, False
-        users_roles = [( uor.user.name, uor.role) for uor in user_object_roles if uor.user]
+        users_roles = [( uor['user_id'], uor['role']) \
+                       for uor in user_object_roles['roles'] \
+                       if uor['user_id']]
         user_role_dict={}
         for u in users:
             for r in possible_roles:
@@ -318,7 +406,9 @@ class BaseController(WSGIController):
                     user_role_dict[(u,r)]=False
 
         # and similarly make a dictionary from (authz_group, role) to True, False
-        authz_groups_roles = [( uor.authorized_group.name, uor.role) for uor in user_object_roles if uor.authorized_group]
+        authz_groups_roles = [( uor['authorized_group_id'], uor['role']) \
+                              for uor in user_object_roles['roles']
+                              if uor['authorized_group_id']]
         authz_groups_role_dict={}
         for u in authz_groups:
             for r in possible_roles:
@@ -333,141 +423,6 @@ class BaseController(WSGIController):
         c.authz_groups = authz_groups
         c.authz_groups_role_dict = authz_groups_role_dict
 
-    def _update_user_object_roles(self, users_or_authz_groups, current_uors, domain_object):
-        '''Update user object roles for this object.
-
-        :param domain_object: the domain object for whom we are adding the user
-        object role.
-        '''
-        # The permissions grid has been saved
-        # which is a grid of checkboxes named user$role
-        rpi = request.params.items()
-
-        # The grid passes us a list of the users/roles that were displayed
-        submitted = [ a for (a,b) in rpi if (b == u'submitted')]
-        # and also those which were checked
-        checked = [ a for (a,b) in rpi if (b == u'on')]
-
-        # from which we can deduce true/false for each user/role combination
-        # that was displayed in the form
-        table_dict={}
-        for a in submitted:
-            table_dict[a]=False
-        for a in checked:
-            table_dict[a]=True
-
-        # now we'll split up the user$role strings to make a dictionary from 
-        # (user,role) to True/False, which tells us what we need to do.
-        new_user_role_dict={}
-        for (ur,val) in table_dict.items():
-            u,r = ur.split('$')
-            new_user_role_dict[(u,r)] = val
-           
-        if users_or_authz_groups=='users':
-            current_users_roles = [( uor.user.name, uor.role) for uor in current_uors if uor.user]
-        elif users_or_authz_groups=='authz_groups':
-            current_users_roles = [( uor.authorized_group.name, uor.role) for uor in current_uors if uor.authorized_group]        
-        else:
-            assert False, "shouldn't be here"
-
-        current_user_role_dict={}
-        for (u,r) in current_users_roles:
-            current_user_role_dict[(u,r)]=True
-
-        # and now we can loop through our dictionary of desired states
-        # checking whether a change needs to be made, and if so making it
-
-        # Here we check whether someone is already assigned a role, in order
-        # to avoid assigning it twice, or attempting to delete it when it
-        # doesn't exist. Otherwise problems can occur.
-        if users_or_authz_groups=='users':
-            for ((u,r), val) in new_user_role_dict.items():
-                if val:
-                    if not ((u,r) in current_user_role_dict):
-                        model.add_user_to_role(model.User.by_name(u),r,domain_object)
-                else:
-                    if ((u,r) in current_user_role_dict):
-                        model.remove_user_from_role(model.User.by_name(u),r,domain_object)
-        elif users_or_authz_groups=='authz_groups':
-            for ((u,r), val) in new_user_role_dict.items():
-                if val:
-                    if not ((u,r) in current_user_role_dict):
-                        model.add_authorization_group_to_role(model.AuthorizationGroup.by_name(u),r,domain_object)
-                else:
-                    if ((u,r) in current_user_role_dict):
-                        model.remove_authorization_group_from_role(model.AuthorizationGroup.by_name(u),r,domain_object)
-        else:
-            assert False, "shouldn't be here"
-
-        # finally commit the change to the database
-        model.repo.commit_and_remove()
-        h.flash_success("Changes Saved")
-
-    # TODO: this repeats much of _update_user_object_roles
-    def _add_user_object_role(self, users_or_authz_groups, current_uors, domain_object):
-        '''
-        current_uors: in order to avoid either creating a role twice or deleting one which is
-        non-existent, we need to get the users' current roles (if any)
-        '''
-        # The user is attempting to set new roles for a named user
-        new_user = request.params.get('new_user_name')
-        # this is the list of roles whose boxes were ticked
-        checked_roles = [ a for (a,b) in request.params.items() if (b == u'on')]
-        # this is the list of all the roles that were in the submitted form
-        submitted_roles = [ a for (a,b) in request.params.items() if (b == u'submitted')]
-
-        # from this we can make a dictionary of the desired states
-        # i.e. true for the ticked boxes, false for the unticked
-        desired_roles = {}
-        for r in submitted_roles:
-            desired_roles[r]=False
-        for r in checked_roles:
-            desired_roles[r]=True
-
-        if users_or_authz_groups=='users':
-            current_roles = [uor.role for uor in current_uors if ( uor.user and uor.user.name == new_user )]
-            user_object = model.User.by_name(new_user)
-            if user_object==None:
-                # The submitted user does not exist. Bail with flash message
-                h.flash_error('unknown user:' + str (new_user))
-            else:
-                # Whenever our desired state is different from our current state, change it.
-                for (r,val) in desired_roles.items():
-                    if val:
-                        if (r not in current_roles):
-                            model.add_user_to_role(user_object, r,
-                                    domain_object)
-                    else:
-                        if (r in current_roles):
-                            model.remove_user_from_role(user_object, r,
-                                    domain_object)
-                h.flash_success("User Added")
-
-        elif users_or_authz_groups=='authz_groups':
-            current_roles = [uor.role for uor in current_uors if ( uor.authorized_group and uor.authorized_group.name == new_user )]
-            user_object = model.AuthorizationGroup.by_name(new_user)
-            if user_object==None:
-                # The submitted user does not exist. Bail with flash message
-                h.flash_error('unknown authorization group:' + str (new_user))
-            else:
-                # Whenever our desired state is different from our current state, change it.
-                for (r,val) in desired_roles.items():
-                    if val:
-                        if (r not in current_roles):
-                            model.add_authorization_group_to_role(user_object,
-                                    r, domain_object)
-                    else:
-                        if (r in current_roles):
-                            model.remove_authorization_group_from_role(user_object,
-                                    r, domain_object)
-                h.flash_success("Authorization Group Added")
-
-        else:
-            assert False, "shouldn't be here"
-
-        # and finally commit all these changes to the database
-        model.repo.commit_and_remove()
-
 
 # Include the '_' function in the public names
 __all__ = [__name for __name in locals().keys() if not __name.startswith('_') \
