@@ -90,7 +90,7 @@ class CreateTestData(cli.CkanCommand):
     @classmethod
     def create_arbitrary(cls, package_dicts, relationships=[],
             extra_user_names=[], extra_group_names=[], 
-            commit_changesets=False, admins=[]):
+            admins=[]):
         '''Creates packages and a few extra objects as well at the
         same time if required.
         @param package_dicts - a list of dictionaries with the package
@@ -164,6 +164,7 @@ class CreateTestData(cli.CkanCommand):
                             pkg.tags.append(tag)
                             model.Session.flush()
                     elif attr == 'groups':
+                        model.Session.flush()
                         if isinstance(val, (str, unicode)):
                             group_names = val.split()
                         elif isinstance(val, list):
@@ -184,7 +185,8 @@ class CreateTestData(cli.CkanCommand):
                                     # session has not yet been committed at this point.
                                     # Fetch from the new_groups dict instead.
                                     group = new_groups[group_name]
-                            pkg.groups.append(group)
+                            member = model.Member(group=group, table_id=pkg.id, table_name='package')
+                            model.Session.add(member)
                     elif attr == 'license':
                         pkg.license_id = val
                     elif attr == 'license_id':
@@ -269,9 +271,6 @@ class CreateTestData(cli.CkanCommand):
 
             model.repo.commit_and_remove()
         
-        if commit_changesets:
-            from ckan.model.changeset import ChangesetRegister
-            changeset_ids = ChangesetRegister().commit()
 
     @classmethod
     def create_groups(cls, group_dicts, admin_user_name=None):
@@ -300,14 +299,15 @@ class CreateTestData(cli.CkanCommand):
             for pkg_name in pkg_names:
                 pkg = model.Package.by_name(unicode(pkg_name))
                 assert pkg, pkg_name
-                pkg.groups.append(group)
+                member = model.Member(group=group, table_id=pkg.id, table_name='package')
+                model.Session.add(member)
             model.Session.add(group)
             model.setup_default_user_roles(group, admin_users)
             cls.group_names.add(group_dict['name'])
         model.repo.commit_and_remove()
 
     @classmethod
-    def create(cls, commit_changesets=False):
+    def create(cls):
         import ckan.model as model
         model.Session.remove()
         rev = model.repo.new_revision()
@@ -372,11 +372,16 @@ left arrow <
         pkg2 = model.Package(name=cls.pkg_names[1])
         tag1 = model.Tag(name=u'russian')
         tag2 = model.Tag(name=u'tolstoy')
-        for obj in [pkg2, tag1, tag2]:
+
+        # Flexible tag, allows spaces, upper-case,
+        # and all punctuation except commas
+        tag3 = model.Tag(name=u'Flexible \u30a1')
+
+        for obj in [pkg2, tag1, tag2, tag3]:
             model.Session.add(obj)
-        pkg1.tags = [tag1, tag2]
-        pkg2.tags = [ tag1 ]
-        cls.tag_names = [u'russian', u'tolstoy']
+        pkg1.tags = [tag1, tag2, tag3]
+        pkg2.tags = [ tag1, tag3 ]
+        cls.tag_names = [ t.name for t in (tag1, tag2, tag3) ]
         pkg1.license_id = u'other-open'
         pkg2.license_id = u'cc-nc' # closed license
         pkg2.title = u'A Wonderful Story'
@@ -394,9 +399,12 @@ left arrow <
         
         cls.group_names.add(u'david')
         cls.group_names.add(u'roger')
-        model.Session.add(model.PackageGroup(package=pkg1, group=david))
-        model.Session.add(model.PackageGroup(package=pkg2, group=david))
-        model.Session.add(model.PackageGroup(package=pkg1, group=roger))
+
+        model.Session.flush()
+        
+        model.Session.add(model.Member(table_id=pkg1.id, table_name='package', group=david))
+        model.Session.add(model.Member(table_id=pkg2.id, table_name='package', group=david))
+        model.Session.add(model.Member(table_id=pkg1.id, table_name='package', group=roger))
         # authz
         model.Session.add_all([
             model.User(name=u'tester', apikey=u'tester', password=u'tester'),
@@ -426,10 +434,6 @@ left arrow <
 
         model.repo.commit_and_remove()
 
-        if commit_changesets:
-            from ckan.model.changeset import ChangesetRegister
-            changeset_ids = ChangesetRegister().commit()
-
         # Create a couple of authorization groups
         for ag_name in [u'anauthzgroup', u'anotherauthzgroup']:
             ag=model.AuthorizationGroup.by_name(ag_name) 
@@ -533,7 +537,7 @@ left arrow <
 search_items = [{'name':'gils',
               'title':'Government Information Locator Service',
               'url':'',
-              'tags':'registry  country-usa  government  federal  gov  workshop-20081101 penguin',
+              'tags':'registry,country-usa,government,federal,gov,workshop-20081101,penguin'.split(','),
               'resources':[{'url':'http://www.dcsf.gov.uk/rsgateway/DB/SFR/s000859/SFR17_2009_tables.xls',
                           'format':'XLS',
                           'last_modified': datetime.datetime(2005,10,01),
@@ -557,7 +561,7 @@ u with umlaut th\xfcmb
               'title':'U.S. Government Photos and Graphics',
               'url':'http://www.usa.gov/Topics/Graphics.shtml',
               'download_url':'http://www.usa.gov/Topics/Graphics.shtml',
-              'tags':'images  graphics  photographs  photos  pictures  us  usa  america  history  wildlife  nature  war  military  todo-split  gov penguin',
+              'tags':'images,graphics,photographs,photos,pictures,us,usa,america,history,wildlife,nature,war,military,todo split,gov,penguin'.split(','),
               'groups':'ukgov test1 penguin',
               'license':'other-open',
               'notes':'''## About
@@ -573,7 +577,7 @@ Collection of links to different US image collections in the public domain.
               'title':'Text of US Federal Cases',
               'url':'http://bulk.resource.org/courts.gov/',
               'download_url':'http://bulk.resource.org/courts.gov/',
-              'tags':'us  courts  case-law  us  courts  case-law  gov  legal  law  access-bulk  penguins penguin',
+              'tags':'us,courts,case-law,us,courts,case-law,gov,legal,law,access-bulk,penguins,penguin'.split(','),
               'groups':'ukgov test2 penguin',
               'license':'cc-zero',
               'notes':'''### Description
@@ -588,7 +592,7 @@ penguin
               },
              {'name':'uk-government-expenditure',
               'title':'UK Government Expenditure',
-              'tags':'workshop-20081101  uk  gov  expenditure  finance  public  funding penguin',
+              'tags':'workshop-20081101,uk,gov,expenditure,finance,public,funding,penguin'.split(','),
               'groups':'ukgov penguin',              
               'notes':'''Discussed at [Workshop on Public Information, 2008-11-02](http://okfn.org/wiki/PublicInformation).
 
@@ -599,7 +603,7 @@ Overview is available in Red Book, or Financial Statement and Budget Report (FSB
               'title':'Sweden - Government Offices of Sweden - Publications',
               'url':'http://www.sweden.gov.se/sb/d/574',
               'groups':'penguin',              
-              'tags':'country-sweden  format-pdf  access-www  documents  publications  government  eutransparency penguin',
+              'tags':u'country-sweden,format-pdf,access-www,documents,publications,government,eutransparency,penguin,CAPITALS,surprise.,greek omega \u03a9,japanese katakana \u30a1'.split(','),
               'license':'',
               'notes':'''### About
 
@@ -615,7 +619,7 @@ Not clear.''',
               'groups':'penguin',              
               'url':'http://www.opengov.se/',
               'download_url':'http://www.opengov.se/data/open/',
-              'tags':'country-sweden  government  data penguin',
+              'tags':'country-sweden,government,data,penguin'.split(','),
               'license':'cc-by-sa',
               'notes':'''### About
 
