@@ -60,9 +60,10 @@ def flatten_schema(schema, flattened=None, key=None):
     return flattened
 
 def get_all_key_combinations(data, flattented_schema):
-    '''compare the schema agaist the given data and get all valid tuples that
-    match the schema ignoring the last value in the tuple.'''
+    '''Compare the schema against the given data and get all valid tuples that
+    match the schema ignoring the last value in the tuple.
 
+    '''
     schema_prefixes = set([key[:-1] for key in flattented_schema])
     combinations = set([()])
 
@@ -167,7 +168,7 @@ def convert(converter, key, converted_data, errors, context):
         return
     except TypeError, e:
         ## hack to make sure the type error was caused by the wrong
-        ## number of arguements given.
+        ## number of arguments given.
         if not converter.__name__ in str(e):
             raise
     except Invalid, e:
@@ -182,7 +183,7 @@ def convert(converter, key, converted_data, errors, context):
         return
     except TypeError, e:
         ## hack to make sure the type error was caused by the wrong
-        ## number of arguements given.
+        ## number of arguments given.
         if not converter.__name__ in str(e):
             raise
 
@@ -206,7 +207,7 @@ def _remove_blank_keys(schema):
     return schema
 
 def validate(data, schema, context=None):
-    '''validate an unflattened nested dict agiast a schema'''
+    '''Validate an unflattened nested dict against a schema.'''
 
     context = context or {}
 
@@ -246,7 +247,7 @@ def validate_flattened(data, schema, context=None):
 
 
 def _validate(data, schema, context):
-    '''validate a flattened dict agiast a schema'''
+    '''validate a flattened dict against a schema'''
     
     converted_data = augment_data(data, schema)
     full_schema = make_full_schema(data, schema)
