@@ -12,7 +12,8 @@ _open_licenses = None
 
 VALID_SOLR_PARAMETERS = set([
     'q', 'fl', 'fq', 'rows', 'sort', 'start', 'wt', 'qf',
-    'facet', 'facet.mincount', 'facet.limit', 'facet.field'
+    'facet', 'facet.mincount', 'facet.limit', 'facet.field',
+    'extras' # Not used by Solr, but useful for extensions
 ])
 
 # for (solr) package searches, this specifies the fields that are searched 
@@ -35,7 +36,7 @@ def convert_legacy_parameters_to_solr(legacy_params):
     non_solr_params = set(legacy_params.keys()) - VALID_SOLR_PARAMETERS
     for search_key in non_solr_params:
         value_obj = legacy_params[search_key]
-        value = str(value_obj).replace('+', ' ')
+        value = value_obj.replace('+', ' ') if isinstance(value_obj, basestring) else value_obj
         if search_key == 'all_fields':
             if value:
                 solr_params['fl'] = '*'
@@ -52,11 +53,12 @@ def convert_legacy_parameters_to_solr(legacy_params):
                 tag_list = [value_obj]
             else:
                 raise SearchQueryError('Was expecting either a string or JSON list for the tags parameter: %r' % value)
-            solr_q_list.extend(['tags:%s' % tag for tag in tag_list])
+            solr_q_list.extend(['tags:"%s"' % tag for tag in tag_list])
         else:
-            if ' ' in value:
-                value = '"%s"' % value
-            solr_q_list.append('%s:%s' % (search_key, value))
+            if len(value.strip()):
+                if ' ' in value:
+                    value = '"%s"' % value
+                solr_q_list.append('%s:%s' % (search_key, value))
         del solr_params[search_key]
     solr_params['q'] = ' '.join(solr_q_list)
     if non_solr_params:
@@ -245,7 +247,14 @@ class PackageSearchQuery(SearchQuery):
             query['q'] = "*:*"
 
         # number of results
-        query['rows'] = min(1000, int(query.get('rows', 10)))
+        rows_to_return = min(1000, int(query.get('rows', 10)))
+        if rows_to_return > 0:
+            # #1683 Work around problem of last result being out of order
+            #       in SOLR 1.4
+            rows_to_query = rows_to_return + 1
+        else:
+            rows_to_query = rows_to_return
+        query['rows'] = rows_to_query
 
         # order by score if no 'sort' term given
         order_by = query.get('sort')
@@ -283,7 +292,6 @@ class PackageSearchQuery(SearchQuery):
 
         conn = make_connection()
         log.debug('Package query: %r' % query)
-        
         try:
             solr_response = conn.raw_query(**query)
         except SolrException, e:
@@ -295,6 +303,9 @@ class PackageSearchQuery(SearchQuery):
             self.count = response.get('numFound', 0)
             self.results = response.get('docs', [])
 
+            # #1683 Filter out the last row that is sometimes out of order
+            self.results = self.results[:rows_to_return]
+
             # get any extras and add to 'extras' dict
             for result in self.results:
                 extra_keys = filter(lambda x: x.startswith('extras_'), result.keys())
