@@ -29,7 +29,10 @@ class NotFound(ActionError):
 class NotAuthorized(ActionError):
     pass
 
-class ValidationError(ActionError):
+class ParameterError(ActionError):
+    pass
+
+class ValidationError(ParameterError):
     def __init__(self, error_dict, error_summary=None, extra_msg=None):
         self.error_dict = error_dict
         self.error_summary = error_summary
@@ -37,12 +40,19 @@ class ValidationError(ActionError):
 
 log = logging.getLogger(__name__)
 
-def parse_params(params):
+def parse_params(params, ignore_keys=None):
+    '''Takes a dict and returns it with some values standardised.
+    This is done on a dict before calling tuplize_dict on it.
+    '''
     parsed = {}
     for key in params:
+        if ignore_keys and key in ignore_keys:
+            continue
         value = params.getall(key)
+        # Blank values become ''
         if not value:
             value = ''
+        # A list with only one item is stripped of being a list
         if len(value) == 1:
             value = value[0]
         parsed[key] = value
@@ -84,12 +94,14 @@ def clean_dict(data_dict):
     return data_dict
 
 def tuplize_dict(data_dict):
-    ''' gets a dict with keys of the form 'table__0__key' and converts them
+    '''Takes a dict with keys of the form 'table__0__key' and converts them
     to a tuple like ('table', 0, 'key').
 
+    Dict should be put through parse_dict before this function, to have
+    values standardized.
+
     May raise a DataError if the format of the key is incorrect.
     '''
-
     tuplized_dict = {}
     for key, value in data_dict.iteritems():
         key_list = key.split('__')
