@@ -7,6 +7,7 @@ from ckan.plugins import (PluginImplementations,
 from ckan.logic import NotFound, ValidationError
 from ckan.logic import check_access
 from ckan.lib.base import _
+import ckan.lib.dictization
 from ckan.lib.dictization.model_dictize import (package_to_api1,
                                                 package_to_api2,
                                                 group_to_api1,
@@ -16,20 +17,28 @@ from ckan.lib.dictization.model_save import (group_api_to_dict,
                                              group_dict_save,
                                              package_api_to_dict,
                                              package_dict_save,
-                                             user_dict_save)
+                                             user_dict_save,
+                                             activity_dict_save)
 
 from ckan.lib.dictization.model_dictize import (group_dictize,
                                                 package_dictize,
-                                                user_dictize)
+                                                user_dictize,
+                                                activity_dictize)
 
 
-from ckan.logic.schema import default_create_package_schema, default_resource_schema
+from ckan.logic.schema import (default_create_package_schema,
+                               default_resource_schema,
+                               default_create_relationship_schema,
+                               default_create_activity_schema)
 
 from ckan.logic.schema import default_group_schema, default_user_schema
 from ckan.lib.navl.dictization_functions import validate 
 from ckan.logic.action.update import (_update_package_relationship,
                                       package_error_summary,
-                                      group_error_summary)
+                                      group_error_summary,
+                                      relationship_error_summary)
+from ckan.logic.action import rename_keys
+
 log = logging.getLogger(__name__)
 
 def package_create(context, data_dict):
@@ -40,7 +49,7 @@ def package_create(context, data_dict):
     model.Session.remove()
     model.Session()._context = context
 
-    check_access('package_create',context,data_dict)
+    check_access('package_create', context, data_dict)
 
     data, errors = validate(data_dict, schema, context)
 
@@ -63,6 +72,7 @@ def package_create(context, data_dict):
     model.setup_default_user_roles(pkg, admins)
     # Needed to let extensions know the package id
     model.Session.flush()
+
     for item in PluginImplementations(IPackageController):
         item.create(pkg)
 
@@ -107,25 +117,31 @@ def package_relationship_create(context, data_dict):
 
     model = context['model']
     user = context['user']
-    id = data_dict["id"]
-    id2 = data_dict["id2"]
-    rel_type = data_dict["rel"]
+    schema = context.get('schema') or default_create_relationship_schema()
     api = context.get('api_version') or '1'
     ref_package_by = 'id' if api == '2' else 'name'
 
-    # Create a Package Relationship.
+    id = data_dict['subject']
+    id2 = data_dict['object']
+    rel_type = data_dict['type']
+    comment = data_dict.get('comment', u'')
+
     pkg1 = model.Package.get(id)
     pkg2 = model.Package.get(id2)
     if not pkg1:
-        raise NotFound('First package named in address was not found.')
+        raise NotFound('Subject package %r was not found.' % id)
     if not pkg2:
-        return NotFound('Second package named in address was not found.')
+        return NotFound('Object package %r was not found.' % id2)
+
+    data, errors = validate(data_dict, schema, context)
+
+    if errors:
+        model.Session.rollback()
+        raise ValidationError(errors, relationship_error_summary(errors))
 
     check_access('package_relationship_create', context, data_dict)
 
-    ##FIXME should have schema
-    comment = data_dict.get('comment', u'')
-
+    # Create a Package Relationship.
     existing_rels = pkg1.get_relationships_with(pkg2, rel_type)
     if existing_rels:
         return _update_package_relationship(existing_rels[0],
@@ -136,12 +152,15 @@ def package_relationship_create(context, data_dict):
     rel = pkg1.add_relationship(rel_type, pkg2, comment=comment)
     if not context.get('defer_commit'):
         model.repo.commit_and_remove()
+    context['relationship'] = rel
+
     relationship_dicts = rel.as_dict(ref_package_by=ref_package_by)
     return relationship_dicts
 
 def group_create(context, data_dict):
     model = context['model']
     user = context['user']
+    session = context['session']
     schema = context.get('schema') or default_group_schema()
 
     check_access('group_create',context,data_dict)
@@ -149,7 +168,7 @@ def group_create(context, data_dict):
     data, errors = validate(data_dict, schema, context)
 
     if errors:
-        model.Session.rollback()
+        session.rollback()
         raise ValidationError(errors, group_error_summary(errors))
 
     rev = model.repo.new_revision()
@@ -168,9 +187,26 @@ def group_create(context, data_dict):
         admins = []
     model.setup_default_user_roles(group, admins)
     # Needed to let extensions know the group id
-    model.Session.flush()
+    session.flush()
     for item in PluginImplementations(IGroupController):
         item.create(group)
+
+    activity_dict = {
+            'user_id': model.User.by_name(user.decode('utf8')).id,
+            'object_id': group.id,
+            'activity_type': 'new group',
+            }
+    activity_dict['data'] = {
+            'group': ckan.lib.dictization.table_dictize(group, context)
+            }
+    activity_create_context = {
+        'model': model,
+        'user': user,
+        'defer_commit':True,
+        'session': session
+    }
+    activity_create(activity_create_context, activity_dict, ignore_auth=True)
+
     if not context.get('defer_commit'):
         model.repo.commit()        
     context["group"] = group
@@ -200,7 +236,7 @@ def rating_create(context, data_dict):
             if rating < ratings.MIN_RATING or rating > ratings.MAX_RATING:
                 opts_err = _('Rating must be between %i and %i.') % (ratings.MIN_RATING, ratings.MAX_RATING)
             elif not package:
-                opts_err = _('Package with name %r does not exist.') % package_ref
+                opts_err = _('Not found') + ': %r' % package_ref
     if opts_err:
         raise ValidationError(opts_err)
 
@@ -216,21 +252,39 @@ def user_create(context, data_dict):
     '''Creates a new user'''
 
     model = context['model']
-    user = context['user']
     schema = context.get('schema') or default_user_schema()
+    session = context['session']
 
     check_access('user_create', context, data_dict)
 
     data, errors = validate(data_dict, schema, context)
 
     if errors:
-        model.Session.rollback()
+        session.rollback()
         raise ValidationError(errors, group_error_summary(errors))
 
     user = user_dict_save(data, context)
 
+    # Flush the session to cause user.id to be initialised, because
+    # activity_create() (below) needs it.
+    session.flush()
+
+    activity_create_context = {
+        'model': model,
+        'user': context['user'],
+        'defer_commit': True,
+        'session': session
+    }
+    activity_dict = {
+            'user_id': user.id,
+            'object_id': user.id,
+            'activity_type': 'new user',
+            }
+    activity_create(activity_create_context, activity_dict, ignore_auth=True)
+
     if not context.get('defer_commit'):
-        model.repo.commit()        
+        model.repo.commit()
+
     context['user'] = user
     context['id'] = user.id
     log.debug('Created user %s' % str(user.name))
@@ -278,3 +332,46 @@ def group_create_rest(context, data_dict):
 
     return group_dict
 
+def activity_create(context, activity_dict, ignore_auth=False):
+    '''Create a new activity stream activity and return a dictionary
+    representation of it.
+
+    '''
+    model = context['model']
+    user = context['user']
+
+    # Any revision_id that the caller attempts to pass in the activity_dict is
+    # ignored and overwritten here.
+    if getattr(model.Session, 'revision', None):
+        activity_dict['revision_id'] = model.Session.revision.id
+    else:
+        activity_dict['revision_id'] = None
+
+    if not ignore_auth:
+        check_access('activity_create', context, activity_dict)
+
+    schema = context.get('schema') or default_create_activity_schema()
+    data, errors = validate(activity_dict, schema, context)
+    if errors:
+        raise ValidationError(errors)
+
+    activity = activity_dict_save(activity_dict, context)
+
+    if not context.get('defer_commit'):
+        model.repo.commit()
+
+    log.debug("Created '%s' activity" % activity.activity_type)
+    return activity_dictize(activity, context)
+
+def package_relationship_create_rest(context, data_dict):
+    # rename keys
+    key_map = {'id': 'subject',
+               'id2': 'object',
+               'rel': 'type'}
+    # Don't be destructive to enable parameter values for
+    # object and type to override the URL parameters.
+    data_dict = rename_keys(data_dict, key_map, destructive=False)
+
+    relationship_dict = package_relationship_create(context, data_dict)
+    return relationship_dict
+
