@@ -1,6 +1,7 @@
 from ckan.logic import NotFound
 from ckan.lib.base import _
 from ckan.logic import check_access
+from ckan.logic.action import rename_keys
 
 from ckan.plugins import PluginImplementations, IGroupController, IPackageController
 
@@ -32,18 +33,16 @@ def package_relationship_delete(context, data_dict):
 
     model = context['model']
     user = context['user']
-    id = data_dict['id']
-    id2 = data_dict['id2']
-    rel = data_dict['rel']
+    id = data_dict['subject']
+    id2 = data_dict['object']
+    rel = data_dict['type']
 
     pkg1 = model.Package.get(id)
     pkg2 = model.Package.get(id2)
     if not pkg1:
-        raise NotFound('First package named in address was not found.')
+        raise NotFound('Subject package %r was not found.' % id)
     if not pkg2:
-        return NotFound('Second package named in address was not found.')
-
-    check_access('package_relationship_delete', context, data_dict)
+        return NotFound('Object package %r was not found.' % id2)
 
     existing_rels = pkg1.get_relationships_with(pkg2, rel)
     if not existing_rels:
@@ -53,7 +52,7 @@ def package_relationship_delete(context, data_dict):
     revisioned_details = 'Package Relationship: %s %s %s' % (id, rel, id2)
 
     context['relationship'] = relationship
-    check_access('relationship_delete', context, data_dict)
+    check_access('package_relationship_delete', context, data_dict)
 
     rev = model.repo.new_revision()
     rev.author = user
@@ -103,3 +102,19 @@ def task_status_delete(context, data_dict):
 
     entity.delete()
     model.Session.commit()
+
+def package_relationship_delete_rest(context, data_dict):
+
+    # rename keys
+    key_map = {'id': 'subject',
+               'id2': 'object',
+               'rel': 'type'}
+    # We want 'destructive', so that the value of the subject,
+    # object and rel in the URI overwrite any values for these
+    # in params. This is because you are not allowed to change
+    # these values.
+    data_dict = rename_keys(data_dict, key_map, destructive=True)
+
+    package_relationship_delete(context, data_dict)
+
+    
