@@ -1,11 +1,15 @@
 from sqlalchemy.sql import select
+from sqlalchemy.orm import aliased
 from sqlalchemy import or_, and_, func, desc, case
 import uuid
 from pylons import config
+import logging
 
 import ckan
-from ckan.logic import NotFound
+from ckan.lib.base import _
+from ckan.logic import NotFound, ParameterError
 from ckan.logic import check_access
+from ckan.model import misc
 from ckan.plugins import (PluginImplementations,
                           IGroupController,
                           IPackageController)
@@ -19,7 +23,9 @@ from ckan.lib.dictization.model_dictize import (package_dictize,
                                                 group_list_dictize,
                                                 tag_dictize,
                                                 task_status_dictize,
-                                                user_dictize)
+                                                user_dictize,
+                                                activity_list_dictize,
+                                                activity_detail_list_dictize)
 
 from ckan.lib.dictization.model_dictize import (package_to_api1,
                                                 package_to_api2,
@@ -28,7 +34,9 @@ from ckan.lib.dictization.model_dictize import (package_to_api1,
                                                 tag_to_api1,
                                                 tag_to_api2)
 from ckan.lib.search import query_for, SearchError
-import logging
+from ckan.lib.base import render
+from webhelpers.html import literal
+from ckan.logic.action import get_domain_object
 
 log = logging.getLogger('ckan.logic')
 
@@ -36,24 +44,7 @@ def _package_list_with_resources(context, package_revision_list):
     package_list = []
     model = context["model"]
     for package in package_revision_list:
-        result_dict = table_dictize(package, context)
-        res_rev = model.resource_revision_table
-        resource_group = model.resource_group_table
-        query = select([res_rev], from_obj = res_rev.join(resource_group,
-                   resource_group.c.id == res_rev.c.resource_group_id))
-        query = query.where(resource_group.c.package_id == package.id)
-        result = query.where(res_rev.c.current == True).execute()
-        result_dict["resources"] = resource_list_dictize(result, context)
-        license_id = result_dict['license_id']
-        if license_id:
-            try:
-                isopen = model.Package.get_license_register()[license_id].isopen()
-                result_dict['isopen'] = isopen
-            except KeyError:
-                # TODO: create a log message this error?
-                result_dict['isopen'] = False
-        else:
-            result_dict['isopen'] = False
+        result_dict = package_dictize(package,context)
         package_list.append(result_dict)
     return package_list
 
@@ -68,7 +59,7 @@ def package_list(context, data_dict):
     user = context["user"]
     api = context.get("api_version", '1')
     ref_package_by = 'id' if api == '2' else 'name'
-    
+
     check_access('package_list', context, data_dict)
 
     query = model.Session.query(model.PackageRevision)
@@ -129,9 +120,9 @@ def group_list(context, data_dict):
     ref_group_by = 'id' if api == '2' else 'name';
     order_by = data_dict.get('order_by', 'name')
     if order_by not in set(('name', 'packages')):
-        raise ValidationError('"order_by" value %r not implemented.' % order_by)
+        raise ParameterError('"order_by" value %r not implemented.' % order_by)
     all_fields = data_dict.get('all_fields',None)
-   
+
     check_access('group_list',context, data_dict)
 
     query = model.Session.query(model.Group).join(model.GroupRevision)
@@ -146,11 +137,11 @@ def group_list(context, data_dict):
     if order_by == 'packages':
         sort_by, reverse = 'packages', True
 
+    group_list = group_list_dictize(groups, context,
+                                    lambda x:x[sort_by], reverse)
+
     if not all_fields:
-        group_list = [getattr(p, ref_group_by) for p in groups]
-    else:
-        group_list = group_list_dictize(groups, context,
-                                        lambda x:x[sort_by], reverse)
+        group_list = [group[ref_group_by] for group in group_list]
 
     return group_list
 
@@ -169,11 +160,11 @@ def group_list_authz(context, data_dict):
 
     query = Authorizer().authorized_query(user, model.Group, model.Action.EDIT)
     groups = set(query.all())
-    
+
     if available_only:
         package = context.get('package')
         if package:
-            groups = groups - set(package.groups)
+            groups = groups - set(package.get_groups())
 
     return [{'id':group.id,'name':group.name} for group in groups]
 
@@ -323,7 +314,7 @@ def package_relationships_list(context, data_dict):
         rel = None
 
     check_access('package_relationships_list',context, data_dict)
-    
+
     # TODO: How to handle this object level authz?
     relationships = Authorizer().\
                     authorized_package_relationships(\
@@ -430,17 +421,20 @@ def group_package_show(context, data_dict):
     query = model.Session.query(model.PackageRevision)\
         .filter(model.PackageRevision.state=='active')\
         .filter(model.PackageRevision.current==True)\
-        .join(model.PackageGroup, model.PackageGroup.package_id==model.PackageRevision.id)\
-        .join(model.Group, model.Group.id==model.PackageGroup.group_id)\
+        .join(model.Member, model.Member.table_id==model.PackageRevision.id)\
+        .join(model.Group, model.Group.id==model.Member.group_id)\
         .filter_by(id=group.id)\
         .order_by(model.PackageRevision.name)
 
     if limit:
         query = query.limit(limit)
 
+    if context.get('return_query'):
+        return query
+
     result = []
     for pkg_rev in query.all():
-        result.append(package_dictize(pkg_rev,context))
+        result.append(package_dictize(pkg_rev, context))
 
     return result
 
@@ -504,9 +498,20 @@ def user_show(context, data_dict):
         revision_dict = revision_show(context,{'id':revision.id})
         revision_dict['state'] = revision.state
         revisions_list.append(revision_dict)
-
     user_dict['activity'] = revisions_list
 
+    user_dict['datasets'] = []
+    dataset_q = model.Session.query(model.Package).join(model.PackageRole
+            ).filter_by(user=user_obj, role=model.Role.ADMIN
+            ).limit(50)
+
+    for dataset in dataset_q:
+        try:
+            dataset_dict = package_show(context, {'id': dataset.id})
+        except ckan.logic.NotAuthorized:
+            continue
+        user_dict['datasets'].append(dataset_dict)
+
     return user_dict
 
 def package_show_rest(context, data_dict):
@@ -556,7 +561,8 @@ def tag_show_rest(context, data_dict):
     return tag_dict
 
 def package_autocomplete(context, data_dict):
-    '''Returns packages containing the provided string'''
+    '''Returns packages containing the provided string in either the name
+    or the title'''
 
     model = context['model']
     session = context['session']
@@ -574,9 +580,17 @@ def package_autocomplete(context, data_dict):
                                 model.PackageRevision.title.ilike(like_q)))
     query = query.limit(10)
 
+    q_lower = q.lower()
     pkg_list = []
     for package in query:
-        result_dict = {'name':package.name,'title':package.title}
+        if package.name.startswith(q_lower):
+            match_field = 'name'
+            match_displayed = package.name
+        else:
+            match_field = 'title'
+            match_displayed = '%s (%s)' % (package.title, package.name)
+        result_dict = {'name':package.name, 'title':package.title,
+                       'match_field':match_field, 'match_displayed':match_displayed}
         pkg_list.append(result_dict)
 
     return pkg_list
@@ -664,38 +678,59 @@ def package_search(context, data_dict):
 
     check_access('package_search', context, data_dict)
 
-    # return a list of package ids
-    data_dict['fl'] = 'id'
+    # check if some extension needs to modify the search params
+    for item in PluginImplementations(IPackageController):
+        data_dict = item.before_search(data_dict)
 
-    query = query_for(model.Package)
-    query.run(data_dict)
+    # the extension may have decided that it's no necessary to perform the query
+    abort = data_dict.get('abort_search',False)
 
     results = []
-    for package in query.results:
-        # get the package object
-        pkg_query = session.query(model.PackageRevision)\
-            .filter(model.PackageRevision.id == package)\
-            .filter(and_(
-                model.PackageRevision.state == u'active', 
-                model.PackageRevision.current == True
-            ))
-        pkg = pkg_query.first()
+    if not abort:
+        # return a list of package ids
+        data_dict['fl'] = 'id'
 
-        ## if the index has got a package that is not in ckan then
-        ## ignore it.
-        if not pkg:
-            log.warning('package %s in index but not in database' % package)
-            continue
+        query = query_for(model.Package)
+        query.run(data_dict)
 
-        result_dict = package_dictize(pkg,context)
-        results.append(result_dict)
+        for package in query.results:
+            # get the package object
+            pkg_query = session.query(model.PackageRevision)\
+                .filter(model.PackageRevision.id == package)\
+                .filter(and_(
+                    model.PackageRevision.state == u'active',
+                    model.PackageRevision.current == True
+                ))
+            pkg = pkg_query.first()
 
-    return {
-        'count': query.count,
-        'facets': query.facets,
+            ## if the index has got a package that is not in ckan then
+            ## ignore it.
+            if not pkg:
+                log.warning('package %s in index but not in database' % package)
+                continue
+
+            result_dict = package_dictize(pkg,context)
+            results.append(result_dict)
+
+        count = query.count
+        facets = query.facets
+    else:
+        count = 0
+        facets = {}
+        results = []
+
+    search_results = {
+        'count': count,
+        'facets': facets,
         'results': results
     }
 
+    # check if some extension needs to modify the search results
+    for item in PluginImplementations(IPackageController):
+        search_results = item.after_search(search_results,data_dict)
+
+    return search_results
+
 def _extend_package_dict(package_dict,context):
     model = context['model']
 
@@ -739,7 +774,7 @@ def resource_search(context, data_dict):
             raise SearchError('Field "%s" not recognised in Resource search.' % field)
         for term in terms:
             model_attr = getattr(model.Resource, field)
-            if field == 'hash':                
+            if field == 'hash':
                 q = q.filter(model_attr.ilike(unicode(term) + '%'))
             elif field in model.Resource.get_extra_columns():
                 model_attr = getattr(model.Resource, 'extras')
@@ -751,7 +786,7 @@ def resource_search(context, data_dict):
                 q = q.filter(like)
             else:
                 q = q.filter(model_attr.ilike('%' + unicode(term) + '%'))
-    
+
     if order_by is not None:
         if hasattr(model.Resource, order_by):
             q = q.order_by(getattr(model.Resource, order_by))
@@ -759,7 +794,7 @@ def resource_search(context, data_dict):
     count = q.count()
     q = q.offset(offset)
     q = q.limit(limit)
-    
+
     results = []
     for result in q:
         if isinstance(result, tuple) and isinstance(result[0], model.DomainObject):
@@ -792,7 +827,8 @@ def tag_search(context, data_dict):
         return
 
     for term in terms:
-        q = q.filter(model.Tag.name.contains(term.lower()))
+        escaped_term = misc.escape_sql_like_special_characters(term, escape='\\')
+        q = q.filter(model.Tag.name.ilike('%' + escaped_term + '%'))
 
     count = q.count()
     q = q.offset(offset)
@@ -843,6 +879,62 @@ def get_site_user(context, data_dict):
     return {'name': user.name,
             'apikey': user.apikey}
 
+def roles_show(context, data_dict):
+    '''Returns the roles that users (and authorization groups) have on a
+    particular domain_object.
+    
+    If you specify a user (or authorization group) then the resulting roles
+    will be filtered by those of that user (or authorization group).
+
+    domain_object can be a package/group/authorization_group name or id.
+    '''
+    model = context['model']
+    session = context['session']
+    domain_object_ref = data_dict['domain_object']
+    user_ref = data_dict.get('user')
+    authgroup_ref = data_dict.get('authorization_group')
+
+    domain_object = get_domain_object(model, domain_object_ref)
+    if isinstance(domain_object, model.Package):
+        query = session.query(model.PackageRole).join('package')
+    elif isinstance(domain_object, model.Group):
+        query = session.query(model.GroupRole).join('group')
+    elif isinstance(domain_object, model.AuthorizationGroup):
+        query = session.query(model.AuthorizationGroupRole).join('authorization_group')
+    else:
+        raise NotFound(_('Cannot list entity of this type: %s') % type(domain_object).__name__)
+    # Filter by the domain_obj
+    query = query.filter_by(id=domain_object.id)
+
+    # Filter by the user / authorized_group
+    if user_ref:
+        user = model.User.get(user_ref)
+        if not user:
+            raise NotFound(_('unknown user:') + repr(user_ref))
+        query = query.join('user').filter_by(id=user.id)
+    if authgroup_ref:
+        authgroup = model.AuthorizationGroup.get(authgroup_ref)
+        if not authgroup:
+            raise NotFound('unknown authorization group:' + repr(authgroup_ref))
+        # we need an alias as we join to model.AuthorizationGroup table twice
+        ag = aliased(model.AuthorizationGroup)
+        query = query.join(ag, model.AuthorizationGroupRole.authorized_group) \
+                .filter_by(id=authgroup.id)
+
+    uors = query.all()
+
+    uors_dictized = [table_dictize(uor, context) for uor in uors]
+
+    result = {'domain_object_type': type(domain_object).__name__,
+              'domain_object_id': domain_object.id,
+              'roles': uors_dictized}
+    if user_ref:
+        result['user'] = user.id
+    if authgroup_ref:
+        result['authorization_group'] = authgroup.id
+
+    return result
+
 def status_show(context, data_dict):
     '''Provides information about the operation of this CKAN instance.'''
     return {
@@ -852,4 +944,196 @@ def status_show(context, data_dict):
         'ckan_version': ckan.__version__,
         'error_emails_to': config.get('email_to'),
         'locale_default': config.get('ckan.locale_default'),
+        'extensions': config.get('ckan.plugins').split(),
         }
+
+def user_activity_list(context, data_dict):
+    '''Return a user\'s public activity stream as a list of dicts.'''
+    model = context['model']
+    user_id = data_dict['id']
+    query = model.Session.query(model.Activity)
+    query = query.filter_by(user_id=user_id)
+    query = query.order_by(desc(model.Activity.timestamp))
+    query = query.limit(15)
+    activity_objects = query.all()
+    return activity_list_dictize(activity_objects, context)
+
+def package_activity_list(context, data_dict):
+    '''Return a package\'s public activity stream as a list of dicts.'''
+    model = context['model']
+    package_id = data_dict['id']
+    query = model.Session.query(model.Activity)
+    query = query.filter_by(object_id=package_id)
+    query = query.order_by(desc(model.Activity.timestamp))
+    query = query.limit(15)
+    activity_objects = query.all()
+    return activity_list_dictize(activity_objects, context)
+
+def group_activity_list(context, data_dict):
+    '''Return a group\'s public activity stream as a list of dicts.'''
+    model = context['model']
+    group_id = data_dict['id']
+    query = model.Session.query(model.Activity)
+    query = query.filter_by(object_id=group_id)
+    query = query.order_by(desc(model.Activity.timestamp))
+    query = query.limit(15)
+    activity_objects = query.all()
+    return activity_list_dictize(activity_objects, context)
+
+def activity_detail_list(context, data_dict):
+    '''Return an activity\'s list of activity detail items, as a list of dicts.
+    '''
+    model = context['model']
+    activity_id = data_dict['id']
+    activity_detail_objects = model.Session.query(
+        model.activity.ActivityDetail).filter_by(activity_id=activity_id).all()
+    return activity_detail_list_dictize(activity_detail_objects, context)
+
+def render_new_package_activity(context, activity):
+    return render('activity_streams/new_package.html',
+        extra_vars = {'activity': activity})
+
+def render_deleted_package_activity(context, activity):
+    return render('activity_streams/deleted_package.html',
+        extra_vars = {'activity': activity})
+
+def render_new_resource_activity(context, activity, detail):
+    return render('activity_streams/new_resource.html',
+        extra_vars = {'activity': activity, 'detail': detail})
+
+def render_changed_resource_activity(context, activity, detail):
+    return render('activity_streams/changed_resource.html',
+        extra_vars = {'activity': activity, 'detail': detail})
+
+def render_deleted_resource_activity(context, activity, detail):
+    return render('activity_streams/deleted_resource.html',
+        extra_vars = {'activity': activity, 'detail': detail})
+
+def render_added_tag_activity(context, activity, detail):
+    return render('activity_streams/added_tag.html',
+            extra_vars = {'activity': activity, 'detail': detail})
+
+def render_removed_tag_activity(context, activity, detail):
+    return render('activity_streams/removed_tag.html',
+            extra_vars = {'activity': activity, 'detail': detail})
+
+def render_new_package_extra_activity(context, activity, detail):
+    return render('activity_streams/new_package_extra.html',
+        extra_vars = {'activity': activity, 'detail': detail})
+
+def render_changed_package_extra_activity(context, activity, detail):
+    return render('activity_streams/changed_package_extra.html',
+        extra_vars = {'activity': activity, 'detail': detail})
+
+def render_deleted_package_extra_activity(context, activity, detail):
+    return render('activity_streams/deleted_package_extra.html',
+        extra_vars = {'activity': activity, 'detail': detail})
+
+def render_changed_package_activity(context, activity):
+    details = activity_detail_list(context=context,
+        data_dict={'id': activity['id']})
+
+    if len(details) == 1:
+        # If an activity has only one activity detail we try to find an
+        # activity detail renderer to use instead of rendering the normal
+        # 'changed package' template.
+        detail = details[0]
+        activity_detail_renderers = {
+            'Resource': {
+              'new': render_new_resource_activity,
+              'changed': render_changed_resource_activity,
+              'deleted': render_deleted_resource_activity
+              },
+            'tag': {
+              'added': render_added_tag_activity,
+              'removed': render_removed_tag_activity,
+              },
+            'PackageExtra': {
+                'new': render_new_package_extra_activity,
+                'changed': render_changed_package_extra_activity,
+                'deleted': render_deleted_package_extra_activity
+              },
+            }
+        object_type = detail['object_type']
+        if activity_detail_renderers.has_key(object_type):
+            activity_type = detail['activity_type']
+            if activity_detail_renderers[object_type].has_key(activity_type):
+                renderer = activity_detail_renderers[object_type][activity_type]
+                return renderer(context, activity, detail)
+
+    return render('activity_streams/changed_package.html',
+        extra_vars = {'activity': activity})
+
+def render_new_user_activity(context, activity):
+    return render('activity_streams/new_user.html',
+        extra_vars = {'activity': activity})
+
+def render_changed_user_activity(context, activity):
+    return render('activity_streams/changed_user.html',
+        extra_vars = {'activity': activity})
+
+def render_new_group_activity(context, activity):
+    return render('activity_streams/new_group.html',
+        extra_vars = {'activity': activity})
+
+def render_changed_group_activity(context, activity):
+    return render('activity_streams/changed_group.html',
+        extra_vars = {'activity': activity})
+
+def render_deleted_group_activity(context, activity):
+    return render('activity_streams/deleted_group.html',
+        extra_vars = {'activity': activity})
+
+# Global dictionary mapping activity types to functions that render activity
+# dicts to HTML snippets for including in HTML pages.
+activity_renderers = {
+  'new package' : render_new_package_activity,
+  'changed package' : render_changed_package_activity,
+  'deleted package' : render_deleted_package_activity,
+  'new user' : render_new_user_activity,
+  'changed user' : render_changed_user_activity,
+  'new group' : render_new_group_activity,
+  'changed group' : render_changed_group_activity,
+  'deleted group' : render_deleted_group_activity,
+  }
+
+def _activity_list_to_html(context, activity_stream):
+    html = []
+    for activity in activity_stream:
+        activity_type = activity['activity_type']
+        if not activity_renderers.has_key(activity_type):
+            raise NotImplementedError, ("No activity renderer for activity "
+                "type '%s'" % str(activity_type))
+        activity_html = activity_renderers[activity_type](context, activity)
+        html.append(activity_html)
+    return literal('\n'.join(html))
+
+def user_activity_list_html(context, data_dict):
+    '''Return an HTML rendering of a user\'s public activity stream.
+
+    The activity stream is rendered as a snippet of HTML meant to be included
+    in an HTML page.
+
+    '''
+    activity_stream = user_activity_list(context, data_dict)
+    return _activity_list_to_html(context, activity_stream)
+
+def package_activity_list_html(context, data_dict):
+    '''Return an HTML rendering of a package\'s public activity stream.
+
+    The activity stream is rendered as a snippet of HTML meant to be included
+    in an HTML page.
+
+    '''
+    activity_stream = package_activity_list(context, data_dict)
+    return _activity_list_to_html(context, activity_stream)
+
+def group_activity_list_html(context, data_dict):
+    '''Return an HTML rendering of a group\'s public activity stream.
+
+    The activity stream is rendered as a snippet of HTML meant to be included
+    in an HTML page.
+
+    '''
+    activity_stream = group_activity_list(context, data_dict)
+    return _activity_list_to_html(context, activity_stream)
