@@ -26,8 +26,8 @@ def package_relationship_create(context, data_dict):
     model = context['model']
     user = context['user']
 
-    id = data_dict['id']
-    id2 = data_dict['id2']
+    id = data_dict['subject']
+    id2 = data_dict['object']
     pkg1 = model.Package.get(id)
     pkg2 = model.Package.get(id2)
 
@@ -102,7 +102,9 @@ def check_group_auth(context, data_dict):
         groups.add(grp)
 
     if pkg:
-        groups = groups - set(pkg.groups)
+        pkg_groups = pkg.get_groups()
+
+        groups = groups - set(pkg_groups)
 
     for group in groups:
         if not check_access_old(group, model.Action.EDIT, context):
@@ -127,3 +129,7 @@ def group_create_rest(context, data_dict):
         return {'success': False, 'msg': _('Valid API key needed to create a group')}
 
     return group_create(context, data_dict)
+
+def activity_create(context, data_dict):
+    user = context['user']
+    return {'success': Authorizer.is_sysadmin(user)}
