@@ -13,12 +13,14 @@ from ckan.logic.validators import (package_id_not_changed,
                                    extras_unicode_convert,
                                    name_validator,
                                    package_name_validator,
+                                   package_version_validator,
                                    group_name_validator,
                                    tag_length_validator,
                                    tag_name_validator,
                                    tag_string_convert,
                                    duplicate_extras_key,
-                                   ignore_not_admin,
+                                   ignore_not_package_admin,
+                                   ignore_not_group_admin,
                                    no_http,
                                    tag_not_uppercase,
                                    user_name_validator,
@@ -28,16 +30,18 @@ from ckan.logic.validators import (package_id_not_changed,
                                    user_password_not_empty,
                                    isodate,
                                    int_validator,
-                                   user_about_validator)
+                                   user_about_validator,
+                                   user_id_exists,
+                                   object_id_validator,
+                                   activity_type_exists)
 from formencode.validators import OneOf
 import ckan.model
 
-
 def default_resource_schema():
 
     schema = {
         'id': [ignore_empty, unicode],
-        'revistion_id': [ignore_missing, unicode],
+        'revision_id': [ignore_missing, unicode],
         'resource_group_id': [ignore],
         'package_id': [ignore],
         'url': [ignore_empty, unicode],#, URL(add_http=False)],
@@ -73,7 +77,7 @@ def default_tags_schema():
                  unicode,
                  tag_length_validator,
                  tag_name_validator,
-                 tag_not_uppercase],
+                ],
         'revision_timestamp': [ignore],
         'state': [ignore],
     }
@@ -93,8 +97,8 @@ def default_package_schema():
         'license_id': [ignore_missing, unicode],
         'notes': [ignore_missing, unicode],
         'url': [ignore_missing, unicode],#, URL(add_http=False)],
-        'version': [ignore_missing, unicode],
-        'state': [ignore_not_admin, ignore_missing],
+        'version': [ignore_missing, unicode, package_version_validator],
+        'state': [ignore_not_package_admin, ignore_missing],
         '__extras': [ignore],
         '__junk': [empty],
         'resources': default_resource_schema(),
@@ -140,6 +144,7 @@ def package_form_schema():
     schema['extras_validation'] = [duplicate_extras_key, ignore]
     schema['save'] = [ignore]
     schema['return_to'] = [ignore]
+    schema['type'] = [ignore_missing, unicode]
 
     ##changes
     schema.pop("id")
@@ -157,7 +162,8 @@ def default_group_schema():
         'name': [not_empty, unicode, name_validator, group_name_validator],
         'title': [ignore_missing, unicode],
         'description': [ignore_missing, unicode],
-        'state': [ignore],
+        'type': [ignore_missing, unicode],        
+        'state': [ignore_not_group_admin, ignore_missing],
         'created': [ignore],
         'extras': default_extras_schema(),
         '__extras': [ignore],
@@ -198,14 +204,40 @@ def default_extras_schema():
 
 def default_relationship_schema():
 
-     schema = {
+    schema = {
          'id': [ignore_missing, unicode],
-         'subject_package_id': [ignore_missing, unicode],
-         'object_package_id': [ignore_missing, unicode],
+         'subject': [ignore_missing, unicode],
+         'object': [ignore_missing, unicode],
          'type': [not_empty, OneOf(ckan.model.PackageRelationship.get_all_types())],
          'comment': [ignore_missing, unicode],
          'state': [ignore],
-     }
+    }
+    return schema
+
+def default_create_relationship_schema():
+
+    schema = default_relationship_schema()
+    schema['id'] = [empty]
+    schema['subject'] = [not_empty, unicode, package_id_or_name_exists]
+    schema['object'] = [not_empty, unicode, package_id_or_name_exists]
+
+    return schema
+
+def default_update_relationship_schema():
+
+    schema = default_relationship_schema()
+    schema['id'] = [ignore_missing, package_id_not_changed]
+
+    # Todo: would like to check subject, object & type haven't changed, but
+    # no way to do this in schema
+    schema['subject'] = [ignore_missing]
+    schema['object'] = [ignore_missing]
+    schema['type'] = [ignore_missing]
+
+    return schema
+
+
+
 
 def default_user_schema():
 
@@ -261,3 +293,18 @@ def default_task_status_schema():
         'error': [ignore_missing]
     }
     return schema
+
+def default_create_activity_schema():
+    schema = {
+        'id': [ignore],
+        'timestamp': [ignore],
+        'user_id': [not_missing, not_empty, unicode, user_id_exists],
+        'object_id': [not_missing, not_empty, unicode, object_id_validator],
+        # We don't bother to validate revision ID, since it's always created
+        # internally by the activity_create() logic action function.
+        'revision_id': [],
+        'activity_type': [not_missing, not_empty, unicode,
+            activity_type_exists],
+        'data': [ignore_empty, ignore_missing, unicode],
+    }
+    return schema
