@@ -1,18 +1,21 @@
-import re
 import datetime
+from itertools import count
+import re
 from pylons.i18n import _, ungettext, N_, gettext
 from ckan.lib.navl.dictization_functions import Invalid, Missing, missing, unflatten
 from ckan.authz import Authorizer
 from ckan.logic import check_access, NotAuthorized
 from ckan.lib.helpers import date_str_to_datetime
-from ckan.model import MAX_TAG_LENGTH
+from ckan.model import (MAX_TAG_LENGTH, MIN_TAG_LENGTH,
+                        PACKAGE_NAME_MIN_LENGTH, PACKAGE_NAME_MAX_LENGTH,
+                        PACKAGE_VERSION_MAX_LENGTH)
 
 def package_id_not_changed(value, context):
 
     package = context.get('package')
     if package and value != package.id:
-        raise Invalid(_('Cannot change value of key from %s to %s. '
-                        'This key is read-only') % (package.id, value))
+        raise Invalid('Cannot change value of key from %s to %s. '
+                      'This key is read-only' % (package.id, value))
     return value
 
 def int_validator(value, context):
@@ -52,7 +55,7 @@ def package_id_exists(value, context):
 
     result = session.query(model.Package).get(value)
     if not result:
-        raise Invalid(_('Dataset was not found.'))
+        raise Invalid('%s: %s' % (_('Not found'), _('Dataset')))
     return value
 
 def package_name_exists(value, context):
@@ -63,7 +66,7 @@ def package_name_exists(value, context):
     result = session.query(model.Package).filter_by(name=value).first()
 
     if not result:
-        raise Invalid(_('Dataset with name %r does not exist.') % str(value))
+        raise Invalid(_('Not found') + ': %r' % str(value))
     return value
 
 def package_id_or_name_exists(value, context):
@@ -78,10 +81,82 @@ def package_id_or_name_exists(value, context):
     result = session.query(model.Package).filter_by(name=value).first()
 
     if not result:
-        raise Invalid(_('Dataset was not found.'))
+        raise Invalid('%s: %s' % (_('Not found'), _('Dataset')))
 
     return result.id
 
+def user_id_exists(user_id, context):
+    """Raises Invalid if the given user_id does not exist in the model given
+    in the context, otherwise returns the given user_id.
+
+    """
+    model = context['model']
+    session = context['session']
+
+    result = session.query(model.User).get(user_id)
+    if not result:
+        raise Invalid('%s: %s' % (_('Not found'), _('User')))
+    return user_id
+
+def group_id_exists(group_id, context):
+    """Raises Invalid if the given group_id does not exist in the model given
+    in the context, otherwise returns the given group_id.
+
+    """
+    model = context['model']
+    session = context['session']
+
+    result = session.query(model.Group).get(group_id)
+    if not result:
+        raise Invalid('%s: %s' % (_('Not found'), _('Group')))
+    return group_id
+
+def activity_type_exists(activity_type):
+    """Raises Invalid if there is no registered activity renderer for the
+    given activity_type. Otherwise returns the given activity_type.
+
+    """
+    from ckan.logic.action.get import activity_renderers
+    if activity_renderers.has_key(activity_type):
+        return activity_type
+    else:
+        raise Invalid('%s: %s' % (_('Not found'), _('Activity type')))
+
+# A dictionary mapping activity_type values from activity dicts to functions
+# for validating the object_id values from those same activity dicts.
+object_id_validators = {
+    'new package' : package_id_exists,
+    'changed package' : package_id_exists,
+    'deleted package' : package_id_exists,
+    'new user' : user_id_exists,
+    'changed user' : user_id_exists,
+    'new group' : group_id_exists,
+    'changed group' : group_id_exists,
+    'deleted group' : group_id_exists,
+    }
+
+def object_id_validator(key, activity_dict, errors, context):
+    """Validate the 'object_id' value of an activity_dict.
+
+    Uses the object_id_validators dict (above) to find and call an 'object_id'
+    validator function for the given activity_dict's 'activity_type' value.
+
+    Raises Invalid if the model given in context contains no object of the
+    correct type (according to the 'activity_type' value of the activity_dict)
+    with the given ID.
+
+    Raises Invalid if there is no object_id_validator for the activity_dict's
+    'activity_type' value.
+
+    """
+    activity_type = activity_dict[('activity_type',)]
+    if object_id_validators.has_key(activity_type):
+        object_id = activity_dict[('object_id',)]
+        return object_id_validators[activity_type](object_id, context)
+    else:
+        raise Invalid('There is no object_id validator for '
+            'activity type "%s"' % str(activity_type))
+
 def extras_unicode_convert(extras, context):
     for extra in extras:
         extras[extra] = unicode(extras[extra])
@@ -92,6 +167,9 @@ def name_validator(val, context):
     # check basic textual rules
     if len(val) < 2:
         raise Invalid(_('Name must be at least %s characters long') % 2)
+    if len(val) > PACKAGE_NAME_MAX_LENGTH:
+        raise Invalid(_('Name must be a maximum of %i characters long') % \
+                      PACKAGE_NAME_MAX_LENGTH)
     if not name_match.match(val):
         raise Invalid(_('Url must be purely lowercase alphanumeric '
                         '(ascii) characters and these symbols: -_'))
@@ -113,6 +191,23 @@ def package_name_validator(key, data, errors, context):
     if result:
         errors[key].append(_('That URL is already in use.'))
 
+    value = data[key]
+    if len(value) < PACKAGE_NAME_MIN_LENGTH:
+        raise Invalid(
+            _('Name "%s" length is less than minimum %s') % (value, PACKAGE_NAME_MIN_LENGTH)
+        )
+    if len(value) > PACKAGE_NAME_MAX_LENGTH:
+        raise Invalid(
+            _('Name "%s" length is more than maximum %s') % (value, PACKAGE_NAME_MAX_LENGTH)
+        )
+
+def package_version_validator(value, context):
+
+    if len(value) > PACKAGE_VERSION_MAX_LENGTH:
+        raise Invalid(_('Version must be a maximum of %i characters long') % \
+                      PACKAGE_VERSION_MAX_LENGTH)
+    return value
+
 def duplicate_extras_key(key, data, errors, context):
 
     unflattened = unflatten(data)
@@ -145,9 +240,9 @@ def group_name_validator(key, data, errors, context):
 
 def tag_length_validator(value, context):
 
-    if len(value) < 2:
+    if len(value) < MIN_TAG_LENGTH:
         raise Invalid(
-            _('Tag "%s" length is less than minimum %s') % (value, 2)
+            _('Tag "%s" length is less than minimum %s') % (value, MIN_TAG_LENGTH)
         )
     if len(value) > MAX_TAG_LENGTH:
         raise Invalid(
@@ -157,7 +252,7 @@ def tag_length_validator(value, context):
 
 def tag_name_validator(value, context):
 
-    tagname_match = re.compile('[\w\-_.]*$', re.UNICODE)
+    tagname_match = re.compile('[\w \-.]*$', re.UNICODE)
     if not tagname_match.match(value):
         raise Invalid(_('Tag "%s" must be alphanumeric '
                         'characters or symbols: -_.') % (value))
@@ -171,18 +266,32 @@ def tag_not_uppercase(value, context):
     return value
 
 def tag_string_convert(key, data, errors, context):
+    '''Takes a list of tags that is a comma-separated string (in data[key])
+    and parses tag names. These are added to the data dict, enumerated. They
+    are also validated.'''
 
-    value = data[key]
+    if isinstance(data[key], basestring):
+        tags = [tag.strip() \
+                for tag in data[key].split(',') \
+                if tag.strip()]
+    else:
+        tags = data[key]
 
-    tags = value.split()
-    for num, tag in enumerate(tags):
-        data[('tags', num, 'name')] = tag.lower()
+    current_index = max( [int(k[1]) for k in data.keys() if len(k) == 3 and k[0] == 'tags'] + [-1] )
+
+    for num, tag in zip(count(current_index+1), tags):
+        data[('tags', num, 'name')] = tag
 
     for tag in tags:
         tag_length_validator(tag, context)
         tag_name_validator(tag, context)
 
 def ignore_not_admin(key, data, errors, context):
+    # Deprecated in favour of ignore_not_package_admin
+    return ignore_not_package_admin(key, data, errors, context)
+
+def ignore_not_package_admin(key, data, errors, context):
+    '''Ignore if the user is not allowed to administer the package specified.'''
 
     model = context['model']
     user = context.get('user')
@@ -204,6 +313,29 @@ def ignore_not_admin(key, data, errors, context):
 
     data.pop(key)
 
+def ignore_not_group_admin(key, data, errors, context):
+    '''Ignore if the user is not allowed to administer for the group specified.'''
+
+    model = context['model']
+    user = context.get('user')
+
+    if user and Authorizer.is_sysadmin(user):
+        return
+
+    authorized = False
+    group = context.get('group')
+    if group:
+        try:
+            check_access('group_change_state',context)
+            authorized = True
+        except NotAuthorized:
+            authorized = False
+
+    if (user and group and authorized):
+        return
+
+    data.pop(key)
+
 def user_name_validator(key, data, errors, context):
     model = context["model"]
     session = context["session"]
