@@ -23,7 +23,7 @@ from resource import *
 from rating import *
 from package_relationship import *
 from task_status import *
-from changeset import Changeset, Change, Changemask
+from activity import *
 import ckan.migration
 from ckan.lib.helpers import OrderedDict, datetime_to_date_str
 from vdm.sqlalchemy.base import SQLAlchemySession
@@ -248,7 +248,7 @@ class Repository(vdm.sqlalchemy.Repository):
 
 
 repo = Repository(metadata, Session,
-        versioned_objects=[Package, PackageTag, Resource, ResourceGroup, PackageExtra, PackageGroup, Group]
+        versioned_objects=[Package, PackageTag, Resource, ResourceGroup, PackageExtra, Member, Group]
         )
 
 
@@ -285,26 +285,6 @@ Revision.packages = property(_get_packages)
 Revision.groups = property(_get_groups)
 Revision.user = property(_get_revision_user)
 
-def strptimestamp(s):
-    '''Convert a string of an ISO date into a datetime.datetime object.
-    
-    raises TypeError if the number of numbers in the string is not between 3
-                     and 7 (see datetime constructor).
-    raises ValueError if any of the numbers are out of range.
-    '''
-    # TODO: METHOD DEPRECATED - use ckan.lib.helpers.date_str_to_datetime
-    log.warn('model.strptimestamp is deprecated - use ckan.lib.helpers.date_str_to_datetime instead')
-    import datetime, re
-    return datetime.datetime(*map(int, re.split('[^\d]', s)))
-
-def strftimestamp(t):
-    '''Takes a datetime.datetime and returns it as an ISO string. For
-    a pretty printed string, use ckan.lib.helpers.render_datetime.
-    '''
-    # TODO: METHOD DEPRECATED - use ckan.lib.helpers.datetime_to_date_str
-    log.warn('model.strftimestamp is deprecated - use ckan.lib.helpers.datetime_to_date_str instead')
-    return t.isoformat()
-
 def revision_as_dict(revision, include_packages=True, include_groups=True,ref_package_by='name'):
     revision_dict = OrderedDict((
         ('id', revision.id),
