@@ -1,11 +1,13 @@
 import datetime
 from time import gmtime
 from calendar import timegm
+import logging
+logger = logging.getLogger(__name__)
 
 from sqlalchemy.sql import select, and_, union, expression, or_
 from sqlalchemy.orm import eagerload_all
 from sqlalchemy import types, Column, Table
-from pylons import config
+from pylons import config, session, c, request
 from meta import metadata, Session
 import vdm.sqlalchemy
 
@@ -14,12 +16,16 @@ from core import make_revisioned_table, Revision, State
 from license import License, LicenseRegister
 from domain_object import DomainObject
 import ckan.misc
+from activity import Activity, ActivityDetail
 
 __all__ = ['Package', 'package_table', 'package_revision_table',
-           'PACKAGE_NAME_MAX_LENGTH', 'PACKAGE_VERSION_MAX_LENGTH']
+           'PACKAGE_NAME_MAX_LENGTH', 'PACKAGE_NAME_MIN_LENGTH',
+           'PACKAGE_VERSION_MAX_LENGTH']
 
 PACKAGE_NAME_MAX_LENGTH = 100
+PACKAGE_NAME_MIN_LENGTH = 2
 PACKAGE_VERSION_MAX_LENGTH = 100
+
 ## Our Domain Object Tables
 package_table = Table('package', metadata,
         Column('id', types.UnicodeText, primary_key=True, default=make_uuid),
@@ -34,6 +40,7 @@ package_table = Table('package', metadata,
         Column('maintainer_email', types.UnicodeText),                      
         Column('notes', types.UnicodeText),
         Column('license_id', types.UnicodeText),
+        Column('type', types.UnicodeText),
 )
 
 
@@ -47,7 +54,7 @@ package_revision_table = make_revisioned_table(package_table)
 class Package(vdm.sqlalchemy.RevisionedObjectMixin,
         vdm.sqlalchemy.StatefulObjectMixin,
         DomainObject):
-    
+
     text_search_fields = ['name', 'title']
 
     def __init__(self, **kw):
@@ -193,7 +200,7 @@ class Package(vdm.sqlalchemy.RevisionedObjectMixin,
         tags = [tag.name for tag in self.tags]
         tags.sort() # so it is determinable
         _dict['tags'] = tags
-        groups = [getattr(group, ref_group_by) for group in self.groups]
+        groups = [getattr(group, ref_group_by) for group in self.get_groups()]
         groups.sort()
         _dict['groups'] = groups
         _dict['extras'] = dict([(key, value) for key, value in self.extras.items()])
@@ -214,7 +221,10 @@ class Package(vdm.sqlalchemy.RevisionedObjectMixin,
 
     def add_relationship(self, type_, related_package, comment=u''):
         '''Creates a new relationship between this package and a
-        related_package. It leaves the caller to commit the change.'''
+        related_package. It leaves the caller to commit the change.
+
+        Raises KeyError if the type_ is invalid.
+        '''
         import package_relationship
         from ckan import model
         if type_ in package_relationship.PackageRelationship.get_forward_types():
@@ -478,6 +488,7 @@ class Package(vdm.sqlalchemy.RevisionedObjectMixin,
             and_(model.package_tag_table.c.package_id == model.package_table.c.id,
                  model.package_tag_table.c.revision_id == model.revision_table.c.id, *where)
             ]
+
         query = union(*[select([model.revision_table.c.timestamp], x) for x in where_clauses]
                       ).order_by('timestamp DESC').limit(1)
         # Use current connection because we might be in a 'before_commit' of
@@ -485,6 +496,7 @@ class Package(vdm.sqlalchemy.RevisionedObjectMixin,
         # at the newly created revision etc. objects.
         conn = model.Session.connection() 
         result = conn.execute(query).fetchone()
+
         if result:
             result_datetime = iso_date_to_datetime_for_sqlite(result[0])
             timestamp_without_usecs = result_datetime.utctimetuple()
@@ -493,6 +505,16 @@ class Package(vdm.sqlalchemy.RevisionedObjectMixin,
             timestamp_float = timegm(timestamp_without_usecs) + usecs
             return datetime.datetime.utcfromtimestamp(timestamp_float)
 
+    def get_groups(self):
+        import ckan.model as model
+        if '_groups' not in self.__dict__:
+            self._groups = model.Session.query(model.Group).\
+               join(model.Member, model.Member.group_id == model.Group.id).\
+               join(model.Package, model.Package.id == model.Member.table_id).\
+               filter(model.Member.state == 'active').\
+               filter(model.Package.id == self.id).all()
+        return self._groups
+
     @property
     def metadata_created(self):
         import ckan.model as model
@@ -522,3 +544,52 @@ class Package(vdm.sqlalchemy.RevisionedObjectMixin,
 
         return fields
 
+    def activity_stream_item(self, activity_type, revision, user_id):
+        import ckan.model
+        import ckan.lib.dictization
+        import ckan.logic
+        assert activity_type in ("new", "changed"), (
+            str(activity_type))
+
+        # Handle 'deleted' objects.
+        # When the user marks a package as deleted this comes through here as
+        # a 'changed' package activity. We detect this and change it to a
+        # 'deleted' activity.
+        if activity_type == 'changed' and self.state == u'deleted':
+            if ckan.model.Session.query(ckan.model.Activity).filter_by(
+                    object_id=self.id, activity_type='deleted').all():
+                # A 'deleted' activity for this object has already been emitted
+                # FIXME: What if the object was deleted and then activated
+                # again?
+                return None
+            else:
+                # Emit a 'deleted' activity for this object.
+                activity_type = 'deleted'
+
+        try:
+            d = {'package': ckan.lib.dictization.table_dictize(self,
+                context={'model': ckan.model})}
+            return Activity(user_id, self.id, revision.id,
+                    "%s package" % activity_type, d)
+        except ckan.logic.NotFound:
+            # This happens if this package is being purged and therefore has no
+            # current revision.
+            # TODO: Purge all related activity stream items when a model object
+            # is purged.
+            return None
+
+    def activity_stream_detail(self, activity_id, activity_type):
+        import ckan.model
+        import ckan.lib.dictization
+
+        # Handle 'deleted' objects.
+        # When the user marks a package as deleted this comes through here as
+        # a 'changed' package activity. We detect this and change it to a
+        # 'deleted' activity.
+        if activity_type == 'changed' and self.state == u'deleted':
+            activity_type = 'deleted'
+
+        package_dict = ckan.lib.dictization.table_dictize(self,
+                context={'model':ckan.model})
+        return ActivityDetail(activity_id, self.id, u"Package", activity_type,
+            {'package': package_dict })
