@@ -10,7 +10,7 @@
     CKAN.Utils.setupMarkdownEditor($('.markdown-editor'));
     // set up ckan js
     var config = {
-      endpoint: '/'
+      endpoint: CKAN.SITE_URL + '/'
     };
     var client = new CKAN.Client(config);
     // serious hack to deal with hacky code in ckanjs
@@ -23,10 +23,21 @@
       CKAN.Utils.setupWelcomeBanner($('.js-welcome-banner'));
     }
 
+    var isGroupView = $('body.group.read').length > 0;
+    if (isGroupView) {
+      // Show extract of notes field
+      CKAN.Utils.setupNotesExtract();
+    }
+
     var isDatasetView = $('body.package.read').length > 0;
     if (isDatasetView) {
-      var _dataset = new CKAN.Model.Dataset(preload_dataset);
-      CKANEXT.DATAPREVIEW.setupDataPreview(_dataset);
+      // Show extract of notes field
+      CKAN.Utils.setupNotesExtract();
+    }
+
+    var isResourceView = $('body.package.resource_read').length > 0;
+    if (isResourceView) {
+      CKANEXT.DATAPREVIEW.loadPreviewDialog(preload_resource);
     }
 
     var isDatasetNew = $('body.package.new').length > 0;
@@ -69,6 +80,18 @@
         el: $el
       });
       view.render();
+
+      // Set up dataset delete button
+      var select = $('select.dataset-delete');
+      select.attr('disabled','disabled');
+      select.css({opacity: 0.3});
+      $('button.dataset-delete').click(function(e) {
+        select.removeAttr('disabled');
+        select.fadeTo('fast',1.0);
+        $(e.target).css({opacity:0});
+        $(e.target).attr('disabled','disabled');
+        return false;
+      });
     }
     var isGroupEdit = $('body.group.edit').length > 0;
     if (isGroupEdit) {
@@ -134,7 +157,7 @@ CKAN.Utils = function($, my) {
     if (urlInput.length==0) throw "No urlInput found.";
     if (validMsg.length==0) throw "No validMsg found.";
 
-    var api_url = '/api/2/util/is_slug_valid';
+    var api_url = CKAN.SITE_URL + '/api/2/util/is_slug_valid';
     // (make length less than max, in case we need a few for '_' chars to de-clash slugs.)
     var MAX_SLUG_LENGTH = 90;
 
@@ -292,8 +315,8 @@ CKAN.Utils = function($, my) {
         minLength: 1,
         source: function(request, callback) {
           // here request.term is whole list of tags so need to get last
-          var _realTerm = request.term.split(' ').pop();
-          var url = '/api/2/util/tag/autocomplete?incomplete=' + _realTerm;
+          var _realTerm = request.term.split(',').pop().trim();
+          var url = CKAN.SITE_URL + '/api/2/util/tag/autocomplete?incomplete=' + _realTerm;
           $.getJSON(url, function(data) {
             // data = { ResultSet: { Result: [ {Name: tag} ] } } (Why oh why?)
             var tags = $.map(data.ResultSet.Result, function(value, idx) {
@@ -309,14 +332,14 @@ CKAN.Utils = function($, my) {
           return false;
         },
         select: function( event, ui ) {
-          var terms = this.value.split(' ');
+          var terms = this.value.split(',');
           // remove the current input
           terms.pop();
           // add the selected item
-          terms.push( ui.item.value );
+          terms.push( " "+ui.item.value );
           // add placeholder to get the comma-and-space at the end
-          terms.push( "" );
-          this.value = terms.join( " " );
+          terms.push( " " );
+          this.value = terms.join( "," );
           return false;
         }
     });
@@ -329,7 +352,7 @@ CKAN.Utils = function($, my) {
     elements.autocomplete({
       minLength: 1,
       source: function(request, callback) {
-        var url = '/api/2/util/resource/format_autocomplete?incomplete=' + request.term;
+        var url = CKAN.SITE_URL + '/api/2/util/resource/format_autocomplete?incomplete=' + request.term;
         $.getJSON(url, function(data) {
           // data = { ResultSet: { Result: [ {Name: tag} ] } } (Why oh why?)
           var formats = $.map(data.ResultSet.Result, function(value, idx) {
@@ -348,7 +371,7 @@ CKAN.Utils = function($, my) {
     elements.autocomplete({
       minLength: 2,
       source: function(request, callback) {
-        var url = '/api/2/util/user/autocomplete?q=' + request.term;
+        var url = CKAN.SITE_URL + '/api/2/util/user/autocomplete?q=' + request.term;
         $.getJSON(url, function(data) {
           $.each(data, function(idx, userobj) {
             var label = userobj.name;
@@ -371,7 +394,7 @@ CKAN.Utils = function($, my) {
     elements.autocomplete({
       minLength: 2,
       source: function(request, callback) {
-        var url = '/api/2/util/authorizationgroup/autocomplete?q=' + request.term;
+        var url = CKAN.SITE_URL + '/api/2/util/authorizationgroup/autocomplete?q=' + request.term;
         $.getJSON(url, function(data) {
           $.each(data, function(idx, userobj) {
             var label = userobj.name;
@@ -399,7 +422,7 @@ CKAN.Utils = function($, my) {
         $target.addClass('depressed');
         raw_markdown=textarea.val();
         preview.html("<em>"+CKAN.Strings.loading+"<em>");
-        $.post("/api/util/markdown", { q: raw_markdown },
+        $.post(CKAN.SITE_URL + "/api/util/markdown", { q: raw_markdown },
           function(data) { preview.html(data); }
         );
         preview.width(textarea.width())
@@ -416,6 +439,31 @@ CKAN.Utils = function($, my) {
     });
   };
 
+  // If notes field is more than 1 paragraph, just show the
+  // first paragraph with a 'Read more' link that will expand
+  // the div if clicked
+  my.setupNotesExtract = function() {
+    var notes = $('#content div.notes');
+    if(notes.find('p').length > 1){
+      var extract = notes.children(':eq(0)');
+      var remainder = notes.children(':gt(0)');
+      notes.html($.tmpl(CKAN.Templates.notesField));
+      notes.find('#notes-extract').html(extract);
+      notes.find('#notes-remainder').html(remainder);
+      notes.find('#notes-remainder').hide();
+      notes.find('#notes-toggle a').click(function(event){
+        notes.find('#notes-toggle a').toggle();
+        var remainder = notes.find('#notes-remainder')
+        if ($(event.target).hasClass('more')) {
+          remainder.slideDown();
+        }
+        else {
+          remainder.slideUp();
+        }
+        return false;
+      })
+    }
+  };
 
   // Show/hide fieldset sections from the edit dataset form. 
   my.setupDatasetEditNavigation = function() {
@@ -456,7 +504,12 @@ CKAN.View.DatasetEditForm = Backbone.View.extend({
       var boundToUnload = false;
       return function() {
         if (!boundToUnload) {
-          CKAN.Utils.flashMessage(CKAN.Strings.youHaveUnsavedChanges,'notice');
+          var parentDiv = $('<div />').addClass('flash-messages');
+          var messageDiv = $('<div />').html(CKAN.Strings.youHaveUnsavedChanges).addClass('notice').hide();
+          parentDiv.append(messageDiv);
+          $('#unsaved-warning').append(parentDiv);
+          messageDiv.show(200);
+
           boundToUnload = true;
           window.onbeforeunload = function () { 
             return CKAN.Strings.youHaveUnsavedChanges; 
@@ -465,7 +518,7 @@ CKAN.View.DatasetEditForm = Backbone.View.extend({
       }
     }();
 
-    $form.find('input').live('change', function(e) {
+    $form.find('input,select').live('change', function(e) {
       $target = $(e.target);
       // Entering text in the 'add' box does not represent a change
       if ($target.closest('.resource-add').length==0) {
@@ -527,7 +580,16 @@ CKAN.View.ResourceEditList = Backbone.View.extend({
     $tr.html($.tmpl(
       CKAN.Templates.resourceEntry, 
       { resource: resource.toTemplateJSON(),
-        num: position
+        num: position,
+        resourceTypeOptions: [
+          ['file', 'Data File']
+          , ['api', 'API']
+          , ['image', 'Image']
+          , ['metadata', 'Metadata']
+          , ['documentation', 'Documentation']
+          , ['code', 'Code']
+          , ['example', 'Example']
+        ]
       }
     ));
     $tr.find('.js-resource-edit-expanded').hide();
@@ -707,88 +769,13 @@ CKAN.View.ResourceAddLink = Backbone.View.extend({
   }
 });
 
-
 (function ($) {
   var my = {};
   my.jsonpdataproxyUrl = 'http://jsonpdataproxy.appspot.com/';
+  my.dialogId = 'ckanext-datapreview';
+  my.$dialog = $('#' + my.dialogId);
 
-  my.setupDataPreview = function(dataset) {
-    var dialogId = 'ckanext-datapreview-dialog';
-    // initialize the tableviewer system
-    DATAEXPLORER.TABLEVIEW.initialize(dialogId);
-    if (dataset.get('resources').length>0) {
-      my.createPreviewButtons(dataset, $('.resources'));
-    }
-  };
-
-  // Public: Creates the base UI for the plugin.
-  //
-  // Also requests the package from the api to see if there is any chart
-  // data stored and updates the preview icons accordingly.
-  //
-  // dataset: Dataset model for the dataset on this page.
-  // resourceElements - The resources table wrapped in jQuery.
-  //
-  // Returns nothing.
-  my.createPreviewButtons = function(dataset, resourceElements) {
-    // rather pointless, but w/o assignment dataset not available in loop below (??!)
-    var currentDataset = dataset;
-    resourceElements.find('tr td:last-child').each(function(idx, element) {
-      var element = $(element);
-      var resource = currentDataset.get('resources').models[idx];
-      var resourceData = resource.toJSON();
-      resourceData.formatNormalized = my.normalizeFormat(resourceData.format);
-
-      // do not create previews for some items
-      var _tformat = resourceData.format.toLowerCase();
-      if (
-        _tformat.indexOf('zip') != -1
-        ||
-        _tformat.indexOf('tgz') != -1
-        ||
-        _tformat.indexOf('targz') != -1
-        ||
-        _tformat.indexOf('gzip') != -1
-        ||
-        _tformat.indexOf('gz:') != -1
-        ||
-        _tformat.indexOf('word') != -1
-        ||
-        _tformat.indexOf('pdf') != -1
-        ||
-        _tformat === 'other'
-        )
-      {
-        return;
-      }
-
-      var _previewSpan = $('<a />', {
-        text: 'Preview',
-        href: resourceData.url,
-        click: function(e) {
-          e.preventDefault();
-          my.loadPreviewDialog(e.target);
-        },
-        'class': 'resource-preview-button'
-      }).data('preview', resourceData).appendTo(element);
-
-      var chartString, charts = {};
-
-      if (resource) {
-        chartString = resource[my.resourceChartKey];
-        if (chartString) {
-          try {
-            charts = $.parseJSON(chartString);
-
-            // If parsing succeeds add a class to the preview button.
-            _previewSpan.addClass('resource-preview-chart');
-          } catch (e) {}
-        }
-      }
-    });
-  };
-
-  // **Public: Loads a data preview dialog for a preview button.**
+  // **Public: Loads a data preview**
   //
   // Fetches the preview data object from the link provided and loads the
   // parsed data from the webstore displaying it in the most appropriate
@@ -797,11 +784,20 @@ CKAN.View.ResourceAddLink = Backbone.View.extend({
   // link - Preview button.
   //
   // Returns nothing.
-  my.loadPreviewDialog = function(link) {
-    var preview  = $(link).data('preview');
-    preview.url  = my.normalizeUrl(link.href);
+  my.loadPreviewDialog = function(resourceData) {
+    my.$dialog.html('<h4>Loading ... <img src="http://assets.okfn.org/images/icons/ajaxload-circle.gif" class="loading-spinner" /></h4>');
 
-    $(link).addClass('resource-preview-loading').text('Loading');
+    function initializeDataExplorer(dataset) {
+      var dataExplorer = new recline.View.DataExplorer({
+        el: my.$dialog
+        , model: dataset
+        , config: {
+          readOnly: true
+        }
+      });
+      // will have to refactor if this can get called multiple times
+      Backbone.history.start();
+    }
 
     // 4 situations
     // a) have a webstore_url
@@ -810,32 +806,38 @@ CKAN.View.ResourceAddLink = Backbone.View.extend({
     // d) none of the above but worth iframing (assumption is
     // that if we got here (i.e. preview shown) worth doing
     // something ...)
-    if (preview.formatNormalized === '') {
-      var tmp = preview.url.split('/');
+    resourceData.formatNormalized = my.normalizeFormat(resourceData.format);
+
+    resourceData.url  = my.normalizeUrl(resourceData.url);
+    if (resourceData.formatNormalized === '') {
+      var tmp = resourceData.url.split('/');
       tmp = tmp[tmp.length - 1];
       tmp = tmp.split('?'); // query strings
       tmp = tmp[0];
       var ext = tmp.split('.');
       if (ext.length > 1) {
-        preview.formatNormalized = ext[ext.length-1];
+        resourceData.formatNormalized = ext[ext.length-1];
       }
     }
 
-    if (preview.webstore_url) {
-      var _url = preview.webstore_url + '.jsontuples?_limit=500';
-      my.getResourceDataDirect(_url, function(data) {
-        DATAEXPLORER.TABLEVIEW.showData(data);
-        DATAEXPLORER.TABLEVIEW.$dialog.dialog('open');
+    if (resourceData.webstore_url) {
+      var backend = new recline.Model.BackendWebstore({
+        url: resourceData.webstore_url
       });
+      recline.Model.setBackend(backend);
+      var dataset = backend.getDataset();
+      initializeDataExplorer(dataset);
     }
-    else if (preview.formatNormalized in {'csv': '', 'xls': ''}) {
-      var _url = my.jsonpdataproxyUrl + '?url=' + preview.url + '&type=' + preview.formatNormalized;
-      my.getResourceDataDirect(_url, function(data) {
-        DATAEXPLORER.TABLEVIEW.showData(data);
-        DATAEXPLORER.TABLEVIEW.$dialog.dialog('open');
+    else if (resourceData.formatNormalized in {'csv': '', 'xls': ''}) {
+      var backend = new recline.Model.BackendDataProxy({
+        url: resourceData.url
+        , type: resourceData.formatNormalized
       });
+      recline.Model.setBackend(backend);
+      var dataset = backend.getDataset();
+      initializeDataExplorer(dataset);
     }
-    else if (preview.formatNormalized in {
+    else if (resourceData.formatNormalized in {
         'rdf+xml': '',
         'owl+xml': '',
         'xml': '',
@@ -850,24 +852,39 @@ CKAN.View.ResourceAddLink = Backbone.View.extend({
         }) {
       // HACK: treat as plain text / csv
       // pass url to jsonpdataproxy so we can load remote data (and tell dataproxy to treat as csv!)
-      var _url = my.jsonpdataproxyUrl + '?type=csv&url=' + preview.url;
+      var _url = my.jsonpdataproxyUrl + '?type=csv&url=' + resourceData.url;
       my.getResourceDataDirect(_url, function(data) {
         my.showPlainTextData(data);
-        DATAEXPLORER.TABLEVIEW.$dialog.dialog('open');
       });
     }
-    else {
-      // HACK: but should work
+    else if (resourceData.formatNormalized in {'html':'', 'htm':''} 
+        ||  resourceData.url.substring(0,23)=='http://docs.google.com/') {
       // we displays a fullscreen dialog with the url in an iframe.
-      // HACK: we borrow dialog from DATAEXPLORER.TABLEVIEW
-      var $dialog = DATAEXPLORER.TABLEVIEW.$dialog;
-      $dialog.empty();
-      $dialog.dialog('option', 'title', 'Preview: ' + preview.url);
+      my.$dialog.empty();
       var el = $('<iframe></iframe>');
-      el.attr('src', preview.url);
+      el.attr('src', resourceData.url);
       el.attr('width', '100%');
       el.attr('height', '100%');
-      $dialog.append(el).dialog('open');;
+      my.$dialog.append(el);
+    }
+    // images
+    else if (resourceData.formatNormalized in {'png':'', 'jpg':'', 'gif':''} 
+        ||  resourceData.resource_type=='image') {
+      // we displays a fullscreen dialog with the url in an iframe.
+      my.$dialog.empty();
+      var el = $('<img />');
+      el.attr('src', resourceData.url);
+      el.css('max-width', '100%');
+      el.css('border', 'solid 4px black');
+      my.$dialog.append(el);
+    }
+    else {
+      // Cannot reliably preview this item - with no mimetype/format information, 
+      // can't guarantee it's not a remote binary file such as an executable.
+      my.showError({
+        title: 'Preview not available for data type: ' + resourceData.formatNormalized
+        , message: ''
+      });
     }
   };
 
@@ -918,22 +935,26 @@ CKAN.View.ResourceAddLink = Backbone.View.extend({
   //
   // Returns nothing.
   my.showPlainTextData = function(data) {
-    // HACK: have to reach into DATAEXPLORER.TABLEVIEW dialog  a lot ...
-    DATAEXPLORER.TABLEVIEW.setupFullscreenDialog();
-
     if(data.error) {
-      DATAEXPLORER.TABLEVIEW.showError(data.error);
+      my.showError(data.error);
     } else {
       var content = $('<pre></pre>');
       for (var i=0; i<data.data.length; i++) {
         var row = data.data[i].join(',') + '\n';
         content.append(my.escapeHTML(row));
       }
-      DATAEXPLORER.TABLEVIEW.$dialog.dialog(DATAEXPLORER.TABLEVIEW.dialogOptions);
-      DATAEXPLORER.TABLEVIEW.$dialog.append(content);
+      my.$dialog.html(content);
     }
   };
 
+  my.showError = function (error) {
+    var _html = _.template(
+        '<div class="alert-message warning"><strong><%= title %></strong><br /><%= message %></div>'
+        , error
+        );
+    my.$dialog.html(_html);
+  };
+
   my.normalizeFormat = function(format) {
     var out = format.toLowerCase();
     out = out.split('/');
