@@ -4,6 +4,8 @@ try:
 except ImportError:
     from StringIO import StringIO
 
+import urllib
+
 from pylons import config
 import webhelpers.util
 from nose.tools import assert_equal
@@ -63,11 +65,26 @@ class ApiTestCase(object):
 
     @classmethod
     def offset(self, path):
+        """
+        Returns the full path to the resource identified in path.
+
+        Performs necessary url-encodings, ie:
+
+         - encodes unicode to utf8
+         - urlencodes the resulting byte array
+
+        This process is described in [1], and has also been confirmed by
+        inspecting what a browser does.
+
+        [1] http://www.w3.org/International/articles/idn-and-iri/
+        """
         assert self.api_version != None, "API version is missing."
         base = '/api'
         if self.api_version:
             base += '/' + self.api_version
-        return '%s%s' % (base, path)
+        utf8_encoded = (u'%s%s' % (base, path)).encode('utf8')
+        url_encoded = urllib.quote(utf8_encoded)
+        return url_encoded
 
     def assert_msg_represents_anna(self, msg):
         assert 'annakarenina' in msg, msg
@@ -95,6 +112,11 @@ class ApiTestCase(object):
         assert 'ckan_url' in msg
         assert '"ckan_url": "http://test.ckan.net/dataset/annakarenina"' in msg, msg
 
+        assert 'tags' in data, "Expected a tags list in json payload"
+        assert self.russian.name in data['tags'], data['tags']
+        assert self.tolstoy.name in data['tags'], data['tags']
+        assert self.flexible_tag.name in data['tags'], data['tags']
+
     def assert_msg_represents_roger(self, msg):
         assert 'roger' in msg, msg
         data = self.loads(msg)
@@ -117,6 +139,19 @@ class ApiTestCase(object):
         differences = expected_pkgs ^ pkgs
         assert not differences, '%r != %r' % (pkgs, expected_pkgs)
 
+    def assert_msg_represents_flexible_tag(self, msg):
+        """
+        Asserts the correct packages are associated with the flexible tag.
+
+        Namely, 'annakarenina' and 'warandpeace'.
+        """
+        data = self.loads(msg)
+        pkgs = set(data)
+        expected_pkgs = set([self.package_ref_from_name('annakarenina'),
+                             self.package_ref_from_name('warandpeace')])
+        differences = expected_pkgs ^ pkgs
+        assert not differences, '%r != %r' % (pkgs, expected_pkgs)
+
     def data_from_res(self, res):
         return self.loads(res.body)
 
@@ -210,30 +245,6 @@ class Api1and2TestCase(object):
             package_ref = self.package_ref_from_name(package_name)
             return self.offset('/rest/rating/%s' % package_ref)
 
-    def relationship_offset(self, package_1_name=None,
-                            relationship_type=None,
-                            package_2_name=None,
-                            ):
-        assert package_1_name
-        package_1_ref = self.package_ref_from_name(package_1_name)
-        if package_2_name is None:
-            if not relationship_type:
-                return self.offset('/rest/dataset/%s/relationships' % \
-                                   package_1_ref)
-            else:
-                return self.offset('/rest/dataset/%s/%s' %
-                                   (package_1_ref, relationship_type))
-        else:
-            package_2_ref = self.package_ref_from_name(package_2_name)
-            if not relationship_type:
-                return self.offset('/rest/dataset/%s/relationships/%s' % \
-                                   (package_1_ref, package_2_ref))
-            else:
-                return self.offset('/rest/dataset/%s/%s/%s' % \
-                                   (package_1_ref,
-                                    relationship_type,
-                                    package_2_ref))
-
     def anna_offset(self, postfix=''):
         return self.package_offset('annakarenina') + postfix
 
@@ -267,13 +278,6 @@ class Api1and2TestCase(object):
         assert cls.ref_group_by in ['id', 'name']
         return getattr(group, cls.ref_group_by)
 
-    @classmethod
-    def _list_package_refs(cls, packages):
-        return [getattr(p, cls.ref_package_by) for p in packages]
-
-    @classmethod
-    def _list_group_refs(cls, groups):
-        return [getattr(p, cls.ref_group_by) for p in groups]
 
 class Api1TestCase(Api1and2TestCase):
 
@@ -319,7 +323,7 @@ class ApiUnversionedTestCase(Api1TestCase):
         return self.oldest_api_version
 
 
-class BaseModelApiTestCase(ModelMethods, ApiTestCase, ControllerTestCase):
+class BaseModelApiTestCase(ApiTestCase, ControllerTestCase):
 
     testpackage_license_id = u'gpl-3.0'
     package_fixture_data = {
@@ -358,17 +362,21 @@ class BaseModelApiTestCase(ModelMethods, ApiTestCase, ControllerTestCase):
 
     def setup(self):
         super(BaseModelApiTestCase, self).setup()
-        self.conditional_create_common_fixtures()
-        self.init_extra_environ()
+#        self.conditional_create_common_fixtures()
+#        self.init_extra_environ()
 
     def teardown(self):
         model.Session.remove()
-        model.repo.rebuild_db()
+#        model.repo.rebuild_db()
         super(BaseModelApiTestCase, self).teardown()
 
-    def init_extra_environ(self):
-        self.user = model.User.by_name(self.user_name)
-        self.extra_environ={'Authorization' : str(self.user.apikey)}
+    @classmethod
+    def init_extra_environ(cls, user_name):
+        # essentially 'logs you in', so the http_request methods
+        # called elsewhere in this class are run with the specified
+        # user logged in.
+        cls.user = model.User.by_name(user_name)
+        cls.extra_environ={'Authorization' : str(cls.user.apikey)}
 
     def post_json(self, offset, data, status=None, extra_environ=None):
         ''' Posts data in the body in application/json format, used by
@@ -409,6 +417,7 @@ class BaseModelApiTestCase(ModelMethods, ApiTestCase, ControllerTestCase):
         if content_length is not None:
             environ['CONTENT_LENGTH'] = str(content_length)
         environ['REQUEST_METHOD'] = request_method
+        environ['QUERY_STRING'] = '' # avoids a warning
         environ['wsgi.input'] = StringIO(data)
         if extra_environ:
             environ.update(extra_environ)
