@@ -1,17 +1,9 @@
-import os
-from paste.deploy import appconfig
-import paste.fixture
-from ckan.config.middleware import make_app
 import ckan.model as model
-from ckan.tests import conf_dir, url_for, CreateTestData
-from controllers.admin import get_sysadmins
+from ckan.tests import url_for, CreateTestData, WsgiAppCase
 
-class TestAdminController:
+class TestAdminController(WsgiAppCase):
     @classmethod
     def setup_class(cls):
-        config = appconfig('config:test.ini', relative_to=conf_dir)
-        wsgiapp = make_app(config.global_conf, **config.local_conf)
-        cls.app = paste.fixture.TestApp(wsgiapp)
         # setup test data including testsysadmin user
         CreateTestData.create()
 
@@ -34,12 +26,9 @@ class TestAdminController:
         assert 'Administration' in response, response
 
 
-class TestAdminAuthzController:
+class TestAdminAuthzController(WsgiAppCase):
     @classmethod
     def setup_class(cls):
-        config = appconfig('config:test.ini', relative_to=conf_dir)
-        wsgiapp = make_app(config.global_conf, **config.local_conf)
-        cls.app = paste.fixture.TestApp(wsgiapp)
         # setup test data including testsysadmin user
         CreateTestData.create()
         # Creating a couple of authorization groups, which are enough to break
@@ -235,11 +224,8 @@ class TestAdminAuthzController:
                "should be a reader now"
 
 
-class TestAdminTrashController:
+class TestAdminTrashController(WsgiAppCase):
     def setup(cls):
-        config = appconfig('config:test.ini', relative_to=conf_dir)
-        wsgiapp = make_app(config.global_conf, **config.local_conf)
-        cls.app = paste.fixture.TestApp(wsgiapp)
         CreateTestData.create()
 
     def teardown(self):
