@@ -17,18 +17,21 @@ import ckan.model as model
 from ckan.lib.create_test_data import CreateTestData
 import ckan.lib.helpers as h
 import ckan.lib.search as search
+from ckan.lib.i18n import set_session_locale
+from ckan.logic.action import get, update
 from ckan.controllers.package import PackageController
 from ckan.plugins import SingletonPlugin, implements, IPackageController
 from ckan import plugins
 from ckan.rating import set_rating
+from ckan.lib.search.common import SolrSettings
 
 class MockPackageControllerPlugin(SingletonPlugin):
     implements(IPackageController)
-    
+
     def __init__(self):
         from collections import defaultdict
         self.calls = defaultdict(int)
-    
+
     def read(self, entity):
         self.calls['read'] += 1
 
@@ -46,7 +49,19 @@ class MockPackageControllerPlugin(SingletonPlugin):
 
     def delete(self, entity):
         self.calls['delete'] += 1
-    
+
+    def before_search(self, search_params):
+        self.calls['before_search'] += 1
+        return search_params
+
+    def after_search(self, search_results, search_params):
+        self.calls['after_search'] += 1
+        return search_results
+
+    def before_index(self, search_params):
+        self.calls['before_index'] += 1
+        return search_params
+
 
 existing_extra_html = ('<label class="field_opt" for="Package-%(package_id)s-extras-%(key)s">%(capitalized_key)s</label>', '<input id="Package-%(package_id)s-extras-%(key)s" name="Package-%(package_id)s-extras-%(key)s" size="20" type="text" value="%(value)s">')
 
@@ -56,7 +71,7 @@ class TestPackageBase(FunctionalTestCase):
     # Note: Can't put a quotation mark in key1 or value1 because
     # paste.fixture doesn't unescape the value in an input field
     # on form submission. (But it works in real life.)
-    
+
     def _assert_form_errors(self, res):
         self.check_tag(res, '<form', 'class="has-errors"')
         assert 'field_error' in res, res
@@ -78,7 +93,7 @@ class TestPackageForm(TestPackageBase):
         main_div_str = main_div.encode('utf8')
         assert params['name'] in main_div, main_div_str
         assert params['title'] in main_div, main_div_str
-        assert params['version'] in main_div, main_div_str 
+        assert params['version'] in main_div, main_div_str
         self.check_named_element(main_div, 'a', 'href="%s"' % params['url'])
         prefix = 'Dataset-%s-' % params.get('id', '')
         for res_index, values in self._get_resource_values(params['resources'], by_resource=True):
@@ -86,7 +101,7 @@ class TestPackageForm(TestPackageBase):
         assert params['notes'] in main_div, main_div_str
         license = model.Package.get_license_register()[params['license_id']]
         assert license.title in main_div, (license.title, main_div_str)
-        tag_names = [tag.lower() for tag in params['tags']]
+        tag_names = list(params['tags'])
         self.check_named_element(main_div, 'ul', *tag_names)
         if params.has_key('state'):
             assert 'State: %s' % params['state'] in main_div.replace('</strong>', ''), main_div_str
@@ -148,14 +163,14 @@ class TestPackageForm(TestPackageBase):
         self.check_tag(main_res, prefix+'version', params['version'])
         self.check_tag(main_res, prefix+'url', params['url'])
         #for res_index, res_field, expected_value in self._get_resource_values(params['resources']):
-        #    ## only check fields that are on the form 
+        #    ## only check fields that are on the form
         #    if res_field not in ['url', 'id', 'description', 'hash']:
         #        continue
         #    self.check_tag(main_res, '%sresources__%i__%s' % (prefix, res_index, res_field), expected_value)
         self.check_tag_and_data(main_res, prefix+'notes', params['notes'])
         self.check_tag_and_data(main_res, 'selected', params['license_id'])
         if isinstance(params['tags'], (str, unicode)):
-            tags = params['tags'].split()
+            tags = map(lambda s: s.strip(), params['tags'].split(','))
         else:
             tags = params['tags']
         for tag in tags:
@@ -179,7 +194,7 @@ class TestPackageForm(TestPackageBase):
                 self.check_tag(main_res, 'extras__%s__deleted' % num, 'checked')
 
         assert params['log_message'] in main_res, main_res
-    
+
     def _check_redirect(self, return_url_param, expected_redirect,
                         pkg_name_to_edit=''):
         '''
@@ -252,14 +267,14 @@ class TestReadOnly(TestPackageForm, HtmlCheckMethods, PylonsTestCase):
         pkg_by_name_main = self.named_div('dataset', res)
         pkg_by_id_main = self.named_div('dataset', res_by_id)
         # rename some things which may be in the wrong order sometimes
-        txt_order_non_deterministic = ('russian', 'tolstoy', 'david', 'roger')
+        txt_order_non_deterministic = (u'Flexible \u30a1', 'russian', 'tolstoy', 'david', 'roger')
         for txt in txt_order_non_deterministic:
             for pkg_ in (pkg_by_name_main, pkg_by_id_main):
                 pkg_ = pkg_.replace(txt, 'placeholder')
         print pkg_by_name_main
         res_diff = self.diff_html(pkg_by_name_main, pkg_by_id_main)
         assert not res_diff, res_diff.encode('utf8')
-        # not true as language selection link return url differs: 
+        # not true as language selection link return url differs:
         #assert len(res_by_id.body) == len(res.body)
 
         # only retrieve after app has been called
@@ -278,6 +293,7 @@ class TestReadOnly(TestPackageForm, HtmlCheckMethods, PylonsTestCase):
         self.check_tag_and_data(res, 'left arrow', '&lt;')
         self.check_tag_and_data(res, 'umlaut', u'\xfc')
         #assert 'OKD Compliant::' in res
+        assert u'Flexible \u30a1' in res, res
         assert 'russian' in res
         assert 'david' in res
         assert 'roger' in res
@@ -286,7 +302,7 @@ class TestReadOnly(TestPackageForm, HtmlCheckMethods, PylonsTestCase):
         assert 'romantic novel' in res, res
         assert 'original media' in res, res
         assert 'book' in res, res
-        assert 'This Dataset is Open' in res, res
+        assert 'This dataset satisfies the Open Definition' in res, res
 
     def test_read_war(self):
         name = u'warandpeace'
@@ -305,16 +321,18 @@ class TestReadOnly(TestPackageForm, HtmlCheckMethods, PylonsTestCase):
         CreateTestData.create_arbitrary([
             {'name':pkg_name,
              'notes':'Decoy link here: decoy:decoy, real links here: package:pkg-1, ' \
-                   'tag:tag_1 group:test-group-1.',
+                   'tag:tag_1 group:test-group-1 and a multi-word tag: tag:"multi word with punctuation."',
              }
             ])
         offset = url_for(controller='package', action='read', id=pkg_name)
         res = self.app.get(offset)
         def check_link(res, controller, id):
-            self.check_tag_and_data(res, 'a ', '/%s/%s' % (controller, id),
+            id_in_uri = id.strip('"').replace(' ', '%20') # remove quotes and percent-encode spaces
+            self.check_tag_and_data(res, 'a ', '/%s/%s' % (controller, id_in_uri),
                                     '%s:%s' % (controller, id))
         check_link(res, 'package', 'pkg-1')
         check_link(res, 'tag', 'tag_1')
+        check_link(res, 'tag', '"multi word with punctuation."')
         check_link(res, 'group', 'test-group-1')
         assert 'decoy</a>' not in res, res
         assert 'decoy"' not in res, res
@@ -328,7 +346,7 @@ class TestReadOnly(TestPackageForm, HtmlCheckMethods, PylonsTestCase):
         #assert '%s - Datasets' % title in res, res
         #main_div = self.main_div(res)
         #assert title in main_div, main_div.encode('utf8')
-    
+
     def test_history(self):
         name = 'annakarenina'
         offset = url_for(controller='package', action='history', id=name)
@@ -336,16 +354,37 @@ class TestReadOnly(TestPackageForm, HtmlCheckMethods, PylonsTestCase):
         assert 'History' in res
         assert 'Revisions' in res
         assert name in res
-        
+
     def test_read_plugin_hook(self):
         plugin = MockPackageControllerPlugin()
         plugins.load(plugin)
         name = u'annakarenina'
         offset = url_for(controller='package', action='read', id=name)
         res = self.app.get(offset)
-        assert plugin.calls['read'] == 1, plugin.calls
+
+        # There are now two reads of the package.  The first to find out
+        # the package's type.  And the second is the actual read that
+        # existed before.  I don't know if this is a problem?  I expect it
+        # can be fixed by allowing the package to be passed in to the plugin,
+        # either via the function argument, or adding it to the c object.
+        assert plugin.calls['read'] == 2, plugin.calls
         plugins.unload(plugin)
 
+    def test_resource_list(self):
+        name = 'annakarenina'
+        cache_url = 'http://thedatahub.org/test_cache_url.csv'
+        # add a cache_url to the first resource in the package
+        context = {'model': model, 'session': model.Session, 'user': 'testsysadmin'}
+        data = {'id': 'annakarenina'}
+        pkg = get.package_show(context, data)
+        pkg['resources'][0]['cache_url'] = cache_url
+        update.package_update(context, pkg)
+        # check that the cache url is included on the dataset view page
+        offset = url_for(controller='package', action='read', id=name)
+        res = self.app.get(offset)
+        assert '[cached]'in res
+        assert cache_url in res
+
 
 class TestReadAtRevision(FunctionalTestCase, HtmlCheckMethods):
 
@@ -357,7 +396,7 @@ class TestReadAtRevision(FunctionalTestCase, HtmlCheckMethods):
         cls.date3 = datetime.datetime(2011, 1, 3)
         cls.today = datetime.datetime.now()
         cls.pkg_name = u'testpkg'
-        
+
         # create dataset
         rev = model.repo.new_revision()
         rev.timestamp = cls.date1
@@ -371,7 +410,7 @@ class TestReadAtRevision(FunctionalTestCase, HtmlCheckMethods):
         rev.timestamp = cls.date2
         pkg = model.Package.by_name(cls.pkg_name)
         pkg.title = u'title2'
-        pkg.add_tag_by_name(u'tag2')
+        pkg.add_tag_by_name(u'tag 2')
         pkg.extras = {'key2': u'value2'}
         model.repo.commit_and_remove()
 
@@ -380,7 +419,7 @@ class TestReadAtRevision(FunctionalTestCase, HtmlCheckMethods):
         rev.timestamp = cls.date3
         pkg = model.Package.by_name(cls.pkg_name)
         pkg.title = u'title3'
-        pkg.add_tag_by_name(u'tag3')
+        pkg.add_tag_by_name(u'tag3.')
         pkg.extras['key2'] = u'value3'
         model.repo.commit_and_remove()
 
@@ -404,8 +443,8 @@ class TestReadAtRevision(FunctionalTestCase, HtmlCheckMethods):
         assert 'key2' in pkg_html
         assert 'value3' in pkg_html
         print 'SIDE', side_html
-        assert 'tag3' in side_html
-        assert 'tag2' in side_html
+        assert 'tag3.' in side_html
+        assert 'tag 2' in side_html
 
     def test_read_date1(self):
         offset = self.offset + self.date1.strftime('@%Y-%m-%d')
@@ -415,8 +454,8 @@ class TestReadAtRevision(FunctionalTestCase, HtmlCheckMethods):
         assert 'title1' in res, res
         assert 'key2' not in pkg_html, pkg_html
         assert 'value3' not in pkg_html, pkg_html
-        assert 'tag3' not in side_html, side_html
-        assert 'tag2' not in side_html, side_html
+        assert 'tag3.' not in side_html, side_html
+        assert 'tag 2' not in side_html, side_html
 
     def test_read_date2(self):
         date2_plus_3h = self.date2 + datetime.timedelta(hours=3)
@@ -429,8 +468,8 @@ class TestReadAtRevision(FunctionalTestCase, HtmlCheckMethods):
         assert 'key2' in pkg_html
         assert 'value2' in pkg_html
         print 'SIDE', side_html
-        assert 'tag3' not in side_html
-        assert 'tag2' in side_html
+        assert 'tag3.' not in side_html
+        assert 'tag 2' in side_html
 
     def test_read_date3(self):
         offset = self.offset + self.date3.strftime('@%Y-%m-%d-%H-%M')
@@ -442,8 +481,8 @@ class TestReadAtRevision(FunctionalTestCase, HtmlCheckMethods):
         assert 'key2' in pkg_html
         assert 'value3' in pkg_html
         print 'SIDE', side_html
-        assert 'tag3' in side_html
-        assert 'tag2' in side_html
+        assert 'tag3.' in side_html
+        assert 'tag 2' in side_html
 
     def test_read_date_before_created(self):
         offset = self.offset + self.before.strftime('@%Y-%m-%d')
@@ -465,15 +504,15 @@ class TestReadAtRevision(FunctionalTestCase, HtmlCheckMethods):
         side_html = self.named_div('sidebar', res)
         print 'MAIN', main_html
         assert 'This is an old revision of this dataset' in main_html
-        assert 'at 2011-01-01 00:00' in main_html
+        assert 'at Jan 01, 2011, 00:00' in main_html
         self.check_named_element(main_html, 'a', 'href="/dataset/%s"' % self.pkg_name)
         print 'PKG', pkg_html
         assert 'title1' in res
         assert 'key2' not in pkg_html
         assert 'value3' not in pkg_html
         print 'SIDE', side_html
-        assert 'tag3' not in side_html
-        assert 'tag2' not in side_html
+        assert 'tag3.' not in side_html
+        assert 'tag 2' not in side_html
 
     def test_read_revision2(self):
         offset = self.offset + '@%s' % self.revision_ids[1]
@@ -483,15 +522,15 @@ class TestReadAtRevision(FunctionalTestCase, HtmlCheckMethods):
         side_html = self.named_div('sidebar', res)
         print 'MAIN', main_html
         assert 'This is an old revision of this dataset' in main_html
-        assert 'at 2011-01-02 00:00' in main_html
+        assert 'at Jan 02, 2011, 00:00' in main_html
         self.check_named_element(main_html, 'a', 'href="/dataset/%s"' % self.pkg_name)
         print 'PKG', pkg_html
         assert 'title2' in res
         assert 'key2' in pkg_html
         assert 'value2' in pkg_html
         print 'SIDE', side_html
-        assert 'tag3' not in side_html
-        assert 'tag2' in side_html
+        assert 'tag3.' not in side_html
+        assert 'tag 2' in side_html
 
     def test_read_revision3(self):
         offset = self.offset + '@%s' % self.revision_ids[2]
@@ -502,24 +541,24 @@ class TestReadAtRevision(FunctionalTestCase, HtmlCheckMethods):
         print 'MAIN', main_html
         assert 'This is an old revision of this dataset' not in main_html
         assert 'This is the current revision of this dataset' in main_html
-        assert 'at 2011-01-03 00:00' in main_html
+        assert 'at Jan 03, 2011, 00:00' in main_html
         self.check_named_element(main_html, 'a', 'href="/dataset/%s"' % self.pkg_name)
         print 'PKG', pkg_html
         assert 'title3' in res
         assert 'key2' in pkg_html
         assert 'value3' in pkg_html
         print 'SIDE', side_html
-        assert 'tag3' in side_html
-        assert 'tag2' in side_html
+        assert 'tag3.' in side_html
+        assert 'tag 2' in side_html
 
     def test_read_bad_revision(self):
         # this revision doesn't exist in the db
         offset = self.offset + '@ccab6798-1f4b-4a22-bcf5-462703aa4594'
         res = self.app.get(offset, status=404)
-        
+
 class TestEdit(TestPackageForm):
     editpkg_name = u'editpkgtest'
-    
+
     @classmethod
     def setup_class(self):
         CreateTestData.create()
@@ -550,7 +589,7 @@ class TestEdit(TestPackageForm):
         self.offset = url_for(controller='package', action='edit', id=self.editpkg_name)
 
         self.editpkg = model.Package.by_name(self.editpkg_name)
-        self.admin = model.User.by_name(u'testadmin')        
+        self.admin = model.User.by_name(u'testadmin')
         self.res = None #get's refreshed by setup
         model.Session.remove()
 
@@ -610,7 +649,7 @@ class TestEdit(TestPackageForm):
             res = self.app.get(offset)
             assert '%s - Datasets' % new_title in res, res
             pkg = model.Package.by_name(new_name)
-            assert pkg.title == new_title 
+            assert pkg.title == new_title
             assert pkg.url == newurl
             #assert pkg.resources[0].url == new_download_url
             assert pkg.version == newversion
@@ -648,12 +687,12 @@ class TestEdit(TestPackageForm):
         prefix = 'Dataset-%s-' % self.pkgid
         fv = self.res.forms['dataset-edit']
         assert not fv.fields.has_key(prefix + 'groups')
-        
+
     def test_edit_2_tags_and_groups(self):
         # testing tag updating
-        newtagnames = [u'russian', u'tolstoy', u'superb']
+        newtagnames = [u'russian', u'tolstoy', u'superb book']
         newtags = newtagnames
-        tagvalues = ' '.join(newtags)
+        tagvalues = ','.join(newtags)
         fv = self.res.forms['dataset-edit']
         prefix = ''
         fv[prefix + 'tag_string'] = tagvalues
@@ -667,7 +706,7 @@ class TestEdit(TestPackageForm):
         assert len(pkg.tags) == len(newtagnames)
         outtags = [ tag.name for tag in pkg.tags ]
         for tag in newtags:
-            assert tag in outtags 
+            assert tag in outtags
         rev = model.Revision.youngest(model.Session)
         assert rev.author == 'Unknown IP Address'
         assert rev.message == exp_log_message
@@ -717,7 +756,7 @@ class TestEdit(TestPackageForm):
             pkg.notes= u'this is editpkg'
             pkg.version = u'2.2'
             t1 = model.Tag(name=u'one')
-            t2 = model.Tag(name=u'two')
+            t2 = model.Tag(name=u'two words')
             pkg.tags = [t1, t2]
             pkg.state = model.State.DELETED
             pkg.license_id = u'other-open'
@@ -752,8 +791,8 @@ class TestEdit(TestPackageForm):
             notes = u'Very important'
             license_id = u'gpl-3.0'
             state = model.State.ACTIVE
-            tags = (u'tag1', u'tag2', u'tag3')
-            tags_txt = u' '.join(tags)
+            tags = (u'tag1', u'tag2', u'tag 3')
+            tags_txt = u','.join(tags)
             extra_changed = 'key1', self.value1 + ' CHANGED', False
             extra_new = 'newkey', 'newvalue', False
             log_message = 'This is a comment'
@@ -855,7 +894,7 @@ class TestEdit(TestPackageForm):
         prefix = ''
         fv['log_message'] = u'Free enlargements: http://drugs.com/' # spam
         res = fv.submit('save')
-        assert 'Error' in res, res 
+        assert 'Error' in res, res
         self.check_tag(res, '<form', 'class="has-errors"')
         assert 'No links are allowed' in res, res
 
@@ -869,7 +908,7 @@ class TestEdit(TestPackageForm):
         # Ensure there is an error at the top of the form and by the field
         self._assert_form_errors(res)
 
-        
+
     def test_edit_plugin_hook(self):
         # just the absolute basics
         try:
@@ -893,7 +932,7 @@ class TestEdit(TestPackageForm):
         try:
             pkg = model.Package.by_name(u'editpkgtest')
             grp = model.Group.by_name(u'david')
-            assert len(pkg.groups) == 0
+            assert len(pkg.get_groups()) == 0
             offset = url_for(controller='package', action='edit', id=pkg.name)
             res = self.app.get(offset)
             prefix = ''
@@ -902,24 +941,24 @@ class TestEdit(TestPackageForm):
             # XXX the following assertion fails since upgrade to
             # sqlalchemy 0.6.5; apparently outer joins are handled
             # differently in such a way that
-            # ckan.lib.base._get_user_editable_groups (which calls 
+            # ckan.lib.base._get_user_editable_groups (which calls
             # ckan.authz.authorized_query) now returns groups when it
-            # shouldn't.                             
+            # shouldn't.
             assert not name in fv.fields.keys(), fv.fields.keys()
             res = fv.submit('save')
             res = res.follow()
             pkg = model.Package.by_name(u'editpkgtest')
-            assert len(pkg.groups) == 0
+            assert len(pkg.get_groups()) == 0
         finally:
-            self._reset_data()            
-    
+            self._reset_data()
+
     def test_edit_700_groups_add(self):
         try:
             pkg = model.Package.by_name(u'editpkgtest')
             grp = model.Group.by_name(u'david')
-            assert len(pkg.groups) == 0
+            assert len(pkg.get_groups()) == 0
             offset = url_for(controller='package', action='edit', id=pkg.name)
-            
+
             res = self.app.get(offset, extra_environ={'REMOTE_USER':'russianfan'})
             prefix = ''
             field_name = prefix + "groups__0__id"
@@ -929,21 +968,21 @@ class TestEdit(TestPackageForm):
             res = fv.submit('save', extra_environ={'REMOTE_USER':'russianfan'})
             res = res.follow()
             pkg = model.Package.by_name(u'editpkgtest')
-            assert len(pkg.groups) == 1, pkg.groups
+            assert len(pkg.get_groups()) == 1, pkg.get_groups()
             assert 'david' in res, res
         finally:
             self._reset_data()
-    
+
     def test_edit_700_groups_remove(self):
         try:
             pkg = model.Package.by_name(u'editpkgtest')
-            assert len(pkg.groups) == 0
+            assert len(pkg.get_groups()) == 0
             grp = model.Group.by_name(u'david')
             model.repo.new_revision()
-            model.Session.add(model.PackageGroup(package=pkg, group=grp))
+            model.Session.add(model.Member(table_id=pkg.id, table_name='package', group=grp))
             model.repo.commit_and_remove()
             pkg = model.Package.by_name(u'editpkgtest')
-            assert len(pkg.groups) == 1
+            assert len(pkg.get_groups()) == 1
             offset = url_for(controller='package', action='edit', id=pkg.name)
             res = self.app.get(offset, extra_environ={'REMOTE_USER':'russianfan'})
             prefix = ''
@@ -954,7 +993,7 @@ class TestEdit(TestPackageForm):
             res = fv.submit('save', extra_environ={'REMOTE_USER':'russianfan'})
             model.repo.commit_and_remove()
             pkg = model.Package.by_name(u'editpkgtest')
-            assert len(pkg.groups) == 0
+            assert len(pkg.get_groups()) == 0
         finally:
             self._reset_data()
 
@@ -964,9 +1003,9 @@ class TestEdit(TestPackageForm):
 
     def test_edit_indexerror(self):
         bad_solr_url = 'http://127.0.0.1/badsolrurl'
-        solr_url = search.common.solr_url
+        solr_url = SolrSettings.get()[0]
         try:
-            search.common.solr_url = bad_solr_url
+            SolrSettings.init(bad_solr_url)
             plugins.load('synchronous_search')
 
             fv = self.res.forms['dataset-edit']
@@ -976,8 +1015,35 @@ class TestEdit(TestPackageForm):
             assert 'Unable to update search index' in res, res
         finally:
             plugins.unload('synchronous_search')
-            search.common.solr_url = solr_url
+            SolrSettings.init(solr_url)
 
+    def test_edit_pkg_with_relationships(self):
+        # 1786
+        try:
+            # add a relationship to a package
+            pkg = model.Package.by_name(self.editpkg_name)
+            anna = model.Package.by_name(u'annakarenina')
+            model.repo.new_revision()
+            pkg.add_relationship(u'depends_on', anna)
+            model.repo.commit_and_remove()
+            
+            # check relationship before the test
+            rels = model.Package.by_name(self.editpkg_name).get_relationships()
+            assert_equal(str(rels), '[<*PackageRelationship editpkgtest depends_on annakarenina>]')
+
+            # edit the package
+            self.offset = url_for(controller='package', action='edit', id=self.editpkg_name)
+            self.res = self.app.get(self.offset)
+            fv = self.res.forms['dataset-edit']
+            fv['title'] = u'New Title'
+            res = fv.submit('save')
+
+            # check relationship still exists
+            rels = model.Package.by_name(self.editpkg_name).get_relationships()
+            assert_equal(str(rels), '[<*PackageRelationship editpkgtest depends_on annakarenina>]')
+            
+        finally:
+            self._reset_data()
 
 class TestNew(TestPackageForm):
     pkg_names = []
@@ -1088,8 +1154,8 @@ class TestNew(TestPackageForm):
         download_url = u'http://something.com/somewhere-else.zip'
         notes = u'Very important'
         license_id = u'gpl-3.0'
-        tags = (u'tag1', u'tag2', u'tag3', u'SomeCaps')
-        tags_txt = u' '.join(tags)
+        tags = (u'tag1', u'tag2.', u'tag 3', u'SomeCaps')
+        tags_txt = u','.join(tags)
         extras = {self.key1:self.value1, 'key2':'value2', 'key3':'value3'}
         log_message = 'This is a comment'
         assert not model.Package.by_name(name)
@@ -1130,10 +1196,9 @@ class TestNew(TestPackageForm):
         assert pkg.license.id == license_id
         saved_tagnames = [str(tag.name) for tag in pkg.tags]
         saved_tagnames.sort()
-        expected_tagnames = [tag.lower() for tag in tags]
-        expected_tagnames.sort()
+        expected_tagnames = sorted(tags)
         assert saved_tagnames == expected_tagnames, '%r != %r' % (saved_tagnames, expected_tagnames)
-        saved_groupnames = [str(group.name) for group in pkg.groups]
+        saved_groupnames = [str(group.name) for group in pkg.get_groups()]
         assert len(pkg.extras) == len(extras)
         for key, value in extras.items():
             assert pkg.extras[key] == value
@@ -1172,7 +1237,7 @@ class TestNew(TestPackageForm):
         assert 'Error' in res, res
         assert 'That URL is already in use.' in res, res
         self._assert_form_errors(res)
-        
+
     def test_missing_fields(self):
         # A field is left out in the commit parameters.
         # (Spammers can cause this)
@@ -1213,11 +1278,12 @@ class TestNew(TestPackageForm):
         assert plugin.calls['create'] == 1, plugin.calls
         plugins.unload(plugin)
 
+
     def test_new_indexerror(self):
         bad_solr_url = 'http://127.0.0.1/badsolrurl'
-        solr_url = search.common.solr_url
+        solr_url = SolrSettings.get()[0]
         try:
-            search.common.solr_url = bad_solr_url
+            SolrSettings.init(bad_solr_url)
             plugins.load('synchronous_search')
             new_package_name = u'new-package-missing-solr'
 
@@ -1234,7 +1300,41 @@ class TestNew(TestPackageForm):
             assert 'Unable to add package to search index' in res, res
         finally:
             plugins.unload('synchronous_search')
-            search.common.solr_url = solr_url
+            SolrSettings.init(solr_url)
+
+    def test_change_locale(self):
+        offset = url_for(controller='package', action='new')
+        res = self.app.get(offset)
+        res = res.click('Deutsch')
+        try:
+            res = res.follow()
+            assert 'Datensatz' in res.body, res.body
+
+
+        finally:
+            self.clear_language_setting()
+        
+class TestSearch(TestPackageForm):
+    pkg_names = []
+
+    @classmethod
+    def setup_class(self):
+        model.repo.init_db()
+
+    @classmethod
+    def teardown_class(self):
+        self.purge_packages(self.pkg_names)
+        model.repo.rebuild_db()
+
+    def test_search_plugin_hooks(self):
+        plugin = MockPackageControllerPlugin()
+        plugins.load(plugin)
+        offset = url_for(controller='package', action='search')
+        res = self.app.get(offset)
+        # get redirected ...
+        assert plugin.calls['before_search'] == 1, plugin.calls
+        assert plugin.calls['after_search'] == 1, plugin.calls
+        plugins.unload(plugin)
 
 class TestNewPreview(TestPackageBase):
     pkgname = u'testpkg'
@@ -1266,11 +1366,11 @@ class TestNonActivePackages(TestPackageBase):
         model.setup_default_user_roles(pkg, [admin])
         model.repo.commit_and_remove()
 
-        model.repo.new_revision()        
+        model.repo.new_revision()
         pkg = model.Session.query(model.Package).filter_by(name=self.non_active_name).one()
         pkg.delete() # becomes non active
         model.repo.commit_and_remove()
-        
+
 
     @classmethod
     def teardown_class(self):
@@ -1309,7 +1409,7 @@ class TestRevisions(TestPackageBase):
             pkg1.notes = cls.notes[i]
             model.repo.commit_and_remove()
 
-        cls.pkg1 = model.Package.by_name(cls.name)        
+        cls.pkg1 = model.Package.by_name(cls.name)
         cls.revision_ids = [rev[0].id for rev in cls.pkg1.all_related_revisions]
                            # revision ids are newest first
         cls.revision_timestamps = [rev[0].timestamp for rev in cls.pkg1.all_related_revisions]
@@ -1318,7 +1418,7 @@ class TestRevisions(TestPackageBase):
     @classmethod
     def teardown_class(cls):
         model.repo.rebuild_db()
-    
+
     def test_0_read_history(self):
         res = self.app.get(self.offset)
         main_res = self.main_div(res)
@@ -1361,9 +1461,8 @@ class TestRevisions(TestPackageBase):
         res = res.click(href=url)
         main_html = self.main_div(res)
         assert 'This is an old revision of this dataset' in main_html
-        assert 'at %s' % str(self.revision_timestamps[1])[:6] in main_html
 
-   
+
 class TestMarkdownHtmlWhitelist(TestPackageForm):
 
     pkg_name = u'markdownhtmlwhitelisttest'
@@ -1436,7 +1535,7 @@ class TestAutocomplete(PylonsTestCase, TestPackageBase):
     def test_package_autocomplete(self):
         query = 'a'
         res = self.app.get('/dataset/autocomplete?q=%s' % query)
-        
+
         expected = ['A Wonderful Story (warandpeace)|warandpeace','annakarenina|annakarenina']
         received = sorted(res.body.split('\n'))
         assert expected == received
