@@ -48,7 +48,7 @@ class TestUserController(FunctionalTestCase, HtmlCheckMethods, PylonsTestCase, S
         main_res = self.main_div(res)
         assert 'annafan' in res, res
         assert 'Logged in' not in main_res, main_res
-        assert 'My Account' not in main_res, main_res
+        assert 'checkpoint:is-myself' not in main_res, main_res
         assert 'about' in main_res, main_res
         assert 'I love reading Annakarenina' in res, main_res
         self.check_named_element(res, 'a',
@@ -56,9 +56,6 @@ class TestUserController(FunctionalTestCase, HtmlCheckMethods, PylonsTestCase, S
                                  'target="_blank"',
                                  'rel="nofollow"')
         assert 'Edit Profile' not in main_res, main_res
-        assert 'Number of edits:</strong> 3' in res, res
-        assert 'Number of datasets administered:</strong> 1' in res, res
-        assert 'Revision History' in res, res
 
     def test_user_read_without_id(self):
         offset = '/user/'
@@ -70,11 +67,11 @@ class TestUserController(FunctionalTestCase, HtmlCheckMethods, PylonsTestCase, S
 
     def test_user_read_without_id_but_logged_in(self):
         user = model.User.by_name(u'annafan')
-        offset = '/user/'
+        offset = '/user/annafan'
         res = self.app.get(offset, status=200, extra_environ={'REMOTE_USER': str(user.name)})
         main_res = self.main_div(res)
         assert 'annafan' in main_res, main_res
-        assert 'My Account' in main_res, main_res
+        assert 'checkpoint:is-myself' in main_res, main_res
 
     def test_user_read_logged_in(self):
         user = model.User.by_name(u'annafan')
@@ -82,7 +79,7 @@ class TestUserController(FunctionalTestCase, HtmlCheckMethods, PylonsTestCase, S
         res = self.app.get(offset, extra_environ={'REMOTE_USER': str(user.name)})
         main_res = self.main_div(res)
         assert 'annafan' in res, res
-        assert 'My Account' in main_res, main_res
+        assert 'checkpoint:is-myself' in main_res, main_res
         assert 'Edit Profile' in main_res, main_res
 
     def test_user_read_about_unfinished(self):
@@ -170,16 +167,18 @@ class TestUserController(FunctionalTestCase, HtmlCheckMethods, PylonsTestCase, S
 
         # first get redirected to user/logged_in
         assert_equal(res.status, 302)
-        assert res.header('Location').startswith('http://localhost/user/logged_in')
+        assert res.header('Location').startswith('http://localhost/user/logged_in') or \
+               res.header('Location').startswith('/user/logged_in')
 
         # then get redirected to user page
         res = res.follow()
         assert_equal(res.status, 302)
-        assert_equal(res.header('Location'), 'http://localhost/user/testlogin')
+        assert res.header('Location') in ('http://localhost/user/testlogin',
+                                          '/user/testlogin')
         res = res.follow()
         assert_equal(res.status, 200)
         assert 'testlogin is now logged in' in res.body
-        assert 'My Account' in res.body
+        assert 'checkpoint:is-myself' in res.body
 
         # check user object created
         user = model.User.by_name(username)
@@ -195,6 +194,10 @@ class TestUserController(FunctionalTestCase, HtmlCheckMethods, PylonsTestCase, S
         assert 'auth_tkt=' in cookie, cookie
         assert 'testlogin!userid_type:unicode' in cookie, cookie
 
+        # navigate to another page and check username still displayed
+        res = res.click('Search')
+        assert 'testlogin' in res.body, res.body
+
     def test_login_wrong_password(self):
         # create test user
         username = u'testloginwrong'
@@ -213,33 +216,153 @@ class TestUserController(FunctionalTestCase, HtmlCheckMethods, PylonsTestCase, S
 
         # first get redirected to logged_in
         assert_equal(res.status, 302)
-        assert res.header('Location').startswith('http://localhost/user/logged_in')
+        assert res.header('Location').startswith('http://localhost/user/logged_in') or \
+               res.header('Location').startswith('/user/logged_in')
 
         # then get redirected to login
         res = res.follow()
         assert_equal(res.status, 302)
-        assert res.header('Location').startswith('http://localhost/user/login')
+        assert res.header('Location').startswith('http://localhost/user/login') or \
+               res.header('Location').startswith('/user/login')
         res = res.follow()
         assert_equal(res.status, 200)
         assert 'Login failed. Bad username or password.' in res.body
         assert 'Login:' in res.body
 
+    def test_relogin(self):
+        '''Login as user A and then (try to) login as user B (without
+        logout). #1799.'''
+        # create test users A & B
+        password = u'letmein'
+        CreateTestData.create_user(name=u'user_a',
+                                   password=password)
+        CreateTestData.create_user(name=u'user_b',
+                                   password=password)
+        userA = model.User.by_name(u'user_a')
+        userB = model.User.by_name(u'user_b')
 
-    # -----------
-    # tests for top links present in every page
-     # TODO: test sign in results in:
-     # a) a username at top of page
-     # b) logout link
+        # do the login
+        offset = url_for(controller='user', action='login')
+        res = self.app.get(offset)
+        fv = res.forms['login']
+        fv['login'] = 'user_a'
+        fv['password'] = str(password)
+        res = fv.submit()
+        while res.status == 302:
+            res = res.follow()
+        assert_equal(res.status, 200)
+
+        # login as userB
+        offset = url_for(controller='user', action='login')
+        res = self.app.get(offset)
+        assert not res.forms.has_key('login') # i.e. no login box is presented
+        assert 'To register or log in as another user' in res.body, res.body
+        assert 'logout' in res.body, res.body
+
+        # Test code left commented - shows the problem if you
+        # let people try to login whilst still logged in. #1799
+##        fv['login'] = 'user_b'
+##        fv['password'] = str(password)
+##        res = fv.submit()
+##        while res.status == 302:
+##            res = res.follow()
+##        assert_equal(res.status, 200)
+
+##        offset = url_for(controller='user', action='me')
+##        res = self.app.get(offset)
+##        assert_equal(res.status, 302)
+##        res = res.follow()
+##        assert 'user_b' in res
+
+    def test_try_to_register_whilst_logged_in(self):
+        '''Login as user A and then (try to) register user B (without
+        logout). #1799.'''
+        # create user A
+        password = u'letmein'
+        CreateTestData.create_user(name=u'user_a_',
+                                   password=password)
+        userA = model.User.by_name(u'user_a_')
+
+        # do the login
+        offset = url_for(controller='user', action='login')
+        res = self.app.get(offset)
+        fv = res.forms['login']
+        fv['login'] = 'user_a_'
+        fv['password'] = str(password)
+        res = fv.submit()
+        while res.status == 302:
+            res = res.follow()
+        assert_equal(res.status, 200)
+
+        # try to register
+        offset = url_for(controller='user', action='register')
+        res = self.app.get(offset)
+        assert not res.forms.has_key('Password') # i.e. no registration form
+        assert 'To register or log in as another user' in res.body, res.body
+        assert 'logout' in res.body, res.body
+
+    def test_register_whilst_logged_in(self):
+        '''Start registration form as user B then in another window login
+        as user A, and then try and then submit form for user B. #1799.'''
+        # create user A
+        password = u'letmein'
+        CreateTestData.create_user(name=u'user_a__',
+                                   password=password)
+        userA = model.User.by_name(u'user_a__')
+        # make him a sysadmin, to ensure he is allowed to create a user
+        model.add_user_to_role(userA, model.Role.ADMIN, model.System())
+        model.repo.commit_and_remove()
+        userA = model.User.by_name(u'user_a__')
+
+        # start to register user B
+        offset = url_for(controller='user', action='register')
+        res = self.app.get(offset)
+        fvA = res.forms['user-edit']
+        fvA['name'] = 'user_b_'
+        fvA['fullname'] = 'User B'
+        fvA['email'] = 'user@b.com'
+        fvA['password1'] = password
+        fvA['password2'] = password
+
+        # login user A
+        offset = url_for(controller='user', action='login')
+        res = self.app.get(offset)
+        fvB = res.forms['login']
+        fvB['login'] = 'user_a__'
+        fvB['password'] = str(password)
+        res = fvB.submit()
+        while res.status == 302:
+            res = res.follow()
+        assert_equal(res.status, 200)
+
+        # finish registration of user B
+        res = fvA.submit('save')
+        assert_equal(res.status, 200)
+        assert 'user_a__</a> is currently logged in' in res.body, res.body
+        assert 'User "user_b_" is now registered but you are still logged in as "user_a__" from before'.replace('"', '&#34;') in res.body, res.body
+        assert 'logout' in res.body, res.body
+
+        # logout and login as user B
+        res = self.app.get('/user/logout')
+        res2 = res.follow()
+        assert 'You have logged out successfully.' in res2, res2
+        offset = url_for(controller='user', action='login')
+        res = self.app.get(offset)
+        fv = res.forms['login']
+        fv['login'] = 'user_b_'
+        fv['password'] = str(password)
+        res = fv.submit()
+        while res.status == 302:
+            res = res.follow()
+        assert_equal(res.status, 200)
+        assert 'User B is now logged in' in res.body, res.body
 
     @search_related
     def test_home_login(self):
         offset = url_for('home')
         res = self.app.get(offset)
-        # cannot use click because it does not allow a 401 response ...
-        # could get round this by checking that url is correct and then doing a
-        # get but then we are back to test_user_login
-        res.click('Login')
-        # assert 'Please Sign In' in res
+        res = res.click('Login')
+        assert 'Login to CKAN' in res, res.body
 
     def test_apikey(self):
         username= u'okfntest'
@@ -257,7 +380,7 @@ class TestUserController(FunctionalTestCase, HtmlCheckMethods, PylonsTestCase, S
 
         offset = url_for(controller='user', action='read', id='okfntest')
         res = self.app.get(offset, extra_environ={'REMOTE_USER': 'okfntest'})
-        assert 'Your API key is: %s' % user.apikey in res, res
+        assert user.apikey in res, res
 
     def test_user_create(self):
         # create/register user
