@@ -10,17 +10,20 @@ from ckan.lib.dictization import (table_dictize,
 from ckan.lib.dictization.model_dictize import (package_dictize,
                                                 resource_dictize,
                                                 group_dictize,
+                                                activity_dictize,
                                                 package_to_api1,
                                                 package_to_api2,
                                                )
 from ckan.lib.dictization.model_save import (package_dict_save,
                                              resource_dict_save,
                                              group_dict_save,
+                                             activity_dict_save,
                                              package_api_to_dict,
                                              group_api_to_dict,
                                              package_tag_list_save,
                                             )
 from ckan.logic.action.update import make_latest_pending_package_active
+import ckan.logic.action.get
 
 class TestBasicDictize:
     @classmethod
@@ -37,16 +40,19 @@ class TestBasicDictize:
                {'key': u'original media', 'state': u'active', 'value': u'"book"'}],
             'groups': [{'description': u'These are books that David likes.',
                         'name': u'david',
+                        'type': u'group',
                         'state': u'active',
                         'title': u"Dave's books"},
                        {'description': u'Roger likes these books.',
                         'name': u'roger',
+                        'type': u'group',
                         'state': u'active',
                         'title': u"Roger's books"}],
             'isopen': True,
             'license_id': u'other-open',
             'maintainer': None,
             'maintainer_email': None,
+            'type': None,
             'name': u'annakarenina',
             'notes': u'Some test notes\n\n### A 3rd level heading\n\n**Some bolded text.**\n\n*Some italicized text.*\n\nForeign characters:\nu with umlaut \xfc\n66-style quote \u201c\nforeign word: th\xfcmb\n \nNeeds escaping:\nleft arrow <\n\n<http://ckan.net/>\n\n',
             'relationships_as_object': [],
@@ -88,7 +94,8 @@ class TestBasicDictize:
                             u'webstore_last_updated': None,
                             u'webstore_url': None}],
             'state': u'active',
-            'tags': [{'name': u'russian', 'state': u'active'},
+            'tags': [{'name': u'Flexible \u30a1', 'state': u'active'},
+                     {'name': u'russian', 'state': u'active'},
                      {'name': u'tolstoy', 'state': u'active'}],
             'title': u'A Novel By Tolstoy',
             'url': u'http://www.annakarenina.com',
@@ -100,11 +107,6 @@ class TestBasicDictize:
         model.repo.rebuild_db()
         model.Session.remove()
 
-    def teardonwn(self):
-        model.Session.remove()
-
-
-
     def remove_changable_columns(self, dict):
         for key, value in dict.items():
             if key.endswith('id') and key <> 'license_id':
@@ -113,6 +115,8 @@ class TestBasicDictize:
                 dict.pop(key)
             if 'timestamp' in key:
                 dict.pop(key)
+            if key in ['metadata_created','metadata_modified']:
+                dict.pop(key)
             if isinstance(value, list):
                 for new_dict in value:
                     self.remove_changable_columns(new_dict)
@@ -145,6 +149,7 @@ class TestBasicDictize:
             'maintainer': None,
             'maintainer_email': None,
             'name': u'annakarenina',
+            'type': None,
             'notes': u'Some test notes\n\n### A 3rd level heading\n\n**Some bolded text.**\n\n*Some italicized text.*\n\nForeign characters:\nu with umlaut \xfc\n66-style quote \u201c\nforeign word: th\xfcmb\n \nNeeds escaping:\nleft arrow <\n\n<http://ckan.net/>\n\n',
             'state': u'active',
             'title': u'A Novel By Tolstoy',
@@ -367,8 +372,8 @@ class TestBasicDictize:
         print anna_original
         print anna_after_save
 
-        assert self.remove_revision_id(anna_dictized) == self.remove_revision_id(package_dictized),\
-                "\n".join(unified_diff(anna_original.split("\n"), anna_after_save.split("\n")))
+        assert self.remove_changable_columns(anna_dictized) == self.remove_changable_columns(package_dictized)
+        assert "\n".join(unified_diff(anna_original.split("\n"), anna_after_save.split("\n")))
 
     def test_10_package_alter_pending(self):
 
@@ -430,17 +435,21 @@ class TestBasicDictize:
 
         print [(tag.state, tag.tag.name) for tag in sorted_tags]
 
-        assert len(sorted_tags) == 4, len(sorted_tags)
-        assert sorted_tags[0].state == 'pending-deleted'
-        assert sorted_tags[1].state == 'pending'
-        assert sorted_tags[2].state == 'active'
-        assert sorted_resources[2].current
-        assert sorted_tags[3].state == 'active'
+        assert len(sorted_tags) == 5, len(sorted_tags)
+        assert sorted_tags[0].state == 'pending'            # new_tag
+        assert sorted_tags[1].state == 'pending-deleted'    # Flexible
+        assert sorted_tags[2].state == 'active'             # tolstoy
+        assert sorted_tags[3].state == 'active'             # russian
+        assert sorted_tags[4].state == 'active'             # Flexible
+        assert sorted_tags[2].current
+        assert sorted_tags[3].current
+        assert sorted_tags[4].current
 
         assert str(sorted_tags[0].expired_timestamp) == '9999-12-31 00:00:00'
         assert str(sorted_tags[1].expired_timestamp) == '9999-12-31 00:00:00'
         assert str(sorted_tags[2].expired_timestamp) == '9999-12-31 00:00:00'
-        assert str(sorted_tags[3].expired_timestamp) != '9999-12-31 00:00:00'
+        assert str(sorted_tags[3].expired_timestamp) == '9999-12-31 00:00:00'
+        assert str(sorted_tags[4].expired_timestamp) != '9999-12-31 00:00:00'
 
         extras_revisions = model.Session.query(model.PackageExtraRevision).filter_by(package_id=anna1.id).all()
 
@@ -512,20 +521,23 @@ class TestBasicDictize:
 
         print [(tag.state, tag.tag.name) for tag in sorted_tags]
 
-        assert len(sorted_tags) == 5, len(sorted_tags)
-        assert sorted_tags[0].state == 'pending'
-        assert sorted_tags[1].state == 'pending-deleted'
-        assert sorted_tags[2].state == 'pending'
-        assert sorted_tags[3].state == 'active'
+        assert len(sorted_tags) == 6, len(sorted_tags)
+        assert sorted_tags[0].state == 'pending'            # newnew_tag
+        assert sorted_tags[1].state == 'pending'            # new_tag
+        assert sorted_tags[2].state == 'pending-deleted'    # Flexible
+        assert sorted_tags[3].state == 'active'             # tolstoy
+        assert sorted_tags[4].state == 'active'             # russian
+        assert sorted_tags[5].state == 'active'             # Flexible
         assert sorted_tags[3].current
-        assert sorted_tags[4].state == 'active'
         assert sorted_tags[4].current
+        assert sorted_tags[5].current
 
         assert str(sorted_tags[0].expired_timestamp) == '9999-12-31 00:00:00'
         assert str(sorted_tags[1].expired_timestamp) == '9999-12-31 00:00:00'
         assert str(sorted_tags[2].expired_timestamp) == '9999-12-31 00:00:00'
         assert str(sorted_tags[3].expired_timestamp) == '9999-12-31 00:00:00'
-        assert str(sorted_tags[4].expired_timestamp) != '9999-12-31 00:00:00'
+        assert str(sorted_tags[4].expired_timestamp) == '9999-12-31 00:00:00'
+        assert str(sorted_tags[5].expired_timestamp) != '9999-12-31 00:00:00'
 
         extras_revisions = model.Session.query(model.PackageExtraRevision).filter_by(package_id=anna1.id).all()
 
@@ -591,22 +603,25 @@ class TestBasicDictize:
         sorted_tags = sorted(tag_revisions, key=lambda x: (x.revision_timestamp, x.tag.name))[::-1]
 
         print [(tag.state, tag.tag.name) for tag in sorted_tags]
-
-        assert len(sorted_tags) == 5, len(sorted_tags)
-        assert sorted_tags[0].state == 'active'
+        
+        assert len(sorted_tags) == 6, len(sorted_tags)
+        assert sorted_tags[0].state == 'active'     # newnew_tag
+        assert sorted_tags[1].state == 'active'     # new_tag
+        assert sorted_tags[2].state == 'deleted'    # Flexible
+        assert sorted_tags[3].state == 'active'     # tolstoy
+        assert sorted_tags[4].state == 'active'     # russian
+        assert sorted_tags[5].state == 'active'     # Flexible
         assert sorted_tags[0].current
-        assert sorted_tags[1].state == 'deleted'
         assert sorted_tags[1].current
-        assert sorted_tags[2].state == 'active'
         assert sorted_tags[2].current
-        assert sorted_tags[3].state == 'active'
-        assert sorted_tags[4].state == 'active'
+        assert not sorted_tags[5].current
 
         assert str(sorted_tags[0].expired_timestamp) == '9999-12-31 00:00:00'
         assert str(sorted_tags[1].expired_timestamp) == '9999-12-31 00:00:00'
         assert str(sorted_tags[2].expired_timestamp) == '9999-12-31 00:00:00'
         assert str(sorted_tags[3].expired_timestamp) == '9999-12-31 00:00:00'
-        assert str(sorted_tags[4].expired_timestamp) != '9999-12-31 00:00:00'
+        assert str(sorted_tags[4].expired_timestamp) == '9999-12-31 00:00:00'
+        assert str(sorted_tags[5].expired_timestamp) != '9999-12-31 00:00:00'
 
         extras_revisions = model.Session.query(model.PackageExtraRevision).filter_by(package_id=anna1.id).all()
 
@@ -805,26 +820,62 @@ class TestBasicDictize:
 
         pkg = model.Session.query(model.Package).filter_by(name='annakarenina3').first()
 
+        simple_group_dict = {'name': 'simple',
+                             'title': 'simple',
+                             'type': 'publisher',
+                            }
+        model.repo.new_revision()
+        group_dict_save(simple_group_dict, context)
+        model.Session.commit()
+        model.Session.remove()
+
+        context = {"model": model,
+                  "session": model.Session}
+
         group_dict = {'name': 'help',
                       'title': 'help',
                       'extras': [{'key': 'genre', 'value': u'"horror"'},
                                  {'key': 'media', 'value': u'"dvd"'}],
-                      'packages':[{'name': 'annakarenina2'}, {'id': pkg.id}]
+                      'packages':[{'name': 'annakarenina2'}, {'id': pkg.id, 'capacity': 'in'}],
+                      'users':[{'name': 'annafan'}],
+                      'groups':[{'name': 'simple'}],
+                      'tags':[{'name': 'russian'}]
                       }
 
-
         model.repo.new_revision()
         group_dict_save(group_dict, context)
         model.Session.commit()
         model.Session.remove()
-
+        
         group = model.Session.query(model.Group).filter_by(name=u'help').one()
 
+        context = {"model": model,
+                  "session": model.Session}
+
         group_dictized = group_dictize(group, context)
 
         expected =  {'description': u'',
                     'extras': [{'key': u'genre', 'state': u'active', 'value': u'"horror"'},
                                {'key': u'media', 'state': u'active', 'value': u'"dvd"'}],
+                    'tags': [{'capacity': 'member', 'name': u'russian'}],
+                    'groups': [{'description': u'',
+                               'capacity' : 'member',
+                               'display_name': u'simple',
+                               'name': u'simple',
+                               'packages': 0,
+                               'state': u'active',
+                               'title': u'simple',
+                               'type': u'publisher'}],
+                    'users': [{'about': u'I love reading Annakarenina. My site: <a href="http://anna.com">anna.com</a>',
+                              'display_name': u'annafan',
+                              'capacity' : 'member',
+                              'email': None,
+                              'email_hash': 'd41d8cd98f00b204e9800998ecf8427e',
+                              'fullname': None,
+                              'name': u'annafan',
+                              'number_administered_packages': 1L,
+                              'number_of_edits': 0L,
+                              'reset_key': None}],
                     'name': u'help',
                     'display_name': u'help',
                     'packages': [{'author': None,
@@ -832,17 +883,22 @@ class TestBasicDictize:
                                   'license_id': u'other-open',
                                   'maintainer': None,
                                   'maintainer_email': None,
+                                  'type': None,
                                   'name': u'annakarenina3',
                                   'notes': u'Some test notes\n\n### A 3rd level heading\n\n**Some bolded text.**\n\n*Some italicized text.*\n\nForeign characters:\nu with umlaut \xfc\n66-style quote \u201c\nforeign word: th\xfcmb\n \nNeeds escaping:\nleft arrow <\n\n<http://ckan.net/>\n\n',
                                   'state': u'active',
+                                  'capacity' : 'in',
                                   'title': u'A Novel By Tolstoy',
                                   'url': u'http://www.annakarenina.com',
                                   'version': u'0.7a'},
                                  {'author': None,
                                   'author_email': None,
+                                  'capacity' : 'member',
+                                  'title': u'A Novel By Tolstoy',
                                   'license_id': u'other-open',
                                   'maintainer': None,
                                   'maintainer_email': None,
+                                  'type': None,
                                   'name': u'annakarenina2',
                                   'notes': u'Some test notes\n\n### A 3rd level heading\n\n**Some bolded text.**\n\n*Some italicized text.*\n\nForeign characters:\nu with umlaut \xfc\n66-style quote \u201c\nforeign word: th\xfcmb\n \nNeeds escaping:\nleft arrow <\n\n<http://ckan.net/>\n\n',
                                   'state': u'active',
@@ -850,7 +906,8 @@ class TestBasicDictize:
                                   'url': u'http://www.annakarenina.com',
                                   'version': u'0.7a'}],
                     'state': u'active',
-                    'title': u'help'}
+                    'title': u'help',
+                    'type': u'group'}
 
         expected['packages'] = sorted(expected['packages'], key=lambda x: x['name'])
 
@@ -910,3 +967,35 @@ class TestBasicDictize:
 
         pkg = model.Package.by_name(name)
         assert_equal(set([tag.name for tag in pkg.tags]), set(('tag1',)))
+
+    def test_20_activity_save(self):
+
+        # Add a new Activity object to the database by passing a dict to
+        # activity_dict_save()
+        context = {"model": model, "session": model.Session}
+        user = model.User.by_name(u'tester')
+        revision = model.repo.new_revision()
+        sent = {
+                'user_id': user.id,
+                'object_id': user.id,
+                'revision_id': revision.id,
+                'activity_type': 'changed user'
+                }
+        activity_dict_save(sent, context)
+        model.Session.commit()
+
+        # Retrieve the newest Activity object from the database, check that its
+        # attributes match those of the dict we saved.
+        got = ckan.logic.action.get.user_activity_list(context,
+                {'id': user.id})[0]
+        assert got['user_id'] == sent['user_id']
+        assert got['object_id'] == sent['object_id']
+        assert got['revision_id'] == sent['revision_id']
+        assert got['activity_type'] == sent['activity_type']
+
+        # The activity object should also have an ID and timestamp.
+        assert got['id']
+        assert got['timestamp']
+
+        # We didn't pass in any data so this should be empty.
+        assert not got['data']
