@@ -204,9 +204,9 @@ class TestPackageWithTags:
     def setup_class(self):
         model.repo.init_db()
         rev1 = model.repo.new_revision()
-        self.tagname = u'testtagm2m'
+        self.tagname = u'test tag m2m!'
         self.tagname2 = u'testtagm2m2'
-        self.tagname3 = u'testtag3'
+        self.tagname3 = u'test tag3!'
         self.pkgname = u'testpkgm2m'
         pkg = model.Package(name=self.pkgname)
         self.tag = model.Tag(name=self.tagname)
@@ -295,7 +295,7 @@ class TestPackageTagSearch:
     def test_0_deleted_package_tags(self):
         pkg = model.Package.by_name(u'annakarenina')
         tag = model.Tag.by_name(self.tagname)
-        assert len(pkg.tags) == 3
+        assert len(pkg.tags) == 4
         assert len(tag.packages) == 0
 
     def test_1_tag_search_1(self):
