@@ -177,6 +177,9 @@ ckan_create_wsgi_handler () {
             echo "Attempting to install 'pip' 1.0 from pypi.python.org into pyenv to be used for extensions ..."
             sudo -u ckan${INSTANCE} /var/lib/ckan/${INSTANCE}/pyenv/bin/easy_install --upgrade "pip>=1.0" "pip<=1.0.99"
             echo "done."
+            echo "Attempting to install 'paster' pypi.python.org into pyenv ..."
+            sudo -u ckan${INSTANCE} /var/lib/ckan/${INSTANCE}/pyenv/bin/easy_install --ignore-installed "pastescript"
+            echo "done."
             cat <<- EOF > /var/lib/ckan/${INSTANCE}/packaging_version.txt
 	1.5
 	EOF
